{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit Automate;
{$B-,R-}
interface

uses
  ComObj, ActiveX, AxCtrls, Classes, DM_TLB, StdVcl;

type

  TDMNotes = class(TAutoObject, IDMNotes)
  protected
    { Protected declarations }
    function Get_LineCount: Integer; safecall;
    function Get_Lines(Index: Integer): WideString; safecall;
    procedure AddLine(const Line: WideString); safecall;
    procedure Clear; safecall;
    procedure DeleteLine(Position: Integer); safecall;
    procedure InsertLine(Position: Integer; const Line: WideString); safecall;
    procedure Load(const FileName: WideString); safecall;
    procedure Save(const FileName: WideString); safecall;
    procedure Set_Lines(Index: Integer; const Value: WideString); safecall;
    function Get_Text: WideString; safecall;
    procedure Set_Text(const Value: WideString); safecall;
  end;

  TDMInternalApp = class(TAutoObject, IDMApplication)
  private
    { Private declarations }
    FNotes: TDMNotes;
  protected
    { Protected declarations }
    function Get_ActiveDocument: IDispatch; safecall;
    function Get_DocumentCount: Integer; safecall;
    function Get_Documents(Index: Integer): IDispatch; safecall;
    procedure DisplayMessage(const Msg: WideString); safecall;
    procedure RecordPoint(Labels, Values: OleVariant); safecall;
    procedure ExecuteCommand(const Name: WideString); safecall;
    function Get_IsCommandChecked(const Name: WideString): WordBool; safecall;
    function Get_IsCommandEnabled(const Name: WideString): WordBool; safecall;
    function Get_IsCommandVisible(const Name: WideString): WordBool; safecall;
    procedure Set_IsCommandChecked(const Name: WideString; Value: WordBool);
      safecall;
    procedure Set_IsCommandEnabled(const Name: WideString; Value: WordBool);
      safecall;
    procedure Set_IsCommandVisible(const Name: WideString; Value: WordBool);
      safecall;
    procedure SetExpSettings(const Settings: WideString); safecall;
    function CreateDocument(const FileName: WideString): IDispatch; safecall;
    procedure ShowProgress(Progress: Integer); safecall;
    procedure DisplayHint(const Hint: WideString); safecall;
    function Get_Notes: IDispatch; safecall;
{    procedure LMFit(X,Y,W: OleVariant; WType: WeightType;
      const Expression: WideString; var Params: OleVariant;
      var Sigma: OleVariant; var NIter: Integer; out ChiSqr: Double;
      out ResultCode: FitResult); safecall;}
    procedure LMFit(X,Y,W: OleVariant; WType: WeightType;
      Expression: OleVariant; var Params: OleVariant;
      var Sigma: OleVariant; var NIter: OleVariant; out ChiSqr: OleVariant;
      out ResultCode: OleVariant); safecall;
    procedure LinearFit(X: OleVariant; Y: OleVariant; NumTerms: Integer;
      BasisType: LinearBasisType; out Solution: OleVariant;
      out Deviation: OleVariant; out ResultCode:OleVariant); safecall;
{   WARNING!!! if we would use Deviation: Double; out ResultCode: FitResult,
    script engine will generate strange "type incompatible" errors!!! }
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
  end;

  TDMDocument=class;

  TDMExternalApp = class(TDMInternalApp, IConnectionPointContainer)
  private
    { Private declarations }
    FConnectionPoints: TConnectionPoints;
    FConnectionPoint: TConnectionPoint;
    FSinkList: TList;
    FEvents: IDMApplicationEvents;
    procedure CreateDoc(Doc: TDMDocument);  // these two called by TDMDocument
    procedure CloseDoc(Doc: TDMDocument);
  public
    { Public declarations }
    procedure Initialize; override;
    destructor Destroy; override;
    procedure Command(CMD: string);                    // fire OnCommand event
  protected
    { Protected declarations }
    property ConnectionPoints: TConnectionPoints read FConnectionPoints
      implements IConnectionPointContainer;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
  end;

  TDMWorksheet=class;
  TDMContainer=class;
  TDMPlot=class;

  TDMDocument = class(TAutoObject, IDMDocument)
  private
    { Private declarations }
    FOwner: TObject;                                  // reference to DataForm
    FWorksheet: TDMWorksheet;              // references to 'child' interfaces
    FContainer: TDMContainer;
    FPlot: TDMPlot;
    FClosing: boolean;// used to avoid COM alert when controller closes window 
  public
    { Public declarations }
    constructor Create(AOwner: TObject);
    procedure CloseDoc;
    procedure AddSerieObject;
    procedure SwapSerieObjects(S1, S2: integer);
    procedure DeleteSerieObject(Idx: integer);
    function IsExternalRefs: boolean;
  protected
    { Protected declarations }
    function Get_Container: IDispatch; safecall;
    function Get_WindowCaption: WideString; safecall;
    function Get_WindowState: WindowStates; safecall;
    procedure Set_WindowCaption(const Value: WideString); safecall;
    procedure Set_WindowState(Value: WindowStates); safecall;
    function Get_Plot: IDispatch; safecall;
    function Get_Worksheet: IDispatch; safecall;
    function Get_IsRecording: WordBool; safecall;
    procedure Set_IsRecording(Value: WordBool); safecall;
    procedure Activate; safecall;
    procedure SaveUndoInfo; safecall;
    procedure Close; safecall;
  end;

  TDMWorksheet = class(TAutoObject, IDMWorksheet)
  private
    { Private declarations }
    FOwner: TObject;                                  // reference to DataForm
  protected
    { Protected declarations }
    function Get_ColumnLabels(Index: Integer): WideString; safecall;
    procedure Set_ColumnLabels(Index: Integer; const Value: WideString);
      safecall;
    procedure Scroll(Line: Integer); safecall;
    function Get_XColumn: Integer; safecall;
    function Get_YColumn: Integer; safecall;
    procedure Set_XColumn(Value: Integer); safecall;
    procedure Set_YColumn(Value: Integer); safecall;
    function Get_SelectionBottom: Integer; safecall;
    function Get_SelectionLeft: Integer; safecall;
    function Get_SelectionRight: Integer; safecall;
    function Get_SelectionTop: Integer; safecall;
    procedure CopyToClipboard(UseTabs: WordBool); safecall;
    procedure Delete; safecall;
    procedure PasteFromClipboard(InsertLines, Overwrite: WordBool); safecall;
    procedure SelectAll; safecall;
    procedure Set_SelectionBottom(Value: Integer); safecall;
    procedure Set_SelectionLeft(Value: Integer); safecall;
    procedure Set_SelectionRight(Value: Integer); safecall;
    procedure Set_SelectionTop(Value: Integer); safecall;
    procedure Select(Left, Right, Top, Bottom: Integer); safecall;
  end;

  TDMContainer = class(TAutoObject, IDMContainer)
  private
    { Private declarations }
    FOwner: TObject;                                  // reference to DataForm
  protected
    { Protected declarations }
    function Get_FileName: WideString; safecall;
    function Get_Modified: WordBool; safecall;
    procedure Set_Modified(Value: WordBool); safecall;
    function Get_ItemCount: Integer; safecall;
    function Get_Items(Index: Integer): OleVariant; safecall;
    function Get_Strings(Index: Integer): WideString; safecall;
    procedure AddItem(Values: OleVariant); safecall;
    procedure Clear; safecall;
    procedure DeleteItem(Position: Integer); safecall;
    procedure InsertItem(Position: Integer; Values: OleVariant); safecall;
    procedure Load(const FileName: WideString); safecall;
    procedure Save(const FileName: WideString); safecall;
    procedure Set_Items(Index: Integer; Values: OleVariant); safecall;
  end;

  TDMAxis = class(TAutoObject, IDMAxis)
  private
    { Private declarations }
    FOwner: TObject;                                  // reference to DataForm
    function IsX: boolean;                       // true if X axis, false if Y
  protected
    { Protected declarations }
    function Get_AutoScale: WordBool; safecall;
    function Get_Expression: WideString; safecall;
    function Get_Margins: Double; safecall;
    function Get_Max: Double; safecall;
    function Get_Min: Double; safecall;
    function Get_Title: WideString; safecall;
    procedure Set_AutoScale(Value: WordBool); safecall;
    procedure Set_Expression(const Value: WideString); safecall;
    procedure Set_Margins(Value: Double); safecall;
    procedure Set_Max(Value: Double); safecall;
    procedure Set_Min(Value: Double); safecall;
    procedure Set_Title(const Value: WideString); safecall;
  end;

  TDMSerie = class(TAutoObject, IDMSerie)
  private
    { Private declarations }
    FOwner: TObject;                                  // reference to DataForm
    FIndex: integer;                     // index in FSeries[] and Plot.Series
  protected
    { Protected declarations }
    function Get_FirstLine: Integer; safecall;
    function Get_IsEmpty: WordBool; safecall;
    function Get_IsFunction: WordBool; safecall;
    function Get_IsRecording: WordBool; safecall;
    function Get_LastLine: Integer; safecall;
    function Get_XColumn: Integer; safecall;
    function Get_XExpression: WideString; safecall;
    function Get_YColumn: Integer; safecall;
    function Get_YExpression: WideString; safecall;
    procedure ClearSelection; safecall;
    procedure Select(X, Y, B, E: Integer); safecall;
    procedure Set_FirstLine(Value: Integer); safecall;
    procedure Set_IsFunction(Value: WordBool); safecall;
    procedure Set_IsRecording(Value: WordBool); safecall;
    procedure Set_LastLine(Value: Integer); safecall;
    procedure Set_XColumn(Value: Integer); safecall;
    procedure Set_XExpression(const Value: WideString); safecall;
    procedure Set_YColumn(Value: Integer); safecall;
    procedure Set_YExpression(const Value: WideString); safecall;
    function Get_Text: WideString; safecall;
    procedure Set_Text(const Value: WideString); safecall;
    function Get_Container: IDispatch; safecall;
    procedure Set_Container(const Value: IDispatch); safecall;
    procedure AddPoint; safecall;
    procedure GetPoint(Index: Integer; out X, Y: OleVariant); safecall;
  end;

  TDMPlot = class(TAutoObject, IDMPlot)
  private
    { Private declarations }
    FOwner: TObject;                                  // reference to DataForm
    FXAxis: TDMAxis;                       // references to 'child' interfaces
    FYAxis: TDMAxis;
    FSeries: array of TDMSerie;
  protected
    { Protected declarations }
    function Get_CurrentSerie: IDispatch; safecall;
    function Get_SelectionBottom: Double; safecall;
    function Get_SelectionLeft: Double; safecall;
    function Get_SelectionRight: Double; safecall;
    function Get_SelectionTop: Double; safecall;
    function Get_SerieCount: Integer; safecall;
    function Get_SerieIndex: Integer; safecall;
    function Get_XAxis: IDispatch; safecall;
    function Get_YAxis: IDispatch; safecall;
    procedure CopyToClipboard(Mode: PlotCopyModes; UseTabs: WordBool);
      safecall;
    procedure Delete; safecall;
    procedure Set_SelectionBottom(Value: Double); safecall;
    procedure Set_SelectionLeft(Value: Double); safecall;
    procedure Set_SelectionRight(Value: Double); safecall;
    procedure Set_SelectionTop(Value: Double); safecall;
    procedure Set_SerieIndex(Value: Integer); safecall;
    procedure Select(X1, X2, Y1, Y2: Double); safecall;
    function AddSerie: IDispatch; safecall;
    function Get_Series(Index: Integer): IDispatch; safecall;
  end;

implementation

uses
  ComServ, MainWnd, DataWnd, Strings, Common, Data, Plot, Variants,
  Sysutils, ActnList, Menus, StdCtrls, Forms, Grids, AppDlg, NotesWnd,
  MonWnd, LeastSqr, Controls, LMFit, LMOptDlg, LMFitDlg;

{ TDMInternalApp }

function TDMInternalApp.Get_ActiveDocument: IDispatch;
begin
  if Assigned(MainForm.ActiveDataForm)
  then Result:=MainForm.ActiveDataForm.DMDocument as IDispatch
  else Result:=nil;
end;

function TDMInternalApp.Get_DocumentCount: Integer;
begin Result:=MainForm.MDIChildCount; end;

function TDMInternalApp.Get_Documents(Index: Integer): IDispatch;
begin
  if Assigned(MainForm.MDIChildren[Index]) then
  Result:=(MainForm.MDIChildren[Index] as TDataForm).DMDocument as IDispatch
  else Result:=nil;
end;

procedure TDMInternalApp.DisplayMessage(const Msg: WideString);
begin MainForm.DisplayMessage(Msg); end;

procedure TDMInternalApp.RecordPoint(Labels, Values: OleVariant);
var I,N: integer; S: TStringList; R: TRealArray;
begin
  if VarIsArray(Values) then
  begin
    N:=VarArrayHighBound(Values,1)+1; for I:=1 to N do R[I]:=Values[I-1];
  end else N:=Str2Real(Values, R);
  S:=TStringList.Create;
  try
    if VarIsArray(Labels) then
    begin for I:=0 to VarArrayHighBound(Labels,1) do S.Add(Labels[I]); end
    else S.Text:=Labels;
    MainForm.RecordPoint(N,R,S);
  finally
    S.Free;
  end;
end;

procedure TDMInternalApp.ExecuteCommand(const Name: WideString);
begin
  if MainForm.FindComponent(Name) is TAction then
  (MainForm.FindComponent(Name) as TAction).Execute else
  if MonitorForm.FindComponent(Name) is TAction then
  (MonitorForm.FindComponent(Name) as TAction).Execute;
end;

function TDMInternalApp.Get_IsCommandChecked(const Name:WideString): WordBool;
begin
  if MainForm.FindComponent(Name) is TAction
  then Result:=(MainForm.FindComponent(Name) as TAction).Checked else
  if MonitorForm.FindComponent(Name) is TAction
  then Result:=(MonitorForm.FindComponent(Name) as TAction).Checked else
  Result:=false;
end;

procedure TDMInternalApp.Set_IsCommandChecked(const Name: WideString;
  Value: WordBool);
begin
  if MainForm.FindComponent(Name) is TAction
  then (MainForm.FindComponent(Name) as TAction).Checked:=Value;
  if MonitorForm.FindComponent(Name) is TAction
  then (MonitorForm.FindComponent(Name) as TAction).Checked:=Value;
end;

function TDMInternalApp.Get_IsCommandEnabled(const Name:WideString): WordBool;
begin
  if MainForm.FindComponent(Name) is TAction
  then Result:=(MainForm.FindComponent(Name) as TAction).Enabled else
  if MonitorForm.FindComponent(Name) is TAction
  then Result:=(MonitorForm.FindComponent(Name) as TAction).Enabled else
  Result:=false;
end;

procedure TDMInternalApp.Set_IsCommandEnabled(const Name: WideString;
  Value: WordBool);
begin
  if MainForm.FindComponent(Name) is TAction
  then (MainForm.FindComponent(Name) as TAction).Enabled:=Value;
  if MonitorForm.FindComponent(Name) is TAction
  then (MonitorForm.FindComponent(Name) as TAction).Enabled:=Value;
end;

function TDMInternalApp.Get_IsCommandVisible(const Name:WideString): WordBool;
begin
  if MainForm.FindComponent(Name) is TAction
  then Result:=(MainForm.FindComponent(Name) as TAction).Visible else
  if MonitorForm.FindComponent(Name) is TAction
  then Result:=(MonitorForm.FindComponent(Name) as TAction).Visible else
  Result:=false;
end;

procedure TDMInternalApp.Set_IsCommandVisible(const Name: WideString;
  Value: WordBool);
begin
  if MainForm.FindComponent(Name) is TAction
  then (MainForm.FindComponent(Name) as TAction).Visible:=Value;
  if MonitorForm.FindComponent(Name) is TAction
  then (MonitorForm.FindComponent(Name) as TAction).Visible:=Value;
end;

procedure TDMInternalApp.SetExpSettings(const Settings: WideString);
begin MainForm.ExpSettings:=Settings; end;

function TDMInternalApp.CreateDocument(const FileName: WideString): IDispatch;
begin
  if FileName='' then with MainForm.CreateDataForm do
  begin                                         // just as in NewActionExecute
    Container.FileName:=Format(strNoname,[MainForm.MDIChildCount]);
    Result:=DMDocument as IDispatch;
  end else
  if FileExists(FileName) then
     if UpperCase(ExtractFileExt(FileName))='.DMW' then        // use template
     begin
       MainForm.ReadDesktop(FileName,true);
       if Assigned(MainForm.ActiveDataForm)
       then Result:=MainForm.ActiveDataForm.DMDocument as IDispatch
       else Result:=nil;
     end else Result:=MainForm.LoadFile(FileName).DMDocument as IDispatch
  else Result:=nil;
  MainForm.UpdateSerieComboBox; // refreshed in Idle (and remain bad until it)
  MainForm.UpdateSerieList;
end;

procedure TDMInternalApp.ShowProgress(Progress: Integer);
begin // produce flickers instead of indication if called by out-of-process!!!
  if ClassName='TDMInternalApp' then MainForm.ContainerProgress(nil,Progress);
end;

procedure TDMInternalApp.DisplayHint(const Hint: WideString); safecall;
begin MainForm.DisplayHint(Hint); end;

constructor TDMInternalApp.Create;
begin inherited; FNotes:=TDMNotes.Create; FNotes._AddRef; end;

destructor TDMInternalApp.Destroy;
begin FNotes._Release; inherited; end;

function TDMInternalApp.Get_Notes: IDispatch;
begin Result:=FNotes as IDispatch; end;

procedure TDMInternalApp.LinearFit(X, Y: OleVariant; NumTerms: Integer;
  BasisType: LinearBasisType; out Solution: OleVariant;
  out Deviation: OleVariant; out ResultCode: OleVariant);
var                                 // as in TLinearFitForm!
  XData, YData : TNColumnVector;    { Data points (X,Y)  }
  Residuals : TNColumnVector;       { YFit - YData  }
  Error : byte;                     { Flags if something went wrong  }
  Solution1: TNRowVector;           { Coefficients of the l.s. fit  }
  NumPoints : integer;              { # of points  }
  YFit : TNColumnVector;            { Least squares solution at XData points }
  Variance,StandardDeviation: Float;{ Indicates goodness of fit  }
  I: integer; Fitter: TFitFunctionClass;
begin
  NumPoints:=VarArrayHighBound(X,1)+1;
  if (VarArrayHighBound(Y,1)+1<>NumPoints) then          // check array length
  begin ResultCode:=frBadParams; Exit; end;
  ResultCode:=frError;
  SetLength(XData, NumPoints+1); SetLength(YData, NumPoints+1); // init arrays
  SetLength(YFit, NumPoints+1); SetLength(Residuals, NumPoints+1);
  case BasisType of
    lfbPoly: Fitter:=TPolyFitFunction;
    lbfFourier: Fitter:=TFourierFitFunction;
    lbfPower: Fitter:=TPowerFitFunction;
    lbfExponent: Fitter:=TExpoFitFunction;
    lbfLorentz: Fitter:=TLorentzFitFunction;
    lbfSine: Fitter:=TSineFitFunction;
  else begin ResultCode:=frBadParams; Exit; end;
  end;
  MainForm.DisplayHint(strWorking); Screen.Cursor:=crHourGlass;
  try
    for I:=1 to NumPoints do              // copy data (zero element ignored!)
    begin XData[I]:=X[I-1]; YData[I]:=Y[I-1]; end;
    LeastSquares(NumPoints, XData, YData, NumTerms, Solution1,        // FIRE!
      YFit, Residuals, StandardDeviation, Variance, Error, Fitter);
  finally
    MainForm.DisplayHint(''); Screen.Cursor:=crDefault;
  end;
  case Error of                                 // calculate OUTPUT parameters
    0: ResultCode:=frOK;
    1..3: ResultCode:=frBadParams;
    4: ResultCode:=frFitError;
  end;
  Deviation:=StandardDeviation; // !?
  Solution:=VarArrayCreate([0,NumTerms-1],varVariant);
  for I:=1 to NumTerms do Solution[I-1]:=Solution1[I];
end;

var NumLMIter: integer; // !!!
procedure LMFitProgress;
begin MainForm.DisplayHint(Format(strLMFitStatus,[NumLMIter])); end;

procedure TDMInternalApp.LMFit(X,Y,W: OleVariant; WType: WeightType;
  Expression: OleVariant; var Params: OleVariant; var Sigma: OleVariant;
  var NIter: OleVariant; out ChiSqr: OleVariant; out ResultCode: OleVariant);
  safecall;
var XX,YY,Yfit,WW: DataPointer;
    Chi_SqR: TReal; A,SigmaA: FitArray; ParDel,ChiDel: TReal;
    Code: TFitResult; WT: TWeightType; I,NParam,NPoints: integer;
begin
  if NLSFOptionsForm.ParDelCheckBox.Checked
  then ParDel:=StrToFloat(NLSFOptionsForm.ParDelEdit.Text) else ParDel:=0;
  if NLSFOptionsForm.ChiDelCheckBox.Checked
  then ChiDel:=StrToFloat(NLSFOptionsForm.ChiDelEdit.Text) else ChiDel:=0;
  LMDelta:=StrToFloat(NLSFOptionsForm.DerivDeltaEdit.Text);
  NParam:=VarArrayHighBound(Params,1)+1;
  if VarArrayHighBound(Sigma,1)+1<>NParam
  then begin ResultCode:=frBadParams; Exit; end;
  for I:=0 to NParam-1 do
  begin A[I+1]:=Params[I]; SigmaA[I+1]:=Sigma[I]; end;
  if not Assigned(NLSFForm)              // NOTE!!! we use its parser & fields
  then NLSFForm:=TNLSFForm.Create(Application);
  if VarIsByRef(Expression) then         // copy parameters for LMExpression()
  NLSFForm.Expression:=Variant(TVarData(Expression).VPointer^) else
  NLSFForm.Expression:=Expression;
{ NOTE: in future we'll implement Expression=IDispatch to transfer
  pointers to procedures in script engine or another binary object! }
  NLSFForm.CalcNParam;
  if NLSFForm.NParam<>NParam
  then begin ResultCode:=frBadParams; Exit; end;
  NPoints:=VarArrayHighBound(X,1)+1;
  if (VarArrayHighBound(Y,1)+1<>NPoints) or
     (VarArrayHighBound(Y,1)+1<>NPoints) or
     ((WType>0) and (VarArrayHighBound(W,1)+1<>NPoints))
  then begin ResultCode:=frBadParams; Exit; end;
  WT:=wtEqual;
  case WType of
    1: WT:=wtRelative;
    2: WT:=wtAbsolute;
  end;
  GetMem(XX,NPoints*SizeOf(TReal));
  GetMem(YY,NPoints*SizeOf(TReal));
  GetMem(YFit,NPoints*SizeOf(TReal));
  GetMem(WW,NPoints*SizeOf(TReal));
  MainForm.DisplayHint(strWorking);
  Screen.Cursor:=crHourGlass;
  NumLMIter:=NIter;// atn: global var!
  ResultCode:=1;
  try
    for I:=1 to NPoints do
    begin
      XX[I]:=X[I-1]; YY[I]:=Y[I-1]; WW[I]:=1;
      if WT<>wtEqual then WW[I]:=W[I-1];
    end;
    CurveFit(XX,YY,WW,NPoints,NParam,A,SigmaA,Chi_SqR,Yfit,NumLMIter,ParDel,
      ChiDel,WT,LMExpression, LMDerivative, LMFitProgress, Code);
  finally
    FreeMem(XX);
    FreeMem(YY);
    FreeMem(YFit);
    FreeMem(WW);
    MainForm.DisplayHint('');
    Screen.Cursor:=crDefault;
  end;
  case Code of
    frParamConv: ResultCode:=4;
    frChiConv: ResultCode:=5;
    frMaxIter: ResultCode:=6;
    frFitError: ResultCode:=2;
  end;
  NIter:=NumLMIter; ChiSqr:=Chi_Sqr;
  for I:=0 to NParam-1 do
  begin Params[I]:=A[I+1]; Sigma[I]:=SigmaA[I+1]; end;
end;

{ TDMExternalApp }

procedure TDMExternalApp.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IDMApplicationEvents;
  if FConnectionPoint <> nil then
     FSinkList := FConnectionPoint.SinkList;
end;

procedure TDMExternalApp.Initialize;
begin
  inherited Initialize;
  FConnectionPoints := TConnectionPoints.Create(Self);
  if AutoFactory.EventTypeInfo <> nil then
    FConnectionPoint := FConnectionPoints.CreateConnectionPoint(
      AutoFactory.EventIID, ckSingle, EventConnect)
  else FConnectionPoint := nil;
  {instead of overriding constructor, we place additional initialization here}
  if Assigned(MainForm) then with MainForm do       // show icon in statusbar!
  begin DMApplication:=Self; StatusBar.Refresh; end;
  if not Assigned(FNotes) then          // "inherited" constructor NOT called!
  begin FNotes:=TDMNotes.Create; FNotes._AddRef; end;
end;

destructor TDMExternalApp.Destroy;
begin
  if Assigned(MainForm) then with MainForm do           // undo initialization
  begin
    DMApplication:=nil;
    if Assigned(StatusBar) then StatusBar.Refresh; // if server forced to exit
  end;
  inherited;
end;

procedure TDMExternalApp.CloseDoc(Doc: TDMDocument);
begin                                     // these 2 methods called internally
  if Assigned(FEvents) then FEvents.OnCloseDocument(Doc as IDispatch);
end;

procedure TDMExternalApp.CreateDoc(Doc: TDMDocument);
begin
  if Assigned(FEvents) then FEvents.OnCreateDocument(Doc as IDispatch);
end;

procedure TDMExternalApp.Command(CMD: string);
begin
  if Assigned(FEvents) then FEvents.OnCommand(CMD);
end;

{ TDMDocument }

procedure TDMDocument.AddSerieObject; // must be called after adding new serie
var N: integer;
begin
  N:=Length(FPlot.FSeries)+1;
  SetLength(FPlot.FSeries,N);
  FPlot.FSeries[N-1]:=TDMSerie.Create;
  with FPlot.FSeries[N-1] do
  begin _AddRef; FOwner:=Self.FOwner; FIndex:=N-1; end;
end;

procedure TDMDocument.SwapSerieObjects(S1, S2: integer); // called by seriemgr
var Tmp: TDMSerie;
begin
  if (S1>=Length(FPlot.FSeries)) or (S2>=Length(FPlot.FSeries))
  or (S1<0) or (S2<0) then Exit;                           // check parameters
  FPlot.FSeries[S1].FIndex:=S2; FPlot.FSeries[S2].FIndex:=S1;  // change index
  Tmp:=FPlot.FSeries[S1]; FPlot.FSeries[S1]:=FPlot.FSeries[S2];
  FPlot.FSeries[S2]:=Tmp;                                      // swap objects
end;

procedure TDMDocument.DeleteSerieObject(Idx: integer);   // called by seriemgr
var I,N: integer;
begin
  N:=Length(FPlot.FSeries); if (Idx<0) or (Idx>=N) then Exit;     // check Idx
  FPlot.FSeries[Idx].FOwner:=nil; FPlot.FSeries[Idx]._Release;
  for I:=Idx to N-2 do                 // move items and correct FIndex fields
  begin
    FPlot.FSeries[I]:=FPlot.FSeries[I+1]; FPlot.FSeries[I].FIndex:=I;
  end;
  SetLength(FPlot.FSeries,N-1);                                // shrink array
end;

constructor TDMDocument.Create(AOwner: TObject); // called in TDataForm.Create
var I: integer;
begin
  inherited Create;
  FOwner:=AOwner;
  _AddRef;                        // COM object must ALWAYS die after DataForm
  FWorksheet:=TDMWorksheet.Create;           // create and setup child objects
  with FWorksheet do begin _AddRef; FOwner:=AOwner; end;
  FContainer:=TDMContainer.Create;
  with FContainer do begin _AddRef; FOwner:=AOwner; end;
  FPlot:=TDMPlot.Create;
  with FPlot do begin _AddRef; FOwner:=AOwner; end;
  FPlot.FXAxis:=TDMAxis.Create;         // create and setup plot child objects
  with FPlot.FXAxis do begin _AddRef; FOwner:=AOwner; end;
  FPlot.FYAxis:=TDMAxis.Create;
  with FPlot.FYAxis do begin _AddRef; FOwner:=AOwner; end;
  for I:=0 to (AOwner as TDataForm).Plot.Series.Count-1 do AddSerieObject;
  if Assigned(MainForm.DMApplication) then     // nil if started not as server
  (MainForm.DMApplication as TDMExternalApp).CreateDoc(Self);  // after all!!!
end;

procedure TDMDocument.CloseDoc;                 // called in TDataForm.OnClose
var I: integer;
begin
  if Assigned(MainForm.DMApplication) then   // check - this object may be nil
  (MainForm.DMApplication as TDMExternalApp).CloseDoc(Self);
  if FOwner is TDataForm then (FOwner as TDataForm).DMDocument:=nil;
  FOwner:=nil;     // DataForm reference used in other methods must be cleared
  _Release;                           // pair to _AddRef called in constructor
  // release plot child objects:
  with FPlot.FXAxis do begin FOwner:=nil; _Release; end;
  with FPlot.FYAxis do begin FOwner:=nil; _Release; end;
  for I:=0 to High(FPlot.FSeries) do
  with FPlot.FSeries[I] do begin FOwner:=nil; _Release; end;
  SetLength(FPlot.FSeries,0); //???
  // release child objects:
  with FWorksheet do begin FOwner:=nil; _Release; end;
  with FContainer do begin FOwner:=nil; _Release; end;
  with FPlot do begin FOwner:=nil; _Release; end;
end;

function TDMDocument.IsExternalRefs: boolean;
var I: integer;
begin
  Result:=false; if FClosing then Exit;          // document closed externally
  Result:=RefCount>1; if Result then Exit;
  Result:=FContainer.RefCount>1; if Result then Exit;
  Result:=FPlot.RefCount>1; if Result then Exit;
  Result:=FWorksheet.RefCount>1; if Result then Exit;
  Result:=FPlot.FXAxis.RefCount>1; if Result then Exit;
  Result:=FPlot.FYAxis.RefCount>1; if Result then Exit;
  for I:=0 to Length(FPlot.FSeries)-1 do
  begin Result:=FPlot.FSeries[I].RefCount>1; if Result then Exit; end;
end;

function TDMDocument.Get_WindowCaption: WideString;
begin
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).Caption
  else Result:=strInvalidOwner;
end;

procedure TDMDocument.Set_WindowCaption(const Value: WideString);
begin
  if FOwner is TDataForm then
  begin
    (FOwner as TDataForm).Caption:=Value;
    MainForm.UpdateMDIMenu; MainForm.UpdateWindowList;
    MainForm.UpdateSerieList;
  end;
end;

function TDMDocument.Get_WindowState: WindowStates;
begin
  if FOwner is TDataForm then Result:=ord((FOwner as TDataForm).ViewState)
  else Result:=0;
end;

procedure TDMDocument.Set_WindowState(Value: WindowStates);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).ViewState:=TDataFormView(Value);
end;

function TDMDocument.Get_IsRecording: WordBool;
begin
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).Recording
  else Result:=false;
end;

procedure TDMDocument.Set_IsRecording(Value: WordBool);
begin if FOwner is TDataForm then (FOwner as TDataForm).Recording:=Value; end;

procedure TDMDocument.Activate;
begin
  if FOwner is TDataForm then with FOwner as TDataForm do
  begin Show; if WindowState=wsMinimized then WindowState:=wsNormal; end;
end;

procedure TDMDocument.SaveUndoInfo;
begin
 if FOwner is TDataForm then (FOwner as TDataForm).SaveUndoInfo(MainForm);
end;

procedure TDMDocument.Close;
begin
  if FOwner is TDataForm then
  begin
    FClosing:=true; (FOwner as TDataForm).Close; {Release?} FClosing:=false;
  end;
end;

function TDMDocument.Get_Container: IDispatch;
begin Result:=FContainer as IDispatch; end;

function TDMDocument.Get_Plot: IDispatch;
begin Result:=FPlot as IDispatch; end;

function TDMDocument.Get_Worksheet: IDispatch;
begin Result:=FWorksheet as IDispatch; end;

{TDMWorksheet}

function TDMWorksheet.Get_ColumnLabels(Index: Integer): WideString;
begin               // note that COLUMNS ALWAYS are counted from 1 to MaxCols!
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).Labels[Index]
  else Result:='';
end;

procedure TDMWorksheet.Set_ColumnLabels(Index: Integer;
  const Value: WideString);
begin
  if FOwner is TDataForm then (FOwner as TDataForm).Labels[Index]:=Value;
end;

procedure TDMWorksheet.Scroll(Line: Integer);
begin if FOwner is TDataForm then (FOwner as TDataForm).Scroll(Line); end;

function TDMWorksheet.Get_XColumn: Integer;
begin
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).XColumn
  else Result:=0;
end;

function TDMWorksheet.Get_YColumn: Integer;
begin
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).YColumn
  else Result:=0;
end;

procedure TDMWorksheet.Set_XColumn(Value: Integer);
begin if FOwner is TDataForm then (FOwner as TDataForm).XColumn:=Value; end;

procedure TDMWorksheet.Set_YColumn(Value: Integer);
begin if FOwner is TDataForm then (FOwner as TDataForm).YColumn:=Value; end;

function TDMWorksheet.Get_SelectionLeft: Integer;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Worksheet do
  Result:=Selection.Left-FixedCols+1 else Result:=0;
end;

function TDMWorksheet.Get_SelectionRight: Integer;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Worksheet do
  Result:=Selection.Right-FixedCols+1 else Result:=0;
end;

function TDMWorksheet.Get_SelectionTop: Integer;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Worksheet do
  Result:=Selection.Top-FixedRows else Result:=0;
end;

function TDMWorksheet.Get_SelectionBottom: Integer;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Worksheet do
  begin
    Result:=Selection.Bottom;                         // need to be corrected!
    if Result=RowCount-1 then Result:=RowCount-2; Result:=Result-FixedRows;
  end else Result:=0;
end;

procedure TDMWorksheet.Set_SelectionBottom(Value: Integer);
var R: TGridRect;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Worksheet do
  begin
    R:=Selection; Value:=Value+FixedRows;
    if (Value>=FixedRows) and (Value<RowCount-1)
    then begin R.Bottom:=Value; Selection:=R; end;
  end;
end;

procedure TDMWorksheet.Set_SelectionLeft(Value: Integer);
var R: TGridRect;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Worksheet do
  begin
    R:=Selection; Value:=Value-1+FixedCols;
    if (Value>=FixedCols) and (Value<ColCount)
    then begin R.Left:=Value; Selection:=R; end;
  end;
end;

procedure TDMWorksheet.Set_SelectionRight(Value: Integer);
var R: TGridRect;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Worksheet do
  begin
    R:=Selection; Value:=Value-1+FixedCols;
    if (Value>=FixedCols) and (Value<ColCount)
    then begin R.Right:=Value; Selection:=R; end;
  end;
end;

procedure TDMWorksheet.Set_SelectionTop(Value: Integer);
var R: TGridRect;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Worksheet do
  begin
    R:=Selection; Value:=Value+FixedRows;
    if (Value>=FixedRows) and (Value<RowCount-1)
    then begin R.Top:=Value; Selection:=R; end;
  end;
end;

procedure TDMWorksheet.CopyToClipboard(UseTabs: WordBool);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Worksheet.CopyToClipBoard(UseTabs);
end;

procedure TDMWorksheet.Delete;
begin if FOwner is TDataForm then (FOwner as TDataForm).Worksheet.Delete; end;

procedure TDMWorksheet.PasteFromClipboard(InsertLines, Overwrite: WordBool);
begin
  if FOwner is TDataForm then
  (FOwner as TDataForm).Worksheet.PasteFromClipboard(InsertLines, Overwrite);
end;

procedure TDMWorksheet.SelectAll;
begin
  if FOwner is TDataForm then (FOwner as TDataForm).Worksheet.SelectAll;
end;

procedure TDMWorksheet.Select(Left, Right, Top, Bottom: Integer);
var R: TGridRect; FC,FR,CC,RC: integer;
begin
  if FOwner is TDataForm then
  begin
    with (FOwner as TDataForm).Worksheet do
    begin
      R:=Selection; FC:=FixedCols; FR:=FixedRows; CC:=ColCount;RC:=RowCount-1;
    end;
    Left:=Left+FC-1; Right:=Right+FC-1;
    Top:=Top+FR; Bottom:=Bottom+FR;
    if (Left>=FC) and (Left<CC) then R.Left:=Left;
    if (Right>=FC) and (Right<CC) then R.Right:=Right;
    if (Top>=FR) and (Top<RC) then R.Top:=Top;
    if (Bottom>=FR) and (Bottom<RC) then R.Bottom:=Bottom;
    (FOwner as TDataForm).Worksheet.Selection:=R;
  end;
end;

{ TDMContainer }

function TDMContainer.Get_FileName: WideString;
begin
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).Container.FileName
  else Result:=strInvalidOwner;
end;

function TDMContainer.Get_Modified: WordBool;
begin
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).Container.Modified
  else Result:=false;
end;

procedure TDMContainer.Set_Modified(Value: WordBool);
begin
  if FOwner is TDataForm then (FOwner as TDataForm).Container.Modified:=Value;
end;

function TDMContainer.Get_ItemCount: Integer;
begin
  if FOwner is TDataForm
  then Result:=(FOwner as TDataForm).Container.Items.Count else Result:=0;
end;

function TDMContainer.Get_Items(Index: Integer): OleVariant;
var I: integer; D: TRealData;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Container do
  if (Index>=0) and (Index<Items.Count) then
  begin
    D:=TRealData(Items[Index]);
    Result:=VarArrayCreate([0,D.Size-1],varVariant);
    for I:=1 to D.Size do Result[I-1]:=D.RData[I];
  end;
end;

procedure TDMContainer.Set_Items(Index: Integer; Values: OleVariant);
var I,N: integer; D: TRealData; R: TRealArray;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Container do
  if (Index>=0) and (Index<Items.Count) then
  if VarIsArray(Values) then
  begin
    N:=VarArrayHighBound(Values,1)+1;
    for I:=1 to N do R[I]:=Values[I-1];
    D:=TRealData(Items[Index]);
    D.SetRData(N,R);
  end else TData(Items[Index]).Data:=Values;
end;

function TDMContainer.Get_Strings(Index: Integer): WideString;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Container do
    if (Index>=0) and (Index<Items.Count)
    then Result:=TData(Items[Index]).Data else Result:=''
  else Result:=strInvalidOwner;
end;

procedure TDMContainer.AddItem(Values: OleVariant);
var I,N: integer; D: TRealData; R: TRealArray;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Container do
  if VarIsArray(Values) then
  begin
    N:=VarArrayHighBound(Values,1)+1;
    for I:=1 to N do R[I]:=Values[I-1];
    D:=TRealData(InitItem);
    D.SetRData(N,R);
    Items.Add(D);
  end else begin D:=TRealData(InitItem); D.Data:=Values; Items.Add(D); end;
end;

procedure TDMContainer.Clear;
begin if FOwner is TDataForm then (FOwner as TDataForm).Container.Clear; end;

procedure TDMContainer.DeleteItem(Position: Integer);
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Container do
  if (Position>=0) and (Position<Items.Count) then
  begin TData(Items[Position]).Free; Items.Delete(Position); end;
end;

procedure TDMContainer.InsertItem(Position: Integer; Values: OleVariant);
var I,N: integer; D: TRealData; R: TRealArray;      // looks like AddItem()...
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Container do
  if (Position>=0) and (Position<Items.Count) then
  if VarIsArray(Values) then
  begin
    N:=VarArrayHighBound(Values,1)+1;
    for I:=1 to N do R[I]:=Values[I-1];
    D:=TRealData(InitItem);
    D.SetRData(N,R);
    Items.Insert(Position,D);
  end
  else begin D:=TRealData(InitItem); D.Data:=Values; Items.Insert(Position,D); end;
end;

procedure TDMContainer.Load(const FileName: WideString);
begin
  if FOwner is TDataForm then
  begin
    (FOwner as TDataForm).Container.Filename:=FileName;  // autoupdate caption
    MainForm.UpdateMDIMenu; MainForm.UpdateWindowList;
    MainForm.StatusImageList.Tag:=1;              // as in MainForm.LoadFile()
    MainForm.DisplayHint(Format(strLoadingFile,[FileName]));
    MainForm.LoadingFile:=true;
    try
      (FOwner as TDataForm).Container.Clear;     // clear all dependent series
      (FOwner as TDataForm).Container.LoadFromFile;        // modified:=false!
    finally
      MainForm.StatusImageList.Tag:=0; MainForm.LoadingFile:=false;
      (FOwner as TDataForm).Worksheet.UpdateSize; MainForm.DisplayHint('');
    end;
  end       // here we don't access metabase - use CreateDocument() instead!!!
end;

procedure TDMContainer.Save(const FileName: WideString);
begin
  if FOwner is TDataForm then
  try
    (FOwner as TDataForm).Container.FileName:=FileName;
    MainForm.UpdateMDIMenu; MainForm.UpdateWindowList;
    MainForm.StatusImageList.Tag:=2;
    MainForm.DisplayHint(Format(strSavingFile,[FileName]));
    (FOwner as TDataForm).Container.SaveToFile(
    AppPropsForm.BackupCheckBox.Checked);
  finally
    MainForm.StatusImageList.Tag:=0; MainForm.DisplayHint('');
  end;
end;

{ TDMPlot }

function TDMPlot.Get_CurrentSerie: IDispatch;
begin
  if FOwner is TDataForm then
  Result:=FSeries[(FOwner as TDataForm).Plot.SerieIndex] as IDispatch
  else Result:=nil;
end;

function TDMPlot.Get_XAxis: IDispatch;
begin Result:=FXAxis as IDispatch; end;

function TDMPlot.Get_YAxis: IDispatch;
begin Result:=FYAxis as IDispatch; end;

function TDMPlot.Get_SerieCount: Integer;
begin
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).Plot.Series.Count
  else Result:=0;
end;

function TDMPlot.Get_SerieIndex: Integer;
begin
  if FOwner is TDataForm then Result:=(FOwner as TDataForm).Plot.SerieIndex
  else Result:=-1;
end;

procedure TDMPlot.Set_SerieIndex(Value: Integer);
begin
  if (FOwner is TDataForm) and (Value>=0) and
  (Value<(FOwner as TDataForm).Plot.Series.Count) then with MainForm do
  begin SerieComboBox.ItemIndex:=Value; SerieComboBoxChange(nil); end;
end;

function TDMPlot.Get_SelectionBottom: Double;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.SelectionBottom else Result:=0;
end;

function TDMPlot.Get_SelectionLeft: Double;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.SelectionLeft else Result:=0;
end;

function TDMPlot.Get_SelectionRight: Double;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.SelectionRight else Result:=0;
end;

function TDMPlot.Get_SelectionTop: Double;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.SelectionTop else Result:=0;
end;

procedure TDMPlot.Set_SelectionBottom(Value: Double);
begin
  if FOwner is TDataForm then
  (FOwner as TDataForm).Plot.SelectionBottom:=Value;
end;

procedure TDMPlot.Set_SelectionLeft(Value: Double);
begin
  if FOwner is TDataForm then
  (FOwner as TDataForm).Plot.SelectionLeft:=Value;
end;

procedure TDMPlot.Set_SelectionRight(Value: Double);
begin
  if FOwner is TDataForm then
  (FOwner as TDataForm).Plot.SelectionRight:=Value;
end;

procedure TDMPlot.Set_SelectionTop(Value: Double);
begin
  if FOwner is TDataForm then
  (FOwner as TDataForm).Plot.SelectionTop:=Value;
end;

procedure TDMPlot.CopyToClipboard(Mode: PlotCopyModes; UseTabs: WordBool);
begin
  if FOwner is TDataForm then
  (FOwner as TDataForm).Plot.CopyToClipBoard(TPlotCopyMode(Mode),UseTabs);
end;

procedure TDMPlot.Delete;
begin if FOwner is TDataForm then (FOwner as TDataForm).Plot.Delete; end;

procedure TDMPlot.Select(X1, X2, Y1, Y2: Double);
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
  begin
    SelectionTop:=Y2; SelectionBottom:=Y1;
    SelectionLeft:=X1; SelectionRight:=X2;
  end;
end;

function TDMPlot.AddSerie: IDispatch;         // as in AddSerieActionExecute()
var S: TSerie;
begin
  if FOwner is TDataForm then with FOwner as TDataForm do
  begin
    S:=Plot.Series.Add; S.Container:=Container;
    S.Text:=Format(strNewSerie,[Plot.Series.Count-1]);
    MainForm.UpdateSerieComboBox;                              // add new item
    MainForm.UpdateSerieList;
    DMDocument.AddSerieObject;                           // add new COM object
    Result:=FSeries[Length(FSeries)-1] as IDispatch;            // return ref.
  end else Result:=nil;
end;

function TDMPlot.Get_Series(Index: Integer): IDispatch;
begin
  if (Index>=0) and (Index<=High(FSeries))
  then Result:=FSeries[Index] as IDispatch else Result:=nil;
end;

{  TDMAxis }

function TDMAxis.IsX: boolean;
begin Result:=(FOwner as TDataForm).DMDocument.FPlot.FXAxis=Self; end;

function TDMAxis.Get_AutoScale: WordBool;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
    if IsX then Result:=XAxis.AutoScale else Result:=YAxis.AutoScale
  else Result:=false;
end;

procedure TDMAxis.Set_AutoScale(Value: WordBool);
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
  if IsX then XAxis.AutoScale:=Value else YAxis.AutoScale:=Value;
end;

function TDMAxis.Get_Expression: WideString;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
    if IsX then Result:=XAxis.Expression else Result:=YAxis.Expression
  else Result:=strInvalidOwner;
end;

procedure TDMAxis.Set_Expression(const Value: WideString);
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
  if IsX then XAxis.Expression:=Value else YAxis.Expression:=Value;
end;

function TDMAxis.Get_Title: WideString;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
    if IsX then Result:=XAxis.Title else Result:=YAxis.Title
  else Result:=strInvalidOwner;
end;

procedure TDMAxis.Set_Title(const Value: WideString);
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
  if IsX then XAxis.Title:=Value else YAxis.Title:=Value;
end;

function TDMAxis.Get_Margins: Double;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
    if IsX then Result:=XAxis.Margins else Result:=YAxis.Margins
  else Result:=0;
end;

procedure TDMAxis.Set_Margins(Value: Double);
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
  if IsX then XAxis.Margins:=Value else YAxis.Margins:=Value;
end;

function TDMAxis.Get_Min: Double;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
    if IsX then Result:=XAxis.Min else Result:=YAxis.Min
  else Result:=0;
end;

procedure TDMAxis.Set_Min(Value: Double);
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
  if IsX then XAxis.Min:=Value else YAxis.Min:=Value;
end;

function TDMAxis.Get_Max: Double;
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
    if IsX then Result:=XAxis.Max else Result:=YAxis.Max
  else Result:=0;
end;

procedure TDMAxis.Set_Max(Value: Double);
begin
  if FOwner is TDataForm then with (FOwner as TDataForm).Plot do
  if IsX then XAxis.Max:=Value else YAxis.Max:=Value;
end;

{ TDMSerie }

function TDMSerie.Get_FirstLine: Integer;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.Series[FIndex].FirstLine else Result:=-1;
end;

function TDMSerie.Get_LastLine: Integer;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.Series[FIndex].LastLine else Result:=-1;
end;

function TDMSerie.Get_XColumn: Integer;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.Series[FIndex].XColumn else Result:=0;
end;

function TDMSerie.Get_YColumn: Integer;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.Series[FIndex].YColumn else Result:=0;
end;

function TDMSerie.Get_IsEmpty: WordBool;
begin
  if FOwner is TDataForm then
  Result:=(FOwner as TDataForm).Plot.Series[FIndex].Empty else Result:=true;
end;

function TDMSerie.Get_IsFunction: WordBool;
begin
  if FOwner is TDataForm 
  then Result:=(FOwner as TDataForm).Plot.Series[FIndex].IsFunction
  else Result:=false;
end;

function TDMSerie.Get_IsRecording: WordBool;
begin
  if FOwner is TDataForm then with FOwner as TDataForm do
  Result:=IsRecordingSerie[FIndex] else Result:=false;
end;

function TDMSerie.Get_XExpression: WideString;
begin
  if FOwner is TDataForm
  then Result:=(FOwner as TDataForm).Plot.Series[FIndex].XExpression
  else Result:=strInvalidOwner;
end;

function TDMSerie.Get_YExpression: WideString;
begin
  if FOwner is TDataForm
  then Result:=(FOwner as TDataForm).Plot.Series[FIndex].YExpression
  else Result:=strInvalidOwner;
end;

function TDMSerie.Get_Text: WideString;
begin
  if FOwner is TDataForm
  then Result:=(FOwner as TDataForm).Plot.Series[FIndex].Text
  else Result:=strInvalidOwner;
end;

procedure TDMSerie.Set_FirstLine(Value: Integer);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Plot.Series[FIndex].FirstLine:=Value;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Set_LastLine(Value: Integer);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Plot.Series[FIndex].LastLine:=Value;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Set_XColumn(Value: Integer);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Plot.Series[FIndex].XColumn:=Value;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Set_YColumn(Value: Integer);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Plot.Series[FIndex].YColumn:=Value;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Set_IsFunction(Value: WordBool);
begin
  if FOwner is TDataForm then
  with (FOwner as TDataForm).Plot.Series[FIndex] do
  begin
    IsFunction:=Value; if Value then
    begin                        // do a little more than simple assignment...
      ClearBlock; LastLine:=200; LineVisible:=true;
      PointVisible:=false; MainForm.SerieComboBox.Refresh;{!!!}
    end;
  end;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Set_IsRecording(Value: WordBool);
begin
  if FOwner is TDataForm then with FOwner as TDataForm do
  IsRecordingSerie[FIndex]:=Value;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Set_XExpression(const Value: WideString);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Plot.Series[FIndex].XExpression:=Value;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Set_YExpression(const Value: WideString);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Plot.Series[FIndex].YExpression:=Value;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Set_Text(const Value: WideString);
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Plot.Series[FIndex].Text:=Value;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.ClearSelection;
begin
  if FOwner is TDataForm
  then (FOwner as TDataForm).Plot.Series[FIndex].ClearBlock;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.Select(X, Y, B, E: Integer);
begin
  if FOwner is TDataForm then
  with (FOwner as TDataForm).Plot.Series[FIndex] do
  begin XColumn:=X; YColumn:=Y; FirstLine:=B; LastLine:=E; end;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

function TDMSerie.Get_Container: IDispatch;
begin
  Result:=nil;
  if FOwner is TDataForm then
  with (FOwner as TDataForm).Plot.Series[FIndex] do
  if Assigned(Container) then
  Result:=(Container.Owner as TDataForm).DMDocument.FContainer as IDispatch;
end;

procedure TDMSerie.Set_Container(const Value: IDispatch);
var punk1,punk2: IUnknown; I: integer;
begin
  if not (FOwner is TDataForm) then Exit;  // Form destroyed - serie not exist
  punk1:=Value as IUnknown;
  for I:=0 to MainForm.MDIChildCount-1 do
  begin
    punk2:=(MainForm.MDIChildren[I] as
      TDataForm).DMDocument.FContainer as IUnknown;
    if punk1=punk2 then
    begin
      (FOwner as TDataForm).Plot.Series[FIndex].Container:=
      (MainForm.MDIChildren[I] as TDataForm).Container;
      Break;
    end;
  end;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.AddPoint;
var S: TSerie; P: TPlot;
    NotFirst: boolean; D,D0: TRealData; X0,Y0,X,Y: extended;
begin // this code looks like MainForm.RecordPoint() and works in the same way
  if not (FOwner is TDataForm) then Exit;
  P:=(FOwner as TDataForm).Plot; S:=P.Series[FIndex];
  if S.Container.Items.Count=0 then Exit;                // no points to draw!
  NotFirst:=S.Container.Items.Count>1;
  D:=S.Container.Items[S.Container.Items.Count-1];
  if NotFirst then                 // move leader as in MainForm.RecordPoint()
  begin
    D0:=S.Container.Items[S.Container.Items.Count-2];
    if not P.GetDataPoint(D0,S.XColumn,S.YColumn,S.XExpression,S.YExpression,
    X0,Y0) then (FOwner as TDataForm).DrawLeader(X0,Y0) else NotFirst:=false;
  end;
  P.Canvas.Pen:=S.Pen; P.Canvas.Brush:=S.Brush;
  if not P.GetDataPoint(D,S.XColumn,S.YColumn,S.XExpression,S.YExpression,X,Y)
  then begin
    if S.LineVisible and NotFirst then
    begin
      P.DrawLine(X0,Y0,X,Y);
      if S.PointVisible then P.DrawPoint(X0,Y0,S.PointType,S.PointSize);
    end;
    if S.PointVisible then P.DrawPoint(X,Y,S.PointType,S.PointSize);
    (FOwner as TDataForm).DrawLeader(X,Y);
  end;
  S.Locked:=true; S.LastLine:=S.Container.Items.Count-1; S.Locked:=false;
  MainForm.CheckPlotScale(P,X,Y);             // expand plot scale if required
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

procedure TDMSerie.GetPoint(Index: Integer; out X, Y: OleVariant);
var xx,yy: TReal; S: TSerie; P: TPlot; D: TRealData;
begin
  X:=0; Y:=0; if not (FOwner is TDataForm) then Exit;
  P:=(FOwner as TDataForm).Plot; S:=P.Series[FIndex];
  D:=S.Container.Items[Index];
  if P.GetDataPoint(D,S.XColumn,S.YColumn,S.XExpression,S.YExpression,xx,yy)
  then begin xx:=0; yy:=0; end; X:=xx; Y:=yy;
end;

{ TDMNotes }

function TDMNotes.Get_LineCount: Integer;
begin Result:=NotesForm.Memo.Lines.Count; end;

function TDMNotes.Get_Lines(Index: Integer): WideString;
begin
  with NotesForm.Memo do
  if (Index>=0) and (Index<Lines.Count)
  then Result:=Lines[Index] else Result:='';
end;

procedure TDMNotes.Set_Lines(Index: Integer; const Value: WideString);
begin
  with NotesForm.Memo do
  if (Index>=0) and (Index<Lines.Count) then Lines[Index]:=Value;
end;

procedure TDMNotes.AddLine(const Line: WideString);
begin NotesForm.Memo.Lines.Add(Line); end;

procedure TDMNotes.DeleteLine(Position: Integer);
begin
  with NotesForm.Memo do
  if (Position>=0) and (Position<Lines.Count) then Lines.Delete(Position);
end;

procedure TDMNotes.InsertLine(Position: Integer; const Line: WideString);
begin
  with NotesForm.Memo do if (Position>=0) and (Position<Lines.Count)
  then Lines.Insert(Position,Line);
end;

procedure TDMNotes.Clear;
begin NotesForm.Memo.Clear; end;

procedure TDMNotes.Load(const FileName: WideString);
begin NotesForm.Memo.Lines.LoadFromFile(FileName); end;

procedure TDMNotes.Save(const FileName: WideString);
begin NotesForm.Memo.Lines.SaveToFile(FileName); end;

function TDMNotes.Get_Text: WideString;
begin Result:=NotesForm.Memo.Lines.Text; end;

procedure TDMNotes.Set_Text(const Value: WideString);
begin NotesForm.Memo.Lines.Text:=Value; end;

initialization
  TAutoObjectFactory.Create(ComServer, TDMExternalApp, Class_DMApplication,
    ciSingleInstance, tmApartment);
  TAutoObjectFactory.Create(ComServer, TDMDocument, Class_DMDocument,
    ciInternal, tmApartment);
  TAutoObjectFactory.Create(ComServer, TDMWorksheet, Class_DMWorksheet,
    ciInternal, tmApartment);
  TAutoObjectFactory.Create(ComServer, TDMContainer, Class_DMContainer,
    ciInternal, tmApartment);
  TAutoObjectFactory.Create(ComServer, TDMPlot, Class_DMPlot,
    ciInternal, tmApartment);
  TAutoObjectFactory.Create(ComServer, TDMAxis, CLASS_DMAxis,
    ciInternal, tmApartment);
  TAutoObjectFactory.Create(ComServer, TDMSerie, CLASS_DMSerie,
    ciInternal, tmApartment);
  TAutoObjectFactory.Create(ComServer, TDMNotes, CLASS_DMNotes,
    ciInternal, tmApartment);
  TAutoObjectFactory.Create(ComServer, TDMInternalApp,
    CLASS_DMInternalApplication, ciInternal, tmApartment);
end.
