{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit BaseDlg;
{$B-,X+}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Plot, StdCtrls, Buttons, Data, ComCtrls;

const
  MinPlotPoints=6;                   {minimal number of points in source plot}
  MinSplinePoints=5;                      {minimal number of points in spline}
  BufSize=100;                            {maximal number of points in spline}
  SplineInts:integer=200;                   {number of spline curve intervals}

type

  TBuf=array[0..BufSize] of extended;          {buffer for spline calculation}

  TBaseLineForm = class(TForm)
    Spline: TContainer;
    ResultData: TContainer;
    ToolPanel: TPanel;
    AddPointSpeedButton: TSpeedButton;
    DeletePointSpeedButton: TSpeedButton;
    ZoomInSpeedButton: TSpeedButton;
    ZoomOutSpeedButton: TSpeedButton;
    SubtractBtn: TBitBtn;
    ClearBtn: TBitBtn;
    OkBtn: TBitBtn;
    SplineBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    PlotPanel: TPanel;
    ResultPlot: TPlot;
    Splitter: TSplitter;
    RescaleBitBtn: TBitBtn;
    StatusBar: TStatusBar;
    MainPlot: TPlot;
    EditSpeedButton: TSpeedButton;
    procedure ClearBtnClick(Sender: TObject);
    procedure MainPlotPaint(Sender: TObject);
    function SplineCompare(Sender: TObject; I1, I2: TData): TCompareResult;
    procedure SubtractBtnClick(Sender: TObject);
    procedure MainPlotMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ToolPanelDblClick(Sender: TObject);
    procedure RescaleBitBtnClick(Sender: TObject);
    procedure PlotModeSpeedButtonClick(Sender: TObject);
    procedure PlotHint(Sender: TObject; H: String);
    procedure MainPlotPointClick(Sender: TObject; Point, Serie: Integer);
  private
    { Private declarations }
    X,F,C: TBuf;                                            {arrays of spline}
    N: integer;  {number of points in spline buffers; <>Spline.Items.Count!!!}
    Min,Max: extended;                                 {range of spline curve}
  public
    { Public declarations }
    function Execute(Plot: TPlot): boolean;
    procedure CopyData;
  end;

var BaseLineForm: TBaseLineForm;

implementation

uses Strings;

{$R *.DFM}

                     {**********************}
{Next 2 procs taken from the book "Numeric methods for PC on BASIC, FORTRAN
& PASCAL languages", by A.E.Mudroff (in russian), item 3.6P. This is VERY
good book, especially from its practic side. All examples are compatible
with Delphi, have simple and comprehensible logic base. Another book used
to design spline algorithms is "A practical guide to computer methods for
engineers", by T.E.Shoup; its more theoretical one.}

{N - max point number (total N+1!); X,F: source function; C: spline koef.    }
procedure CS(N: integer; var X,F,C: TBuf);               {spline coefficients}
var I,J,M: integer; A,B,R: extended; K: TBuf;
begin
  K[1]:=0; C[1]:=0;
  for I:=2 to N do
  begin
    J:=I-1; M:=J-1;
    A:=X[I]-X[J]; B:=X[J]-X[M]; R:=2*(A+B)-B*C[J]; C[I]:=A/R;
    K[I]:=(3*((F[I]-F[J])/A-(F[J]-F[M])/B)-B*K[J])/R;
  end;
  C[N]:=K[N];
  for I:=N-1 downto 2 do C[I]:=K[I]-C[I]*C[I+1];
end;

{N - max numofpoint; X,F,C - as in CS; X1-source X; P-P2 - spline & deriv.   }
{note: don't use bissection, simply scan X!                                  }
procedure SP(N: integer; var X,F,C: TBuf; var X1,P,P1,P2: extended);
var I,J: integer; A,B,D,Q,R: extended;                          {calc splines}
begin
  I:=1; while (X1>X[I]) and (I<>N) do Inc(I);
  J:=I-1; A:=F[J]; B:=X[J]; Q:=X[I]-B;
  R:=X1-B; P:=C[I]; D:=C[I+1];
  B:=(F[I]-A)/Q-(D+2*P)*Q/3; D:=(D-P)/Q*R;
  P1:=B+R*(2*P+D); P2:=2*(P+D); P:=A+R*(B+R*(P+D/3));
end;
                     {**********************}

{ TBaseLineForm }

function TBaseLineForm.Execute(Plot: TPlot): boolean;
{!!! source container must contain realdata!}
var S: string;
begin
  with Plot, Plot.ThisSerie do
  begin
    Result:=false;
    if LastLine-FirstLine<MinPlotPoints then
    begin
      MessageDlg(Format(strBaseLine,[MinPlotPoints]),mtError,[mbCancel],0);
      Exit;
    end;
    if ((XExpression<>'') or (YExpression<>'')) then S:=strSerieEmptyExpr;
    if ((XAxis.Expression<>'') or (YAxis.Expression<>''))
    then S:=S+#13#10+strAxisEmptyExpr;
    if S<>'' then begin MessageDlg(S,mtError,[mbCancel],0); Exit; end;
  end;
  AddPointSpeedButton.Down:=true; RescaleBitBtn.Click;
  MainPlot.MouseMode:=pmNone;                  // reset plot mode and controls
  // MainPlot.Series[0].Assign(Plot.ThisSerie);       {serie 0 - source data!}
  // note! instead of assignment we use selective copy, as in FitDlg!!!
  MainPlot.Series[0].Container:=Plot.ThisSerie.Container;
  MainPlot.Series[0].FirstLine:=Plot.ThisSerie.FirstLine;
  MainPlot.Series[0].LastLine:=Plot.ThisSerie.LastLine;
  MainPlot.Series[0].XColumn:=Plot.ThisSerie.XColumn;
  MainPlot.Series[0].YColumn:=Plot.ThisSerie.YColumn;
  MainPlot.Series[0].XExpression:=Plot.ThisSerie.XExpression;
  MainPlot.Series[0].YExpression:=Plot.ThisSerie.YExpression;
  ////MainPlot.OnHint:=Plot.OnHint; ResultPlot.OnHint:=Plot.OnHint;
  ClearBtn.Click;                  {clear spline points & disable subtractbtn}
  N:=0;        {disable spline curve display; N<>0 means bufs contain spline!}
  ResultData.Clear; ResultPlot.Series[0].LastLine:=-1;       {disable result plot}
  Result:=(ShowModal<>mrCancel)
    and (ResultPlot.Series[0].LastLine>MinPlotPoints);
  ////MainPlot.OnHint:=nil; ResultPlot.OnHint:=nil;
end;

procedure TBaseLineForm.CopyData;
var I: integer; D: TRealData; Dd: TFunction;
begin
  if ((ModalResult=mrOk) or (ModalResult=mrYes)){pressed Ok or spline buttons}
     and (ResultPlot.Series[0].LastLine>MinPlotPoints) then
  begin
    Screen.Cursor:=crHourGlass;
    try                                {if OK,copy result to source container}
      for I:=0 to ResultData.Items.Count-1 do
      begin
        MainPlot.Series[0].Container.ShowProgress(Round(
          I*100/(ResultData.Items.Count-1)));
        D:=MainPlot.Series[0].Container.Items[I+MainPlot.Series[0].FirstLine];
        Dd:=ResultData.Items[I];
        if D is TRealData then if ModalResult=mrOK then D.InsItem(Dd.Y)   {ok}
        else D.InsItem(D.RData[MainPlot.Series[0].YColumn]-Dd.Y);     {spline}
      end;
    finally
      MainPlot.Series[0].Container.Modified:=true; {modify window's container}
      Screen.Cursor:=crDefault;
    end;
  end;
end;

procedure TBaseLineForm.ClearBtnClick(Sender: TObject);
begin
  Spline.Clear; SubtractBtn.Enabled:=false; N:=0;
  MainPlot.Series[1].LastLine:=-1;
  OkBtn.Enabled:=false; SplineBtn.Enabled:=false;
end;

function TBaseLineForm.SplineCompare(Sender: TObject; I1,
  I2: TData): TCompareResult;
begin
  Result:=crEqual;
  try
    if (I1 as TFunction).X>(I2 as TFunction).X then Result:=crGreater;
    if (I1 as TFunction).X<(I2 as TFunction).X then Result:=crLess;
  except
    Result:=crEqual;                {called recoursivly, so does no messages!}
  end;
end;

procedure TBaseLineForm.MainPlotPaint(Sender: TObject);
var I: integer; X1,Xx,P,Pp,P1,P2,H: extended;
begin
  if N<>0 then                                 {spline curve, if ready - N<>0}
  begin
    H:=(Max-Min)/(SplineInts); X1:=Min;                    {calculate delta X}
    for I:=0 to SplineInts-1 do
    begin
      SP(N,X,F,C,X1,P,P1,P2); Xx:=X1+H; SP(N,X,F,C,Xx,Pp,P1,P2);
      MainPlot.DrawLine(x1,p,X1+H,pp); X1:=X1+H;
    end;{for}
  end;{if}
  if (ResultPlot.XAxis.Min<>MainPlot.XAxis.Min) or  // syncronize X axes scale
  (ResultPlot.XAxis.Max<>MainPlot.XAxis.Max) then 
  begin
    ResultPlot.XAxis.Min:=MainPlot.XAxis.Min;
    ResultPlot.XAxis.Max:=MainPlot.XAxis.Max;
    ResultPlot.XAxis.AutoScale:=false;
  end;
end;

procedure TBaseLineForm.SubtractBtnClick(Sender: TObject);
var I: integer; D,D1: TFunction; Dd: TData; X1,P,P1,P2,Y: extended;
begin
  Spline.Sort(0,Spline.Items.Count-1,false);              {sort spline points}
  D:=Spline.Items.First; D1:=Spline.Items.Last; Min:=D.X; Max:=D1.X; {sorted!}
  for I:=0 to Spline.Items.Count-1 do                 {copy points to buffers}
  begin D:=Spline.Items[I]; X[I]:=D.X; F[I]:=D.Y; end;
  N:=Spline.Items.Count-1; CS(N,X,F,C);                    {calc spline coefs}
  MainPlot.Refresh;                                        {show spline curve}
  Screen.Cursor:=crHourGlass;
  try
    ResultData.Clear; ResultPlot.Series[0].LastLine:=-1;
    for I:=MainPlot.Series[0].FirstLine to MainPlot.Series[0].LastLine do
    begin
      D:=TFunction.Create; Dd:=MainPlot.Series[0].Container.Items[I];
      with MainPlot do
      GetDataPoint(Dd,Series[0].XColumn,Series[0].YColumn,'','',X1,Y);
{! TFunction inserted in ResultData, & from source data get polymorphically! }
      D.X:=X1; SP(N,X,F,C,X1,P,P1,P2); D.Y:=Y-P;                 {SUBTRACT...}
      ResultData.Items.Add(D);                                     {add point}
    end;
  finally
    Screen.Cursor:=crDefault;
    ResultPlot.Series[0].LastLine:=ResultData.Items.Count-1;     {update plot}
  end;
  OkBtn.Enabled:=true; SplineBtn.Enabled:=true;
end;

procedure TBaseLineForm.MainPlotMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var rx,ry: extended; F: TFunction; I,ix,iy: integer;
begin
  with MainPlot do if IntToRealCoords(X,Y,rx,ry) and
    (AddPointSpeedButton.Down or DeletePointSpeedButton.Down) then
  begin
    if (Button=mbLeft) and (Spline.Items.Count<BufSize)            {add point}
       and AddPointSpeedButton.Down then
    begin
      F:=TFunction.Create; F.X:=rx; F.Y:=ry; Spline.Items.Add(F);     {insert}
      with Series[1] do
      begin
        Locked:=true; LastLine:=LastLine+1; Locked:=false;   {add w/o refresh}
        Canvas.Pen:=Pen; Canvas.Brush:=Brush;            {setup drawing tools}
        DrawPoint(rx,ry,PointType,PointSize);                 {draw new point}
      end;
    end;
    if (Button=mbRight) or DeletePointSpeedButton.Down then     {remove point}
    for I:=0 to Spline.Items.Count-1 do
    begin
      F:=Spline.Items[I]; RealToIntCoords(F.X,F.Y,ix,iy);      {get intcoords}
      if (abs(x-ix)<=Series[1].PointSize) and (abs(y-iy)<=Series[1].PointSize)
      then begin
        Spline.Items.Delete(I); F.Free;
        Series[1].LastLine:=Series[1].LastLine-1; Break;{once!}
      end;
    end;
    SubtractBtn.Enabled:=Spline.Items.Count>MinSplinePoints;
  end;
  with MainPlot do                           {if axes clicked, set autoscale!}
  begin
    if ClickedAt=claXAxis then XAxis.AutoScale:=true;
    if ClickedAt=claYAxis then YAxis.AutoScale:=true;
  end;
end;

procedure TBaseLineForm.ToolPanelDblClick(Sender: TObject);
begin                    // "secret" ability useful for slow x486 computers...
  SplineInts:=StrToInt(InputBox(strEnter,strSplineInts,IntToStr(SplineInts)));
end;

procedure TBaseLineForm.RescaleBitBtnClick(Sender: TObject);
begin MainPlot.XAxis.AutoScale:=true; MainPlot.YAxis.AutoScale:=true; end;

procedure TBaseLineForm.PlotModeSpeedButtonClick(Sender: TObject);
begin
  with MainPlot do
  if (AddPointSpeedButton.Down or DeletePointSpeedButton.Down)
  then MouseMode:=pmNone else if ZoomInSpeedButton.Down then
  MouseMode:=pmZoom else if ZoomOutSpeedButton.Down then MouseMode:=pmUnZoom
  else if EditSpeedButton.Down then MouseMode:=pmPointEdit;
end;

procedure TBaseLineForm.PlotHint(Sender: TObject; H: String);
begin StatusBar.Panels[0].Text:=H; StatusBar.Update; end;

procedure TBaseLineForm.MainPlotPointClick(Sender: TObject; Point,
  Serie: Integer);
begin if Serie<>1 then Abort; end;             // allow edit ONLY spline data!

end.
