{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit BrwsrWnd;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ToolWin, StdCtrls, ImgList, OleCtrls, SHDocVw, ActnList,
  StdActns, ExtCtrls, Variants;

type
  TBrowserForm = class(TForm)
    CoolBar: TCoolBar;
    StatusBar: TStatusBar;
    ImageList: TImageList;
    WebToolBar: TToolBar;
    FileToolBar: TToolBar;
    BackToolButton: TToolButton;
    ForwardToolButton: TToolButton;
    StopToolButton: TToolButton;
    RefreshToolButton: TToolButton;
    HomeToolButton: TToolButton;
    OpenToolButton: TToolButton;
    NavigateToolButton: TToolButton;
    ToolButton8: TToolButton;
    UrlComboBox: TComboBox;
    ActionList: TActionList;
    NavigateAction: TAction;
    BackAction: TAction;
    ForwardAction: TAction;
    StopAction: TAction;
    RefreshAction: TAction;
    HomeAction: TAction;
    ProgressBar: TProgressBar;
    OpenAction: TFileOpen;
    InsertAction: TAction;
    InsertToolButton: TToolButton;
    procedure FormShow(Sender: TObject);
    procedure FileToolBarResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OpenActionAccept(Sender: TObject);
    procedure BackActionExecute(Sender: TObject);
    procedure ForwardActionExecute(Sender: TObject);
    procedure NavigateActionExecute(Sender: TObject);
    procedure RefreshActionExecute(Sender: TObject);
    procedure StopActionExecute(Sender: TObject);
    procedure HomeActionExecute(Sender: TObject);
    procedure UrlComboBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormEndDock(Sender, Target: TObject; X, Y: Integer);
    procedure FormHide(Sender: TObject);
    procedure InsertActionExecute(Sender: TObject);
    procedure InsertActionUpdate(Sender: TObject);
  private
    { Private declarations }
  protected
    { Protected declarations }
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public declarations }
    WebBrowser: TWebBrowser;
    DMApplication: variant;
    procedure BeforeNavigate2(Sender: TObject; const pDisp: IDispatch;
      var URL, Flags, TargetFrameName, PostData,
      Headers: OleVariant; var Cancel: WordBool);
    procedure CommandStateChange(Sender: TObject; Command: Integer;
      Enable: WordBool);
    procedure StatusTextChange(Sender: TObject;
      const Text: WideString);
    procedure ProgressChange(Sender: TObject; Progress,
      ProgressMax: Integer);
    procedure DownloadComplete(Sender: TObject);
  end;

var BrowserForm: TBrowserForm;

implementation

uses ActiveX, ComObj, Automate, AppDlg, StrDlg, MainWnd, ScrptWnd, MSHTML;

{$R *.DFM}

type
  PDOCHOSTUIINFO = ^TDOCHOSTUIINFO;
  TDOCHOSTUIINFO = record
    cbSize: ULONG;
    dwFlags: DWORD;
    dwDoubleClick: DWORD;
    chHostCss: POLESTR;
    chHostNS: POLESTR;
  end;

  // IDocHostUIHandler used to expose DM object model to embedded scripts
  IDocHostUIHandler=interface(IUnknown)
    ['{bd3f23c0-d43e-11cf-893b-00aa00bdce1a}']
    function ShowContextMenu(const dwID: DWORD; const ppt: PPOINT;
      const pcmdtReserved: IUnknown;
      const pdispReserved: IDispatch): HRESULT; stdcall;
    function GetHostInfo(var pInfo: TDOCHOSTUIINFO): HRESULT; stdcall;
    function ShowUI(const dwID: DWORD;
      const pActiveObject: IOleInPlaceActiveObject;
      const pCommandTarget: IOleCommandTarget; const pFrame: IOleInPlaceFrame;
      const pDoc: IOleInPlaceUIWindow): HRESULT; stdcall;
    function HideUI: HRESULT; stdcall;
    function UpdateUI: HRESULT; stdcall;
    function EnableModeless(const fEnable: BOOL): HRESULT; stdcall;
    function OnDocWindowActivate(const fActivate: BOOL): HRESULT; stdcall;
    function OnFrameWindowActivate(const fActivate: BOOL): HRESULT; stdcall;
    function ResizeBorder(const prcBorder: PRECT;
      const pUIWindow: IOleInPlaceUIWindow;
      const fRameWindow: BOOL): HRESULT; stdcall;
    function TranslateAccelerator(const lpMsg: PMSG;
      const pguidCmdGroup: PGUID;
      const nCmdID: DWORD): HRESULT; stdcall;
    function GetOptionKeyPath(var pchKey: POLESTR;
      const dw: DWORD): HRESULT; stdcall;
    function GetDropTarget(const pDropTarget: IDropTarget;
      out ppDropTarget: IDropTarget): HRESULT; stdcall;
    function GetExternal(out ppDispatch: IDispatch): HRESULT; stdcall;
    function TranslateUrl(const dwTranslate: DWORD; const pchURLIn: POLESTR;
      var ppchURLOut: POLESTR): HRESULT; stdcall;
    function FilterDataObject(const pDO: IDataObject;
      out ppDORet: IDataObject): HRESULT; stdcall;
  end;

  TDMWebBrowser=class(TWebBrowser,IDocHostUIHandler)
  private
    { Private declarations }
    function ShowContextMenu(const dwID: DWORD; const ppt: PPOINT;
      const pcmdtReserved: IUnknown;
      const pdispReserved: IDispatch): HRESULT; stdcall;
    function GetHostInfo(var pInfo: TDOCHOSTUIINFO): HRESULT; stdcall;
    function ShowUI(const dwID: DWORD;
      const pActiveObject: IOleInPlaceActiveObject;
      const pCommandTarget: IOleCommandTarget; const pFrame: IOleInPlaceFrame;
      const pDoc: IOleInPlaceUIWindow): HRESULT; stdcall;
    function HideUI: HRESULT; stdcall;
    function UpdateUI: HRESULT; stdcall;
    function EnableModeless(const fEnable: BOOL): HRESULT; stdcall;
    function OnDocWindowActivate(const fActivate: BOOL): HRESULT; stdcall;
    function OnFrameWindowActivate(const fActivate: BOOL): HRESULT; stdcall;
    function ResizeBorder(const prcBorder: PRECT;
      const pUIWindow: IOleInPlaceUIWindow;
      const fRameWindow: BOOL): HRESULT; stdcall;
    function TranslateAccelerator(const lpMsg: PMSG;
      const pguidCmdGroup: PGUID;
      const nCmdID: DWORD): HRESULT; stdcall;
    function GetOptionKeyPath(var pchKey: POLESTR;
      const dw: DWORD): HRESULT; stdcall;
    function GetDropTarget(const pDropTarget: IDropTarget;
      out ppDropTarget: IDropTarget): HRESULT; stdcall;
    function GetExternal(out ppDispatch: IDispatch): HRESULT; stdcall;
    function TranslateUrl(const dwTranslate: DWORD; const pchURLIn: POLESTR;
      var ppchURLOut: POLESTR): HRESULT; stdcall;
    function FilterDataObject(const pDO: IDataObject;
      out ppDORet: IDataObject): HRESULT; stdcall;
  end;

{ TDMWebBrowser }

function TDMWebBrowser.EnableModeless(const fEnable: BOOL): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.FilterDataObject(const pDO: IDataObject;
  out ppDORet: IDataObject): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.GetDropTarget(const pDropTarget: IDropTarget;
  out ppDropTarget: IDropTarget): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.GetExternal(out ppDispatch: IDispatch): HRESULT;
begin ppDispatch:=BrowserForm.DMApplication; Result:=S_OK; end;

function TDMWebBrowser.GetHostInfo(var pInfo: TDOCHOSTUIINFO): HRESULT;
begin Result:=S_OK; end;

function TDMWebBrowser.GetOptionKeyPath(var pchKey: POLESTR;
  const dw: DWORD): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.HideUI: HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.OnDocWindowActivate(const fActivate: BOOL): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.OnFrameWindowActivate(
  const fActivate: BOOL): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.ResizeBorder(const prcBorder: PRECT;
  const pUIWindow: IOleInPlaceUIWindow; const fRameWindow: BOOL): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.ShowContextMenu(const dwID: DWORD;
  const ppt: PPOINT; const pcmdtReserved: IUnknown;
  const pdispReserved: IDispatch): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.ShowUI(const dwID: DWORD;
  const pActiveObject: IOleInPlaceActiveObject;
  const pCommandTarget: IOleCommandTarget; const pFrame: IOleInPlaceFrame;
  const pDoc: IOleInPlaceUIWindow): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.TranslateAccelerator(const lpMsg: PMSG;
  const pguidCmdGroup: PGUID; const nCmdID: DWORD): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.TranslateUrl(const dwTranslate: DWORD;
  const pchURLIn: POLESTR; var ppchURLOut: POLESTR): HRESULT;
begin Result:=E_NOTIMPL; end;

function TDMWebBrowser.UpdateUI: HRESULT;
begin
  Result:=E_NOTIMPL;
(* manipulations with focus here solve some problem. Namely, if comment
the rest of code, docked webbrowser will NOT grab focus when clicked - only
if dock handler clicked. This is strange, isn't it? But I have no answer...
This code was introduced in the [\Test\MemoBorder\] project. Lately it was
found that it causes errors with message "unable to focus invisible window" *)

(*  code moved to MainForm.ApplicationEvents.OnMessage...
  if (not BrowserForm.Floating) and (BrowserForm.ControlAtPos(// XP/IE6 bugfix
  BrowserForm.ScreenToClient(Mouse.CursorPos),false,true)=Self)
  and (Screen.ActiveControl<>BrowserForm.WebBrowser) and BrowserForm.Visible
  and (BrowserForm.ComponentState<>[csDestroying])   // when error in onUnload
  then BrowserForm.WebBrowser.SetFocus;
*)
end;

{ TBrowserForm }

procedure TBrowserForm.CreateParams(var Params: TCreateParams);
begin
  if UseTaskBar then FormStyle:=fsNormal; inherited;  
  if UseTaskBar then Params.ExStyle:=Params.ExStyle or WS_EX_APPWINDOW;
end;

procedure TBrowserForm.FormShow(Sender: TObject);
begin           // correct (as in ScrptWnd!) and restore browser after docking
  UrlComboBox.Top:=2; FormEndDock(nil,nil,0,0);
  if (Parent is TPanel) and (Parent.VisibleDockClientCount=1) and
  MainForm.DocksLoaded then MainForm.ShowDockPanel(Parent as TPanel, true);
end;

procedure TBrowserForm.FormHide(Sender: TObject);           // docking support
begin
  if (Parent is TPanel) and (Parent.VisibleDockClientCount=0)
  then MainForm.ShowDockPanel(Parent as TPanel, false);
end;

procedure TBrowserForm.FileToolBarResize(Sender: TObject);
begin UrlComboBox.Width:=FileToolBar.Width-95; end;

procedure TBrowserForm.FormCreate(Sender: TObject);
begin
  ProgressBar.Top:=StatusBar.Top+2;   // correct height changes as in MainForm
  DMApplication:=TDMInternalApp.Create as IDispatch;
  WebBrowser:=TDMWebBrowser.Create(Self);
  InsertControl(WebBrowser);
  with WebBrowser do
  begin
    Align:=alClient;
    Visible:=true;
    OnBeforeNavigate2:=BeforeNavigate2;
    OnCommandStateChange:=CommandStateChange;
    OnStatusTextChange:=StatusTextChange;
    OnProgressChange:=ProgressChange;
    OnDownloadComplete:=DownloadComplete;
    TabStop:=true;
    Navigate('about:blank');
  end;
  ActiveControl:=WebBrowser;
end;

procedure TBrowserForm.DownloadComplete(Sender: TObject);
var Doc: IHTMLDocument2; Element: IHTMLElement;
begin
  if WebBrowser.Document=nil then Exit;                      // many checks...
  WebBrowser.Document.QueryInterface(IID_IHTMLDocument2, Doc);
  if Doc=nil then Exit;
  Element:=Doc.body; if Element=nil then Exit;
  //if WBActive then Element.style.borderStyle:=''
  {else }Element.style.borderStyle:='none';               // always frameless!
end;

procedure TBrowserForm.BeforeNavigate2(Sender: TObject;
  const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
var S,Ss: string;
begin
  S:=URL;
  if Pos('DM:',UpperCase(S))>0 then         // if pseudolink - call DM action!
  begin
    Cancel:=true; Ss:=Copy(S,4,length(S)-3); DMApplication.ExecuteCommand(Ss);
  end else
  begin                                            // else add URL to URLCombo
    with UrlComboBox.Items do
    if (S<>'') and (IndexOf(S)=-1) then Insert(0, S);
    UrlComboBox.Text:=S;
  end;
end;

procedure TBrowserForm.OpenActionAccept(Sender: TObject);
begin WebBrowser.Navigate(OpenAction.Dialog.Filename); end;

procedure TBrowserForm.BackActionExecute(Sender: TObject);
begin WebBrowser.GoBack; end;

procedure TBrowserForm.ForwardActionExecute(Sender: TObject);
begin WebBrowser.GoForward; end;

procedure TBrowserForm.CommandStateChange(Sender: TObject;
  Command: Integer; Enable: WordBool);
//var B: boolean;
begin
  if Command=CSC_NAVIGATEFORWARD then ForwardAction.Enabled:=Enable;
  if Command=CSC_NAVIGATEBACK then BackAction.Enabled:=Enable;
  {if Command=integer(CSC_UPDATECOMMANDS) then
  begin
    B:=WebBrowser.Busy;
    StopAction.Enabled:=B; RefreshAction.Enabled:=not B;
    HomeAction.Enabled:=not B; OpenAction.Enabled:=not B;
    NavigateAction.Enabled:=not B;
  end;}
end;

procedure TBrowserForm.NavigateActionExecute(Sender: TObject);
var URL: string;
begin
  if Floating then
     if UrlComboBox.Text<>'' then WebBrowser.Navigate(UrlComboBox.Text) else
  else if InputString(NavigateAction.Caption,UrlComboBox.Items,URL) then
     WebBrowser.Navigate(URL);
end;

procedure TBrowserForm.RefreshActionExecute(Sender: TObject);
begin WebBrowser.Refresh; end;

procedure TBrowserForm.StopActionExecute(Sender: TObject);
begin WebBrowser.Stop; end;

procedure TBrowserForm.HomeActionExecute(Sender: TObject);
begin WebBrowser.Navigate(AppPropsForm.HomepageEdit.Text); end;

procedure TBrowserForm.StatusTextChange(Sender: TObject;
  const Text: WideString);
begin
  if Floating then StatusBar.Panels[StatusBar.Panels.Count-1].Text:=Text
  else MainForm.DisplayHint(Text);
end;            {^StatusBar.Panels[StatusBar.Panels.Count-1].Text:= BAD!}

procedure TBrowserForm.ProgressChange(Sender: TObject; Progress,
  ProgressMax: Integer);
var P: TProgressBar;
begin
  if Floating then P:=ProgressBar else P:=MainForm.ProgressBar;
  if (Progress=-1) or (ProgressMax=0) then P.Hide else
  begin P.Visible:=true; P.Position:=Round(Progress*100/ProgressMax); end;
end;

procedure TBrowserForm.UrlComboBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin if Key=VK_RETURN then NavigateAction.Execute; end;

procedure TBrowserForm.FormEndDock(Sender, Target: TObject; X, Y: Integer);
begin
{NOTE!!! browser control become corrupted when docking even if
MDIChildCount>0, so we need to reload page. But if no MDI children opened,
UrlComboBox ALSO is cleared (like memos), and we have to navigate to blank...
WARNING!!! because this event fired any time when floating window is moved,
it is necessary to check whether WebBrowser.LocationName is empty!}
  if WebBrowser.LocationName='' then
  if UrlComboBox.Text='' then WebBrowser.Navigate('about:blank')
  else WebBrowser.Navigate(UrlComboBox.Text);
end;

procedure TBrowserForm.InsertActionExecute(Sender: TObject);
var Document: IHtmlDocument2; V: Variant;
begin
  Show;                 // else this function produces error "cannot focus..."
  WebBrowser.Navigate('about:blank');
  while WebBrowser.ReadyState<READYSTATE_INTERACTIVE
  do Application.ProcessMessages;
  Document:=WebBrowser.Document as IHtmlDocument2;
  V:=VarArrayCreate([0, 0], varVariant);
  V[0]:=ScriptingForm.Memo.Text;
  Document.Write(PSafeArray(TVarData(v).VArray));
  Document.Close;
end;

procedure TBrowserForm.InsertActionUpdate(Sender: TObject);
begin InsertAction.Enabled:=ScriptingForm.Memo.Text<>''; end;

initialization
  OleInitialize(nil);
finalization
  OleUninitialize;
end.
