{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit CalcDlg;
{$B-}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Common, Parser, Buttons, Grids, ClipBrd;

type
  TCalculatorForm = class(TForm)
    ExpressionComboBox: TComboBox;
    ParametersStringGrid: TStringGrid;
    EvaluateBitBtn: TBitBtn;
    HelpBitBtn: TBitBtn;
    ParametersLabel: TLabel;
    ExpressionLabel: TLabel;
    ResultLabel: TLabel;
    ResultEdit: TEdit;
    CopyCheckBox: TCheckBox;
    OkBitBtn: TBitBtn;
    TextSpeedButton: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ExpressionComboBoxChange(Sender: TObject);
    procedure OkBitBtnClick(Sender: TObject);
    procedure EvaluateBitBtnClick(Sender: TObject);
    procedure TextSpeedButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Parameters: TRealArray;
    X,Y: TReal;                                 // additional parameters - X,Y
    Parser: TMathParser;
  end;

var CalculatorForm: TCalculatorForm;

implementation

uses Strings, TextDlg;

{$R *.DFM}

procedure TCalculatorForm.FormCreate(Sender: TObject);
var I: integer;
begin
  Parser:=TMathParser.Create;
  with Parser do
  begin  // note that we allow here cx/cy (for plot calc.), but not num/absnum
    Init(90); AddGonio; AddLogic; AddMath; AddMisc; AddSpec;
    AddStdParams(@Parameters);  // also DPR (not mainform!!!) must add table()
    AddObject(@X,'CX',tfp_realvar); AddObject(@Y,'CY',tfp_realvar);
  end;
  ParametersStringGrid.Cells[0,0]:=WordStr(strParametersStringGrid,1);
  ParametersStringGrid.Cells[1,0]:=WordStr(strParametersStringGrid,2);
  for I:=1 to MaxCols do with ParametersStringGrid do
  begin Cells[0,I]:=char(byte('A')+I-1); Cells[1,I]:='0'; end;
end;

procedure TCalculatorForm.FormDestroy(Sender: TObject);
begin Parser.Free; end;

procedure TCalculatorForm.ExpressionComboBoxChange(Sender: TObject);
begin EvaluateBitBtn.Enabled:=ExpressionComboBox.Text<>''; end;

procedure TCalculatorForm.OkBitBtnClick(Sender: TObject);
begin Hide; end;

procedure TCalculatorForm.EvaluateBitBtnClick(Sender: TObject);
var B: boolean; I: integer;
begin
  if Sender=EvaluateBitBtn then           // if called externally, sender=nil!
  for I:=1 to MaxCols do if ParametersStringGrid.Cells[1,I]<>''
  then Parameters[I]:=StrToFloat(ParametersStringGrid.Cells[1,I]) else
  else for I:=1 to MaxCols do          // update grid with external parameters
  ParametersStringGrid.Cells[1,I]:=FloatToStr(Parameters[I]);
  ResultEdit.Text:=FloatToStr(Parser.Parse(ExpressionComboBox.Text));
  if CopyCheckBox.Checked then Clipboard.AsText:=ResultEdit.Text;
  B:=true;       {check expression for duplication and add to the top of list}
  with ExpressionComboBox.Items do for I:=0 to Count-1 do
  if Strings[I]=ExpressionComboBox.Text then B:=false;
  if B and (ExpressionComboBox.Text<>'')
  then ExpressionComboBox.Items.Insert(0, ExpressionComboBox.Text);
end;

procedure TCalculatorForm.TextSpeedButtonClick(Sender: TObject);
var S: string;
begin
  S:=ExpressionComboBox.Text;
  if InputText(S,ExpressionComboBox) then ExpressionComboBox.Text:=S;
end;

end.
