{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
program DM2000;
{$B-}
{%ToDo 'DM2000.todo'}

uses
  Forms,
  Controls,
  Printers,
  SysUtils,
  Windows,
  Dialogs,
  Registry,
  ActnMan,
  ActnList,
  MainWnd in 'MainWnd.pas' {MainForm},
  NotesWnd in 'NotesWnd.pas' {NotesForm},
  Strings in 'Strings.pas',
  BrwsrWnd in 'BrwsrWnd.pas' {BrowserForm},
  DataWnd in 'DataWnd.pas' {DataForm},
  ScrptWnd in 'ScrptWnd.pas' {ScriptingForm},
  CalcDlg in 'CalcDlg.pas' {CalculatorForm},
  Splash in 'Splash.pas' {SplashForm},
  AppDlg in 'AppDlg.pas' {AppPropsForm},
  WorksDlg in 'WorksDlg.pas' {WorksheetPropsForm},
  PageDlg in '..\Library\PageDlg.pas' {PageSetupForm},
  AxisDlg in '..\Library\AxisDlg.pas' {AxisPropsForm},
  SerieDlg in '..\Library\SerieDlg.pas' {SeriePropsForm},
  TestDlg in 'TestDlg.pas' {SpeedTestForm},
  LMOptDlg in 'LMOptDlg.pas' {NLSFOptionsForm},
  StrDlg in 'StrDlg.pas' {InputStringForm},
  TransDlg in 'TransDlg.pas' {TransformationForm},
  BaseDlg in 'BaseDlg.pas' {BaseLineForm},
  DigDlg in 'DigDlg.pas' {DigitizerForm},
  FitDlg in 'FitDlg.pas' {FittingForm},
  IntDlg in 'IntDlg.pas' {InputIntForm},
  LFitDlg in 'LFitDlg.pas' {LinearFitForm},
  LMFitDlg in 'LMFitDlg.pas' {NLSFForm},
  Process in 'Process.pas',
  SelDlg in 'SelDlg.pas' {SelectForm},
  DM_TLB in 'DM_TLB.pas',
  Automate in 'Automate.pas',
  MonWnd in 'MonWnd.pas' {MonitorForm},
  SetupDlg in 'SetupDlg.pas' {SetupForm},
  SerieWnd in 'SerieWnd.pas' {SerieMgrForm},
  FCopyDlg in 'FCopyDlg.pas' {FunctionCopyForm};

{$E exe}

{$R *.TLB}

{$R *.res}

var HMutex: THandle;
const MutexName='DM2000Mutex';

begin
  CheckOldDesktop;                          // may halt DM2000 and launch DMW9
  Application.Initialize;
  Application.Title:='Data Master';
  Application.HelpFile:='dm2000.chm';
  HMutex:=OpenMutex(MUTEX_ALL_ACCESS, False, MutexName);
  if HMutex=0 then                         // no mutex => this is 1st instance
  begin
    HMutex:=CreateMutex(Nil, False, MutexName);
    if HMutex=0 then MessageDlg('Can"t create mutex!',mtError,[mbCancel],0);
    SplashForm:=TSplashForm.Create(Application);    // don't become main form!
  end else begin ReleaseMutex(HMutex); HMutex:=0; end;   // release if not 1st
  MakeForm(TMainForm, MainForm);
  MakeForm(TAppPropsForm, AppPropsForm);
  AppPropsForm.HelpBitBtn.OnClick:=MainForm.HelpButtonClick;
  with TRegistry.Create do     // read this flag separately because it is used
  try                                      // prior to MainForm.ReadRegistry!!
    UseTaskBar:=OpenKeyReadOnly(DataMasterRegKey+'\'+AppPropsForm.ClassName)
    and ValueExists(AppPropsForm.UseTaskbarCheckBox.Name)
    and ReadBool(AppPropsForm.UseTaskbarCheckBox.Name);
  finally
    Free;
  end;
  if UseTaskBar then     // not in TMainForm.CreateParams (after AppPropsForm)
  SetWindowLong(MainForm.Handle, GWL_EXSTYLE,
  GetWindowLong(MainForm.Handle, GWL_EXSTYLE) or WS_EX_APPWINDOW);
  MakeForm(TCalculatorForm, CalculatorForm);
  CalculatorForm.HelpBitBtn.OnClick:=MainForm.HelpButtonClick;
  MainForm.InitParser(CalculatorForm.Parser); // call after container creation
  SplashProgress(Format(strCreateForm,[TNotesForm.ClassName]));
  NotesForm:=TNotesForm.Create(MainForm);  // parent=mainform to remember dock
  SplashProgress(Format(strCreateForm,[TScriptingForm.ClassName]));
  ScriptingForm:=TScriptingForm.Create(MainForm);
  SplashProgress(Format(strCreateForm,[TBrowserForm.ClassName]));
  BrowserForm:=TBrowserForm.Create(MainForm);
  SplashProgress(Format(strCreateForm,[TSerieMgrForm.ClassName]));
  SerieMgrForm:=TSerieMgrForm.Create(MainForm);
  MakeForm(TWorksheetPropsForm, WorksheetPropsForm);
  WorksheetPropsForm.HelpBitBtn.OnClick:=MainForm.HelpButtonClick;
  MakeForm(TSeriePropsForm, SeriePropsForm);
  SeriePropsForm.WorksheetComboBox.OnChange:=MainForm.WorksheetComboBoxChange;
  SeriePropsForm.HelpBitBtn.OnClick:=MainForm.HelpButtonClick;
  MakeForm(TAxisPropsForm, AxisPropsForm);
  AxisPropsForm.HelpBitBtn.OnClick:=MainForm.HelpButtonClick;
  MakeForm(TPageSetupForm, PageSetupForm);
  PageSetupForm.HelpBitBtn.OnClick:=MainForm.HelpButtonClick;
  if Printer.Printers.Count=0 then    // else errors if no printers installed!
  begin
    MainForm.PageSetupAction.Enabled:=false;
    MainForm.PrintSetupAction.Enabled:=false;
  end;
  MakeForm(TNLSFOptionsForm, NLSFOptionsForm);
  NLSFOptionsForm.HelpBitBtn.OnClick:=MainForm.HelpButtonClick;
  MakeForm(TMonitorForm, MonitorForm);
  MainForm.ReadRegistry;
  MainForm.ReadLayout;
  if not AppPropsForm.UseDockingCheckBox.Checked then       // disable docking
  begin
    NotesForm.DragMode:=dmManual; NotesForm.DragKind:=dkDrag;
    ScriptingForm.DragMode:=dmManual; ScriptingForm.DragKind:=dkDrag;
    BrowserForm.DragMode:=dmManual; BrowserForm.DragKind:=dkDrag;
    SerieMgrForm.DragMode:=dmManual; SerieMgrForm.DragKind:=dkDrag;
  end;
  with TRegistry.Create do            // disable BrowserForm docking for MSIE4
  try
    RootKey:=HKEY_LOCAL_MACHINE;
    if OpenKeyReadOnly('\SOFTWARE\Microsoft\Internet Explorer') and
    (StrToInt(Copy(ReadString('Version'),1,1))<5) then
    with BrowserForm do begin DragMode:=dmManual; DragKind:=dkDrag; end;
  finally
    Free;
  end;
  with MainForm do begin Show; DocksLoaded:=true; end; // save docks positions
  if UseTaskBar then ShowWindow(Application.Handle, SW_HIDE);
  if Assigned(SplashForm) then SplashForm.Show;        // bring to front again
  MainForm.ReadParamFiles;              // AFTER MainForm.Show to fix crash in
  SplashProgress('');                   // ReadParamFiles when loading HTML
  if Assigned(SplashForm) then
  begin SplashForm.AlphaBlend:=true; SplashForm.Timer.Enabled:=true; end;
  Application.Run;
  if HMutex<>0 then CloseHandle(HMutex);          // close mutex if was opened
end.
