{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit DataWnd;
{$B-}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ActnList, ExtCtrls, Plot, Grids, WorkGrid, Data, Common, Automate;

const MaxRecordingSeries=100;
type
  TDataFormView=(dfvWorksheet,dfvPlot,dfvBoth);
  TRecordingSeries=packed array[0..MaxRecordingSeries] of boolean;
  TDataForm = class(TForm)
    Container: TContainer;
    PlotPanel: TPanel;
    Worksheet: TWorksheet;
    Splitter: TSplitter;
    Plot: TPlot;
    UndoContainer: TContainer;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure WorksheetDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure WorksheetMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ContainerChanged(Sender: TObject);
    function ContainerCompare(Sender: TObject; I1,
      I2: TData): TCompareResult;
    function ContainerInitItem(Sender: TObject): TData;
    procedure PlotHint(Sender: TObject; H: String);
    procedure ContainerProgress(Sender: TObject; P: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PlotDblClick(Sender: TObject);
    procedure PlotPaint(Sender: TObject);
    procedure PlotError(Sender: TObject; H: String);
    procedure PlotPointClick(Sender: TObject; Point, Serie: Integer);
    procedure PlotMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PlotSelectionChange(Sender: TObject);
    procedure WorksheetEnter(Sender: TObject);
    procedure WorksheetExit(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
  private
    { Private declarations }
    FXColumn,FYColumn: longint;        // X,Y columns - for various operations
    FRecordingSeries: TRecordingSeries;     // used to mark "recording" series
    FLastOperation: TObject;        // remembers last operation for group undo
    procedure SetXColumn(X: longint);    // automatically update column header
    procedure SetYColumn(Y: longint);
    function GetLabel(I: integer): string;
    procedure SetLabel(I: integer; const Value: string);
    function GetRecording: boolean;
    procedure SetRecording(const Value: boolean);
    function GetViewState: TDataFormView;
    procedure SetViewState(const Value: TDataFormView);
    function GetIsRecordingSerie(I: integer): boolean;
    procedure SetIsRecordingSerie(I: integer; const Value: boolean);
  public
    { Public declarations }
    DataFormat: TFormatArray;                        // format of real numbers
    FormID: TDateTime;                   // unique value used in desktop files
    DMDocument: TDMDocument;        // reference to Document automation object
    procedure DrawLeader(x,y: extended);  // last point marker for rec. series
    procedure Scroll(Line: integer);            // 0<=Line<Container.ItemCount
    procedure SaveUndoInfo(Op: TObject);         // encapsulate undo algorithm
    procedure Undo;
    function CanUndo: boolean;
    function UndoHint: string;
    property XColumn: longint read FXColumn write SetXColumn;
    property YColumn: longint read FYColumn write SetYColumn;
    property Labels[I: integer]: string read GetLabel write SetLabel;
    property Recording: boolean read GetRecording write SetRecording;
    property ViewState: TDataFormView read GetViewState write SetViewState;
    property IsRecordingSerie[I: integer]: boolean read GetIsRecordingSerie
      write SetIsRecordingSerie;
  end;

implementation

uses MainWnd, Strings, AppDlg, StdCtrls;

{$R *.DFM}

{ TDataForm }

procedure TDataForm.DrawLeader(x,y: extended);
var ix,iy,ls: integer;
begin
  ls:=AppPropsForm.LeaderSpinEdit.Value;
  with Plot do
  begin
    if RealToIntCoords(x,y,ix,iy) then Exit;
    Canvas.Pen:=Pen;       // DrawLeader() called not only from TPlot.Paint()!
    Canvas.MoveTo(ix-ls,iy); Canvas.LineTo(ix+ls,iy);
    Canvas.MoveTo(ix,iy-ls); Canvas.LineTo(ix,iy+ls);
  end;
end;

function TDataForm.GetIsRecordingSerie(I: integer): boolean;
begin
  if (I>=0) and (I<=MaxRecordingSeries) then Result:=FRecordingSeries[I]
  else Result:=false;
end;

procedure TDataForm.SetIsRecordingSerie(I: integer; const Value: boolean);
begin
  if (I>=0) and (I<=MaxRecordingSeries) and (FRecordingSeries[I]<>Value) then
  begin
    FRecordingSeries[I]:=Value; Plot.Invalidate;         // these are changed!
    MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
  end;
end;

function TDataForm.GetLabel(I: integer): string;               // labels: 1..N
begin
  if (I>0) and (I<=Worksheet.Header.Count)                           // check!
  then Result:=Worksheet.Header[I-1] else Result:='';   
end;

procedure TDataForm.SetLabel(I: integer; const Value: string);
begin
  if (I>0) and (I<=Worksheet.Header.Count) and
  (Worksheet.Header[I-1]<>Value)
  then begin Worksheet.Header[I-1]:=Value; MainForm.UpdateSerieList; end;
end;

procedure TDataForm.SetXColumn(X: Integer);
begin
  if (X<>FXColumn) and (X>0) and (X<=MaxCols)
  then begin FXColumn:=X; Worksheet.Invalidate; end;
end;

procedure TDataForm.SetYColumn(Y: Integer);
begin
  if (Y<>FYColumn) and (Y>0) and (Y<=MaxCols)
  then begin FYColumn:=Y; Worksheet.Invalidate; end;
end;

function TDataForm.GetRecording: boolean; // indicate window "recording" state
begin Result:=BorderWidth>0; end;

procedure TDataForm.SetRecording(const Value: boolean);
begin   // note that we can't make permanently Color=clRed due to red flickers
  if Value=Recording then Exit else
  if Value then begin Color:=clHighlight; BorderWidth:=3; end
  else begin Color:=clWindow; BorderWidth:=0; end;
end;
          
function TDataForm.GetViewState: TDataFormView;
begin
  if Worksheet.Visible
  then if PlotPanel.Visible then Result:=dfvBoth else Result:=dfvWorksheet
  else Result:=dfvPlot;
end;

procedure TDataForm.SetViewState(const Value: TDataFormView);
begin                                     // this code ported from old DMW8,9!
  if Value=dfvBoth then Splitter.Align:=alLeft else Splitter.Align:=alNone;
  if Value=dfvWorksheet then Worksheet.Align:=alClient
  else begin Worksheet.Align:=alLeft; Worksheet.Width:=Splitter.Left; end;
  Splitter.Visible:=Value=dfvBoth;
  PlotPanel.Visible:=Value<>dfvWorksheet;
  Worksheet.Visible:=Value<>dfvPlot;
  if Value=dfvWorksheet then Worksheet.SetFocus;          // set correct focus
  if Value=dfvPlot then PlotPanel.SetFocus;
end;

procedure TDataForm.Scroll(Line: integer);
begin
  if (Line>=0) and (Line<Container.Items.Count) then
  begin
    Line:=Line+Worksheet.FixedRows;
    if Line>Worksheet.RowCount-Worksheet.VisibleRowCount
    then Line:=Worksheet.RowCount-Worksheet.VisibleRowCount;
    Worksheet.TopRow:=Line;
  end;
end;

procedure TDataForm.SaveUndoInfo(Op: TObject);
begin
  if AppPropsForm.GroupUndoCheckBox.Checked and (FLastOperation=Op)
  then Exit else FLastOperation:=Op;
  MainForm.StatusImageList.Tag:=8; MainForm.DisplayHint(strSavingUndo);
  try
    UndoContainer.Assign(Container);   // may be longer than operation itself!
  finally
    MainForm.StatusImageList.Tag:=0; MainForm.Progressbar.Hide;
    MainForm.DisplayHint('');
  end;
end;

procedure TDataForm.Undo;
begin
  MainForm.StatusImageList.Tag:=8; MainForm.DisplayHint(strWorking);
  try
    Container.Assign(UndoContainer);
    UndoContainer.Clear; FLastOperation:=nil;
  finally
    MainForm.StatusImageList.Tag:=0; MainForm.DisplayHint('');
  end;
end;

function TDataForm.CanUndo: boolean;
begin Result:=UndoContainer.Items.Count>0; end;

function TDataForm.UndoHint: string;
begin
  Result:='';
  if FLastOperation is TAction
  then Result:=' ('+(FLastOperation as TAction).Caption+')';
  if FLastOperation=MainForm then Result:=' (Automation client)';
  if Pos('&',Result)<>0 then Delete(Result,Pos('&',Result),1); {remove hotkey}
end;

/////// Form handlers /////////////
procedure TDataForm.FormCreate(Sender: TObject);
var I: integer;
begin
  for I:=1 to MaxCols do with DataFormat[I] do
  begin Width:=7; Decimals:=4; FType:=ffGeneral; end;
  XColumn:=1; YColumn:=2; FormID:=Now;           // formID is a unique number!
  DMDocument:=TDMDocument.Create(Self);   // other stuff moved to Automate.pas
end;

procedure TDataForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if Container.Modified then
  case MessageDlg(Format(strCloseQuery, [Container.Filename]),
       mtWarning, mbYesNoCancel, 0) of
    mrYes: MainForm.SaveAction.Execute;
    mrCancel: CanClose:=false;
  end;
  if DMDocument.IsExternalRefs and (MessageDlg(Caption+CRLF+strExtRefsAlert,
  mtWarning,mbOkCancel,0)=mrCancel) then CanClose:=false;
end;

procedure TDataForm.FormClose(Sender: TObject; var Action: TCloseAction);
var I,J: integer;
begin
  Action:=caFree;                              {default behavior is Minimize!}
  for I:=0 to MainForm.MDIChildCount-1 do
  for J:=0 to (MainForm.MDIChildren[I] as TDataForm).Plot.Series.Count-1 do
  if (MainForm.MDIChildren[I] as TDataForm).Plot.Series[J].Container=Container
  then begin
    (MainForm.MDIChildren[I] as TDataForm).Plot.Series[J].ClearBlock;
    (MainForm.MDIChildren[I] as TDataForm).Plot.Series[J].Container:=nil;
  end;
  DMDocument.CloseDoc;//release COM object and zero its reference to TDataForm
end;

procedure TDataForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=vk_Insert) and (Shift=[]) then MainForm.Insert:=not MainForm.Insert;
end;

procedure TDataForm.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  if Plot.ClickedAt=claXAxis then
     if WheelDelta<0 then MainForm.ScaleX1Action.Execute
     else MainForm.ScaleX2Action.Execute;
  if Plot.ClickedAt=claYAxis then
     if WheelDelta<0 then MainForm.ScaleY1Action.Execute
     else MainForm.ScaleY2Action.Execute;
  Handled:=true;
end;

/////// Worksheet handlers /////////////
procedure TDataForm.WorksheetDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var S: string[20];
begin                                           // draw column numbers + [X,Y]
  if (ARow=1) and (ACol>0) then
  begin
    S:=char(byte('A')+ACol-1){+'('+IntToStr(ACol)+')'+' '};
    if ACol=XColumn then S:=S+'[X]'; if ACol=YColumn then S:=S+'[Y]';
    if Pos('][',S)<>0 then Delete(S,Pos('][',S),2);
    Worksheet.AlignTextOut(Rect,S);
  end;
end;

procedure TDataForm.WorksheetMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var C,R: longint; S: TGridRect;
begin
  Worksheet.MouseToCell(X,Y,C,R);
  if (R=1) and (C>0) and (Button=mbLeft) then
  begin
    if ssShift in Shift then YColumn:=C else XColumn:=C;  // select X/Y column
    if ssDouble in Shift then
    begin                                               // select whole column
      S.Left:=C; S.Right:=C; S.Top:=Worksheet.FixedRows;
      S.Bottom:=Container.Items.Count+Worksheet.FixedRows-1;
      Worksheet.Selection:=S;
    end;
  end;
  if (ssDouble in Shift) and (C>0) and (R=0) then          // set column label
  Labels[C]:=InputBox(strEnter, strColumnLabel, Labels[C]);
  if (C=0) and (R>=Worksheet.FixedRows) and (R<Worksheet.RowCount-1) then
  begin
    S:=Worksheet.Selection;
    if (Button=mbLeft) and (R<S.Bottom) and (not (ssShift in Shift))   {start}
    then S.Top:=R;
    if (R>S.Top) and (((Button=mbLeft) and (ssShift in Shift)) or
       (Button=mbMiddle)) then S.Bottom:=R;                         {end line}
    S.Left:=Worksheet.FixedCols; S.Right:=Worksheet.ColCount;
    Worksheet.Selection:=S;                         // all columns, as in DMW!
  end;
  Windows.SetFocus(MainForm.Clienthandle);      // if SerieCombobox focused...
end;

procedure TDataForm.WorksheetEnter(Sender: TObject);
begin Plot.BorderStyle:=bsNone; end;

procedure TDataForm.WorksheetExit(Sender: TObject);
begin Plot.BorderStyle:=bsSingle; end;

/////// Container handlers /////////////
procedure TDataForm.ContainerChanged(Sender: TObject);
var R: TGridRect; I,J: integer; b: boolean;
begin
  if Container.Modified then                 {correct blocks, selection, etc.}
  begin
    R:=Worksheet.Selection;
    if R.Top>Worksheet.FixedRows+Container.Items.Count-1
    then R.Top:=Worksheet.FixedRows+Container.Items.Count-1;
    if R.Bottom>Worksheet.FixedRows+Container.Items.Count-1
    then R.Bottom:=Worksheet.FixedRows+Container.Items.Count-1;
    if R.Top<Worksheet.FixedRows then R.Top:=Worksheet.FixedRows;
    if R.Bottom<Worksheet.FixedRows then R.Bottom:=Worksheet.FixedRows;
    {if ^ not corrected, weird errors occurs in DelLines!!!}
    with Worksheet do
    begin
      b:=goEditing in Options; Options:=Options+[goEditing];
      Selection:=R; UpdateSize;                  // fix pathological scrolling
      if not b then Options:=Options-[goEditing];
      // if not Recording then with Worksheet do Col:=FixedCols; !!!!!!!!!!!!!
    end;
    for I:=0 to MainForm.MDIChildCount-1 do                  // ALL WINDOWS !!
    for J:=0 to (MainForm.MDIChildren[I] as TDataForm).Plot.Series.Count-1 do
    with (MainForm.MDIChildren[I] as TDataForm).Plot.Series[J] do
    if (Container=Self.Container) and (LastLine>=Self.Container.Items.Count)
    then LastLine:=Self.Container.Items.Count-1;        {COLUMNS not checked!}
  end;
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
  if not Recording then begin Worksheet.Refresh; Plot.Refresh; end;
end;

function TDataForm.ContainerCompare(Sender: TObject; I1,
  I2: TData): TCompareResult;       // XColumn values used to compare elements
begin
  Result:=crEqual;
  try
    if (I1 as TRealData).RData[XColumn]>(I2 as TRealData).RData[XColumn]
    then Result:=crGreater;
    if (I1 as TRealData).RData[XColumn]<(I2 as TRealData).RData[XColumn]
    then Result:=crLess;
  except
    Result:=crEqual;                  {NOTE!!! missing data NOT PROCESSED!!!!}
  end;
end;

function TDataForm.ContainerInitItem(Sender: TObject): TData;
begin Result:=TRealData.Create; (Result as TRealData).Format:=@DataFormat;end;

procedure TDataForm.ContainerProgress(Sender: TObject; P: Integer);
begin MainForm.ContainerProgress(Sender,P); end;

/////// Plot handlers /////////////
procedure TDataForm.PlotHint(Sender: TObject; H: String);
begin
  // if Screen.ActiveControl is TMemo then Exit; works incorrectly
  if MainForm.GetActiveCtrlType<>actPlot then Exit;
  MainForm.StatusBar.Panels[MainForm.StatusBar.Panels.Count-2].Text:=H;
  MainForm.StatusBar.Update;        // refresh immediately only modified panel
end;

procedure TDataForm.PlotDblClick(Sender: TObject);
begin
  case Plot.ClickedAt of
    claPlot: if Plot.MouseMode=pmNone
             then MainForm.SeriePropertiesAction.Execute;
    claXAxis: MainForm.XAxisPropertiesAction.Execute;
    claYAxis: MainForm.YAxisPropertiesAction.Execute;
  end;
end;

procedure TDataForm.PlotPaint(Sender: TObject);
var I: integer; D: TRealData; X,Y: extended;
begin             // Paint leaders for ALL "recording" series (NOT as in dmw9)
  for I:=0 to Plot.Series.Count-1 do
  with Plot.Series[I] do
  if IsRecordingSerie[I] and (not Empty) and Assigned(Container) then
  begin
    D:=Container.Items[LastLine];
    if not Plot.GetDataPoint(D,XColumn,YColumn,XExpression,YExpression,X,Y)
    then DrawLeader(X,Y);
  end;
end;

procedure TDataForm.PlotError(Sender: TObject; H: String);
begin
  MainForm.DisplayMessage(H);
  MainForm.SerieComboBox.Refresh; MainForm.UpdateSerieList;
end;

var ShiftBuffer: TShiftState;

procedure TDataForm.PlotPointClick(Sender: TObject; Point, Serie: Integer);
var I,N: integer; R: TRealArray; X,Y: TReal; S: TGridRect;
begin
  MainForm.UpdatePicture(3);          // hint user - for all point click modes
  if Plot.Tag<>2 then                 //  except "calculator" (display result)
  begin
    MainForm.DisplayHint(Format(strPointClick,[Point,Serie]));
    MessageBeep($ffffffff);
  end;
  with MainForm do
  if Plot.Tag=1 then for I:=0 to MDIChildCount-1 do                {scroller:}
  if (MDIChildren[I] as TDataform).Container=Plot.Series[Serie].Container then
  begin
    (MDIChildren[I] as TDataForm).Scroll(Point); 
    if ssAlt in ShiftBuffer then                    // set worksheet selection
    begin                         // Alt+click - top, Alt+Shift+click - bottom
      S:=Worksheet.Selection; Point:=Point+Worksheet.FixedRows;
      if (Point<S.Bottom) and not (ssShift in ShiftBuffer) then S.Top:=Point;
      if (Point>S.Top) and (ssShift in ShiftBuffer) then S.Bottom:=Point;
      Worksheet.Selection:=S;
    end;
    if ssCtrl in ShiftBuffer then with MainForm do
    begin SerieComboBox.ItemIndex:=Serie; SerieComboBoxChange(nil); end;
  end;
  if Plot.Tag=2 then with MainForm, MainForm.ActiveDataForm.Plot do
  begin                                                          {calculator:}
    N:=TRealData(Series[Serie].Container.Items[Point]).GetRData(R);
    GetDataPoint(Series[Serie].Container.Items[Point],
      Series[Serie].XColumn, Series[Serie].YColumn,
      Series[Serie].XExpression, Series[Serie].YExpression,X,Y);
    Evaluate(R,N,X,Y);
  end;
end;

procedure TDataForm.PlotMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var R: TRealArray{dummy!}; rX,rY: TReal;
begin
  if (Button=mbLeft) and (Plot.Tag=2) and
  (ssDouble in Shift) and Plot.IntToRealCoords(X,Y,rX,rY) then
  MainForm.Evaluate(R,0,rX,rY);
  ShiftBuffer:=Shift;                // remember for using in PlotPointClick()
  Windows.SetFocus(MainForm.Clienthandle);      // if SerieCombobox focused...
  PlotPanel.SetFocus;                                   // focus plot by mouse
end;

procedure TDataForm.PlotSelectionChange(Sender: TObject);
begin   // note: undo not THIS window! (thisserie may reference any container)
  if Plot.MouseMode=pmTranslate then (Plot.ThisSerie.Container.Owner as
  TDataForm).SaveUndoInfo(MainForm.TranslationAction);
end;

end.
