{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit DigDlg;
{$B-,X+}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, ExtDlgs, ClipBrd, Data;

type
  TDigitizerForm = class(TForm)
    Digitizer: TPaintBox;
    ToolPanel: TPanel;
    OKBtn: TBitBtn;
    HelpBtn: TBitBtn;
    CancelBtn: TBitBtn;
    AxesGroupBox: TGroupBox;
    MinYE: TEdit;
    Label2: TLabel;
    MaxYE: TEdit;
    MaxXE: TEdit;
    Label1: TLabel;
    MinXE: TEdit;
    Data: TContainer;
    OpenDlg: TOpenPictureDialog;
    OpenBtn: TBitBtn;
    ClearPointsBitBtn: TBitBtn;
    PasteBitBtn: TBitBtn;
    DrawFrameSpeedButton: TSpeedButton;
    AddPointSpeedButton: TSpeedButton;
    DeletePointSpeedButton: TSpeedButton;
    HidePictureSpeedButton: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DigitizerPaint(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
    procedure ClearPointsBitBtnClick(Sender: TObject);
    procedure DigitizerMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DigitizerMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure DigitizerMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure EditChange(Sender: TObject);
    procedure PasteBitBtnClick(Sender: TObject);
    procedure HidePictureSpeedButtonClick(Sender: TObject);
    procedure ModeSpeedButtonClick(Sender: TObject);
  private
    BMP: TBitmap;
    MinX,MaxX,MinY,MaxY: extended;                          {real coordinates}
    Selection: TRect;                                    {integer coordinates}
    Selecting: boolean;
    Org,Temp: TPoint;
    procedure PaintPoints;                                 {only paint points}
    procedure wmInitMenu(var M: TMessage); message WM_INITMENU;
  public
    function Execute: boolean;
    procedure CopyData(Container: TContainer);
  end;

var DigitizerForm: TDigitizerForm=nil;

implementation

const PS=3; PC=clRed; FC=clBlue;          // point size and color, frame color

{$R *.DFM}

{$I DMRES.INC}                                                 // cursor codes

procedure TDigitizerForm.wmInitMenu(var M: TMessage);
var H: THandle;
begin
  H:=GetSystemMenu(Handle,false);     // get sysmenu handler (NOT M.wParam!!!)
  if WindowState=wsMaximized
  then EnableMenuItem(H,SC_RESTORE, MF_ENABLED+MF_BYCOMMAND)
  else EnableMenuItem(H,SC_MAXIMIZE, MF_ENABLED+MF_BYCOMMAND);
  M.Result:=0;                                // this means message processed!
end;

procedure TDigitizerForm.FormCreate(Sender: TObject);
begin
  BMP:=TBitmap.Create; EditChange(Self); Selecting:=false;
  Digitizer.Cursor:=crSelect;                      // init cursors for buttons
  AddPointSpeedButton.Tag:=crFrame;
  DeletePointSpeedButton.Tag:=crTarget;
  DrawFrameSpeedButton.Tag:=crSelect;
  with Selection do
  begin
    Top:=5; Bottom:=Digitizer.Height-5; Left:=5; Right:=Digitizer.Width-5;
  end;
end;

procedure TDigitizerForm.FormDestroy(Sender: TObject);
begin BMP.Free; end;

procedure TDigitizerForm.PaintPoints;
var oldC,oldW,I,X,Y: integer; F: TFunction;
begin
  for I:=0 to Data.Items.Count-1 do with Selection do
  begin
    F:=Data.Items[I];
    X:=Left+Round((Right-Left)*(F.X-MinX)/(MaxX-MinX));
    Y:=Bottom-Round((Bottom-Top)*(F.Y-MinY)/(MaxY-MinY));
    with Digitizer.Canvas do
    begin
      oldC:=Pen.Color; oldW:=Pen.Width; Pen.Color:=PC; Pen.Width:=2;
      MoveTo(X-PS,Y); LineTo(X+PS,Y); MoveTo(X,Y-PS); LineTo(X,Y+PS);
      Pen.Color:=oldC; Pen.Width:=oldW;
    end;
  end;
end;

procedure TDigitizerForm.DigitizerPaint(Sender: TObject);
var oldC: TColor;
begin
  if not HidePictureSpeedButton.Down
  then Digitizer.Canvas.Draw(0,0,BMP);                          {draw picture}
  oldC:=Digitizer.Canvas.Brush.Color;
  Digitizer.Canvas.Brush.Color:=FC;                          {set frame color}
  Digitizer.Canvas.FrameRect(Selection);                          {axes frame}
  Digitizer.Canvas.Brush.Color:=oldC;
  PaintPoints;                                                    {and points}
end;

procedure TDigitizerForm.OpenBtnClick(Sender: TObject);
begin
  if OpenDlg.Execute then
  try
    Screen.Cursor:=crHourGlass; BMP.LoadFromFile(OpenDlg.Filename);
  finally
    Screen.Cursor:=crDefault; Refresh;
  end;
end;

procedure TDigitizerForm.PasteBitBtnClick(Sender: TObject);
begin
  if Clipboard.HasFormat(CF_BITMAP) then
  begin BMP.Assign(Clipboard); Refresh; end;
end;

procedure TDigitizerForm.HidePictureSpeedButtonClick(Sender: TObject);
begin Refresh; end;                                        {hide/show picture}

procedure TDigitizerForm.ClearPointsBitBtnClick(Sender: TObject);
begin Data.Clear; Refresh; end;

procedure TDigitizerForm.DigitizerMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Selecting or (not DrawFrameSpeedButton.Down) then Exit;
  Org:=Point(X,Y); Temp:=Org; Selecting:=true;               {start selecting}
end;

function CorRect(x1,y1,x2,y2: integer): TRect;           {check: x1<x2, y1,y2}
var i: integer;
begin
  if x1>x2 then begin i:=x1; x1:=x2; x2:=i; end;
  if y1>y2 then begin i:=y1; y1:=y2; y2:=i; end;
  Result:=Rect(x1,y1,x2,y2);
end;

procedure TDigitizerForm.DigitizerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if Selecting then                                             {selecting...}
  begin
    Digitizer.Color:=clWhite;
    Digitizer.Canvas.DrawFocusRect(CorRect(Org.X, Org.Y, Temp.X, Temp.Y));
    Temp:=Point(X,Y);
    Digitizer.Canvas.DrawFocusRect(CorRect(Org.X, Org.Y, Temp.X, Temp.Y));
  end;
end;

procedure TDigitizerForm.DigitizerMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var F: TFunction; I: integer; iX,iY: extended;
begin
  if Selecting then                                         {finish selecting}
  begin
    Selecting:=false;
    if (Abs(Org.X-Temp.X)>3) and (Abs(Org.Y-Temp.Y)>3){check for single click}
    then Selection:=CorRect(Org.X, Org.Y, Temp.X, Temp.Y);
    Digitizer.Color:=clBlack; Refresh;
  end else
  begin                                                  {record/delete point}
    if AddPointSpeedButton.Down and (Button=mbLeft) then
    begin
      F:=TFunction.Create;
      F.X:=MinX+(MaxX-MinX)*(X-Selection.Left)/
      (Selection.Right-Selection.Left);
      F.Y:=MaxY-(MaxY-MinY)*(Y-Selection.Top)/
      (Selection.Bottom-Selection.Top);
      Data.Items.Add(F); PaintPoints;      {update only points - it's faster!}
    end;
    if DeletePointSpeedButton.Down or (Button=mbRight) then
    for I:=0 to Data.Items.Count-1 do
    begin
      F:=Data.Items[I];
      iX:=(F.X-MinX)/(MaxX-MinX)*(Selection.Right-Selection.Left)
      +Selection.Left;
      iY:=Selection.Top-
      (F.Y-MaxY)/(MaxY-MinY)*(Selection.Bottom-Selection.Top);
      if (Abs(X-iX)<PS) and (Abs(Y-iY)<PS) then
      begin Data.Items.Delete(I); F.Free; Refresh; Break; end;
    end;
  end;
end;

procedure TDigitizerForm.EditChange(Sender: TObject);
begin
  if Sender is TEdit then with Sender as TEdit do
  if (Text='+') or (Text='-') then Exit;            // filter some wrong input
  MinX:=StrToFloat(MinXE.Text); MaxX:=StrToFloat(MaxXE.Text);
  MinY:=StrToFloat(MinYE.Text); MaxY:=StrToFloat(MaxYE.Text);
end;

function TDigitizerForm.Execute: boolean;
begin Result:=(ShowModal=mrOK) and (Data.Items.Count>0); end;

procedure TDigitizerForm.CopyData(Container: TContainer);
var I: integer; F: TFunction; D: TData;
begin
  for I:=0 to Data.Items.Count-1 do
  begin
    F:=Data.Items[I];
    D:=Container.InitItem;
    // D.InsItem(F.X); D.InsItem(F.Y);
    D.Data:=F.Data;                              // copy data polymorphically!
    Container.Items.Add(D);
  end;
  Container.Modified:=true;
end;

procedure TDigitizerForm.ModeSpeedButtonClick(Sender: TObject);
begin Digitizer.Cursor:=(Sender as TSpeedButton).Tag; end;

end.
