{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit FitDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Plot, StdCtrls, Buttons, DataWnd, ComCtrls{!};

type
  TFittingForm = class(TForm)
    Plot: TPlot;
    Panel: TPanel;
    SelectSpeedButton: TSpeedButton;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    FitBitBtn: TBitBtn;
    ResultPanel: TPanel;
    ParametersLabel: TLabel;
    SerieCheckBox: TCheckBox;
    SerieComboBox: TComboBox;
    WorksheetCheckBox: TCheckBox;
    WorksheetComboBox: TComboBox;
    NotesCheckBox: TCheckBox;
    DataCheckBox: TCheckBox;
    ParametersListView: TListView;
    ZoomInSpeedButton: TSpeedButton;
    ZoomOutSpeedButton: TSpeedButton;
    RescaleBitBtn: TBitBtn;
    StatusBar: TStatusBar;
    procedure SerieCheckBoxClick(Sender: TObject);
    procedure PlotSpeedButtonClick(Sender: TObject);
    procedure WorksheetCheckBoxClick(Sender: TObject);
    procedure WorksheetComboBoxChange(Sender: TObject);
    procedure FitBitBtnClick(Sender: TObject);
    procedure RescaleBitBtnClick(Sender: TObject);
    procedure PlotHint(Sender: TObject; H: String);
    procedure PlotError(Sender: TObject; H: String);
  protected
    { Protected declarations }
    Plt: TPlot;
    DstForm: TDataForm;
    function GetFitData: string; virtual; abstract;
{    procedure wmInitMenu(var M: TMessage); message WM_INITMENU;             }
  public
    { Public declarations }
    procedure CleanFit;
    procedure PresetControls;
    procedure doExec;
  end;

implementation

uses MainWnd, Common, Data, NotesWnd, Strings;

{$R *.DFM}
{$I DMRes.inc}                                             // cursor constants

{ TFittingForm }
{ NOTE: now we decide to display context help ONLY in statusbar!
procedure TFittingForm.wmInitMenu(var M: TMessage);   // as in TDigitizerForm!
var H: THandle;
begin
  H:=GetSystemMenu(Handle,false);     // get sysmenu handler (NOT M.wParam!!!)
  if WindowState=wsMaximized
  then EnableMenuItem(H,SC_RESTORE, MF_ENABLED+MF_BYCOMMAND)
  else EnableMenuItem(H,SC_MAXIMIZE, MF_ENABLED+MF_BYCOMMAND);
  M.Result:=0;                                // this means message processed!
end;
}
procedure TFittingForm.CleanFit;
begin
  ParametersListView.Items.Clear;
  Plot.Series[1].YExpression:='';
  ResultPanel.Visible:=false;
end;

procedure TFittingForm.SerieCheckBoxClick(Sender: TObject);
begin SerieComboBox.Enabled:=SerieCheckBox.Checked; end;

procedure TFittingForm.PlotSpeedButtonClick(Sender: TObject);
begin
  Plot.SelectionVisible:=SelectSpeedButton.Down;
  if Plot.SelectionVisible
  then begin Plot.MouseMode:=pmSelect; Plot.Cursor:=crSelect; end else
  if ZoomInSpeedButton.Down then
  begin Plot.MouseMode:=pmZoom; Plot.Cursor:=crCross; end else
  if ZoomOutSpeedButton.Down then
  begin Plot.MouseMode:=pmUnZoom; Plot.Cursor:=crCross; end
  else begin Plot.MouseMode:=pmNone; Plot.Cursor:=crArrow; end;
end;

procedure TFittingForm.WorksheetCheckBoxClick(Sender: TObject);
begin WorksheetComboBox.Enabled:=WorksheetCheckBox.Checked; end;

procedure TFittingForm.WorksheetComboBoxChange(Sender: TObject);
begin
  if WorksheetComboBox.ItemIndex>=0
  then DstForm:=WorksheetComboBox.Items.Objects[WorksheetComboBox.ItemIndex]
  as TDataForm else DstForm:=nil;
end;

procedure TFittingForm.PresetControls;
var I: integer;
begin
  Plt:=MainForm.ActiveDataForm.Plot;                    // check points number
  SerieComboBox.Items.Clear;                           // setup serie combobox
  for I:=0 to MainForm.SerieComboBox.Items.Count-1 do
  SerieComboBox.Items.Add(' ');    // SerieComboBoxDrawItem event is shared!!!
  SerieComboBox.OnDrawItem:=MainForm.SerieComboBoxDrawItem;
  Plot.SelectionTop:=Plt.SelectionTop; Plot.SelectionLeft:=Plt.SelectionLeft;
  Plot.SelectionBottom:=Plt.SelectionBottom;           // copy plot properties
  Plot.SelectionRight:=Plt.SelectionRight;
  SelectSpeedButton.Down:=Plt.SelectionVisible;
  PlotSpeedButtonClick(nil);                         // update plot indirectly
  if (Plt.XAxis.Expression<>'') or (Plt.YAxis.Expression<>'')
  then MessageDlg(strAxisEmptyExpr,mtWarning,[mbOk],0);
  Plot.Series[0].FirstLine:=Plt.ThisSerie.FirstLine;
  Plot.Series[0].LastLine:=Plt.ThisSerie.LastLine;
  Plot.Series[0].XColumn:=Plt.ThisSerie.XColumn;
  Plot.Series[0].YColumn:=Plt.ThisSerie.YColumn;
  Plot.Series[0].XExpression:=Plt.ThisSerie.XExpression;
  Plot.Series[0].YExpression:=Plt.ThisSerie.YExpression;
  Plot.Series[0].Container:=Plt.ThisSerie.Container;
  WorksheetComboBox.Items.Clear;         // correct destination worksheet list
  for I:=0 to Mainform.MDIChildCount-1 do
  begin
    WorksheetComboBox.Items.AddObject(MainForm.GetContainerName(
    (MainForm.MDIChildren[I] as TDataForm).Container,true),
    MainForm.MDIChildren[I]);
    if MainForm.MDIChildren[I]=DstForm then WorksheetComboBox.ItemIndex:=I;
  end;
end;

procedure TFittingForm.doExec;
var I: integer; D: TRealData; S: string; X,Y: extended;
begin
  if (ShowModal<>mrOk) or (not ResultPanel.Visible) then Exit; // no fit data!
  if SerieCheckBox.Checked and (SerieComboBox.ItemIndex>=0) then
  with Plt.Series[SerieComboBox.ItemIndex] do
  begin
    ClearBlock; IsFunction:=true; LastLine:=Plot.Series[1].LastLine;
    YExpression:=Plot.Series[1].YExpression; PointVisible:=false;
    LineVisible:=true;
  end;
  if WorksheetCheckBox.Checked and (WorksheetComboBox.ItemIndex>=0) then
  begin
    DstForm.SaveUndoInfo(MainForm.LinearFittingAction); I:=1; S:='';
    while (I<=MaxCols) and (I<=ParametersListView.Items.Count) and
      (ParametersListView.Items[I-1].Caption<>'') do    // ALL nonempty values
    begin                               // caption - Value, SubItems[0] - Name
      DstForm.Labels[I]:=ParametersListView.Items[I-1].SubItems[0];
      S:=S+' '+ParametersListView.Items[I-1].Caption; Inc(I);
    end;
    D:=TRealData(DstForm.Container.InitItem);
    D.Data:=S; DstForm.Container.Items.Add(D);
    DstForm.Container.Modified:=true;
  end;
  if NotesCheckBox.Checked then
  begin
    NotesForm.Memo.Lines.Add(Format(strFitResults,[Caption,
    MainForm.GetContainerName(Plot.Series[0].Container, false),
    Plot.Series[0].XColumn, Plot.Series[0].YColumn,
    Plot.Series[0].FirstLine, Plot.Series[0].LastLine,
    Plot.Series[0].XExpression, Plot.Series[0].YExpression, GetFitData]));
    I:=1;
    while (I<=ParametersListView.Items.Count) and
      (ParametersListView.Items[I-1].Caption<>'') do
    begin
      S:=' '+ParametersListView.Items[I-1].SubItems[0]+' '#9+
      ParametersListView.Items[I-1].Caption;
      if ParametersListView.Items[I-1].SubItems.Count=2       // only for LM?!
      then S:=S+' '#9+ParametersListView.Columns[2].Caption+':   '+
      ParametersListView.Items[I-1].SubItems[1];
      NotesForm.Memo.Lines.Add(S); Inc(I);
    end;
  end;
  if DataCheckBox.Checked then
  begin
    (Plot.Series[0].Container.Owner as   // note: NOT MainForm.ActiveDataForm!
    TDataForm).SaveUndoInfo(MainForm.LinearFittingAction);
    Screen.Cursor:=crHourGlass;
    MainForm.StatusImageList.Tag:=8; MainForm.DisplayHint(strWorking);
    try
      for I:=Plot.Series[0].FirstLine to Plot.Series[0].LastLine do
      begin
        D:=TRealData(Plot.Series[0].Container.Items[I]);
        with Plot do GetDataPoint(D,Series[0].XColumn,1,'',
          Series[1].YExpression,X,Y);
        D.InsItem(Y);
      end;
    finally
      MainForm.StatusImageList.Tag:=0; MainForm.DisplayHint('');
      Screen.Cursor:=crDefault; Plot.Series[0].Container.Modified:=true;
    end;
  end;
end;

procedure TFittingForm.FitBitBtnClick(Sender: TObject);
begin if Plot.Series[1].YExpression<>'' then ResultPanel.Visible:=true; end;

procedure TFittingForm.RescaleBitBtnClick(Sender: TObject);
begin Plot.XAxis.AutoScale:=true; Plot.YAxis.AutoScale:=true; end;

procedure TFittingForm.PlotHint(Sender: TObject; H: String);
begin StatusBar.Panels[0].Text:=H; StatusBar.Update; end;

procedure TFittingForm.PlotError(Sender: TObject; H: String);
begin
  MessageBeep($ffffffff);
  with StatusBar do begin Panels[Panels.Count-1].Text:=H; Update; end;
end;

end.
