{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit LFitDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FitDlg, ImgList, StdCtrls, Spin, Buttons, ExtCtrls, Plot, LeastSqr,
  ComCtrls;

type
  TLinearFitForm = class(TFittingForm)
    EquationLabel: TLabel;
    EquationComboBox: TComboBox;
    TermsLabel: TLabel;
    TermsSpinEdit: TSpinEdit;
    EquationImageList: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FitBitBtnClick(Sender: TObject);
    procedure EquationComboBoxDrawItem(Control: TWinControl;
      Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure EquationComboBoxChange(Sender: TObject);
  protected
    { Protected declarations }
    StandardDeviation : Float;        { Square root of variance;   }
    function GetFitData: string; override;
  public
    { Public declarations }
  end;

var LinearFitForm: TLinearFitForm;

implementation

uses Data, Strings, MainWnd;

{$R *.DFM}

procedure TLinearFitForm.FormCreate(Sender: TObject);
begin EquationComboBox.ItemIndex:=0; end;

function TLinearFitForm.GetFitData: string;
begin
  Result:=Format(strLFitData,
  [EquationComboBox.Text,StandardDeviation]);
end;

procedure TLinearFitForm.FitBitBtnClick(Sender: TObject);
var
  XData, YData : TNColumnVector;    { Data points (X,Y)  }
  NumTerms : integer;               { # of terms in least squares  }
  Residuals : TNColumnVector;       { YFit - YData  }
  Error : byte;                     { Flags if something went wrong  }
  Solution : TNRowVector;           { Coefficients of the l.s. fit  }
  NumPoints : integer;              { # of points  }
  YFit : TNColumnVector;            { Least squares solution at XData points }
  Variance : Float;                 { Indicates goodness of fit  }
  I: integer; s,ss: string; X,Y: extended; Fitter: TFitFunctionClass;
begin
  CleanFit; NumPoints:=Plot.Series[0].LastLine-Plot.Series[0].FirstLine+1;
  SetLength(XData, NumPoints+1); SetLength(YData, NumPoints+1); // init arrays
  SetLength(YFit, NumPoints+1); SetLength(Residuals, NumPoints+1);
  NumPoints:=0; Screen.Cursor:=crHourGlass;
  try
    with Plot.Series[0] do for I:=FirstLine to LastLine do
    begin                                         // init input data (1..Max!)
      Plot.GetDataPoint(TData(Container.Items[I]),
      XColumn,YColumn,XExpression,YExpression,X,Y);
      if ((X>=Plot.SelectionLeft) and (X<=Plot.SelectionRight)
        and (Y>=Plot.SelectionBottom) and (Y<=Plot.SelectionTop))
        or (not Plot.SelectionVisible) then
      begin XData[NumPoints+1]:=X; YData[NumPoints+1]:=Y; Inc(NumPoints); end;
    end;
  finally
    Screen.Cursor:=crDefault;
  end;
  case EquationComboBox.ItemIndex of
    0: Fitter:=TPolyFitFunction;
    1: Fitter:=TFourierFitFunction;
    2: Fitter:=TPowerFitFunction;
    3: Fitter:=TExpoFitFunction;
    4: Fitter:=TLorentzFitFunction;
    5: Fitter:=TSineFitFunction;
  end;
{From execute:  I:=Plt.ThisSerie.LastLine-Plt.ThisSerie.FirstLine+1;
  if I<5 then
  begin MessageDlg(Format(strBaseLine,[5]),mtError,[mbCancel],0); Exit; end;
  with TermsSpinEdit do
  begin Value:=2; if I<Tag then MaxValue:=I else MaxValue:=Tag; end;}
  if EquationComboBox.ItemIndex<2 then
  begin
    NumTerms:=TermsSpinEdit.Value;
    if NumPoints<NumTerms then
    begin
      MessageDlg(Format(strBaseLine,[NumTerms]),mtError,[mbCancel],0); Exit;
    end;
  end else NumTerms:=1{actual value assigned by fitter!};
{^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^}
  Screen.Cursor:=crHourGlass; Error:=5;                              {play!!!}
  StatusBar.Panels[1].Text:=strWorking; StatusBar.Refresh;
  try
    LeastSquares(NumPoints, XData, YData, NumTerms, Solution,
    YFit, Residuals, StandardDeviation, Variance, Error, Fitter);
  finally
    StatusBar.Panels[1].Text:=''; Screen.Cursor:=crDefault;
  end;
  if Error<>0 then
  begin
    MessageDlg(Format(strLFitError,[Error]),mtError,[mbCancel],0); Exit;
  end;
  case EquationComboBox.ItemIndex of                       // start output fit
  0,1: begin
    with ParametersListView do
    for I:=1 to NumTerms do with Items.Add do SubItems.Add('a'+IntToStr(I-1));
    S:=FloatToStrF(Solution[1],ffExponent,6,2);         // 1st term (constant)
    ParametersListView.Items[0].Caption:=S;
    for I:=2 to NumTerms do
    begin
      ss:=FloatToStrF(Solution[I],ffExponent,6,2);
      if Solution[I]>=0 then SS:='+'+SS;
      ParametersListView.Items[I-1].Caption:=SS;
      if EquationComboBox.ItemIndex=0 then S:=S+SS+'*CX^'+IntToStr(I-1){poly}
      else if Odd(I) then S:=S+SS+'*sin(cx*'+IntToStr(I div 2)+')' else
           S:=S+SS+'*cos(cx*'+IntToStr(I div 2)+')';{fourier}
    end;
  end;
  2,3: begin
    S:=FloatToStrF(Solution[1],ffExponent,6,2);
    SS:=FloatToStrF(Solution[2],ffExponent,6,2);
    with ParametersListView.Items.Add do
    begin Caption:=S; SubItems.Add('A'); end;
    with ParametersListView.Items.Add do
    begin Caption:=SS; SubItems.Add('B'); end;
    if EquationComboBox.ItemIndex=2 then S:=S+'*CX^'+SS
    else S:=S+'*exp(CX*'+SS+')';
  end;
  4,5: begin
    ParametersListView.Items.Add.SubItems.Add('A');
    if EquationComboBox.ItemIndex=4 then
    begin
      ParametersListView.Items.Add.SubItems.Add('Xo');
      ParametersListView.Items.Add.SubItems.Add('B');
    end else
    begin
      ParametersListView.Items.Add.SubItems.Add('B');
      ParametersListView.Items.Add.SubItems.Add('C');
    end;
    for I:=1 to 3 do
    begin
      S:=FloatToStrF(Solution[I],ffExponent,6,2);
      if Solution[I]>=0 then S:='+'+S;
      ParametersListView.Items[I-1].Caption:=S;
    end;
    with ParametersListView do if EquationComboBox.ItemIndex=4 then S:=
   Items[0].Caption+'*cx/(('+Items[1].Caption+'-cx)^2'+Items[2].Caption+'*cx)'
    else S:=Items[0].Caption+Items[1].Caption+'*sin(cx'+Items[2].Caption+')';
  end;
  end;{case}
  Plot.Series[1].YExpression:=S; inherited;
end;

procedure TLinearFitForm.EquationComboBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  EquationComboBox.Canvas.FillRect(Rect);
  if Index>=0 then
  EquationImageList.Draw(EquationComboBox.Canvas,Rect.Left,Rect.Top,Index);
end;

procedure TLinearFitForm.EquationComboBoxChange(Sender: TObject);
begin
  if EquationComboBox.ItemIndex=0 then
  begin
    TermsSpinEdit.Increment:=1;
    TermsSpinEdit.MinValue:=1; TermsSpinEdit.Value:=2;
  end else
  begin
    TermsSpinEdit.Increment:=2; TermsSpinEdit.MinValue:=3;
    TermsSpinEdit.Value:=3;
  end;
  TermsSpinEdit.Visible:=EquationComboBox.ItemIndex<2;
  TermsLabel.Visible:=EquationComboBox.ItemIndex<2;
end;

end.
