{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit LMFitDlg;
{$R-}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FitDlg, StdCtrls, Grids, Buttons, ExtCtrls, Plot, ComCtrls,
  LMFit, Common, Parser, Data;

type
  TNLSFForm = class(TFittingForm)
    FunctionBitBtn: TBitBtn;
    OptionsBitBtn: TBitBtn;
    UpdateBitBtn: TBitBtn;
    PrevBitBtn: TBitBtn;
    NextBitBtn: TBitBtn;
    ChiSqrLabel: TLabel;
    ChiSqrEdit: TEdit;
    AddBitBtn: TBitBtn;
    Container: TContainer;
    procedure FunctionBitBtnClick(Sender: TObject);
    procedure FitBitBtnClick(Sender: TObject);
    procedure UpdateBitBtnClick(Sender: TObject);
    procedure OptionsBitBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AddBitBtnClick(Sender: TObject);
    procedure PrevNextBitBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  protected
    { Protected declarations }
    Parser: TMathParser;
    Parameters: TRealArray;
    ChiSqR,X: TReal;
    A,SigmaA: FitArray;
    function GetFitData: string; override;
    procedure UpdateFit;
    procedure UpdateNextPrev;
  public
    { Public declarations }
    Expression: string;                       // holds expression being fitted
    NParam,NIter: integer;
    Func: TFunc;    // 2 procedural variables for CurveFit() - may be changed!
    Deriv: TDeriv;
    procedure CalcNParam;                     // also called from Automate.pas
  end;

var NLSFForm: TNLSFForm;

function LMExpression(X: TReal; A: FitArray; NParam: Integer): TReal;
procedure LMDerivative(X: TReal; A,SigA : FitArray; Nparam: Integer;
  var Deriv : FitArray);

const LMDelta: TReal=1e-10;

implementation

uses LMOptDlg, MainWnd, Strings, CalcDlg, StrDlg, DataWnd;

{$R *.DFM}

procedure Progress;
var S: string[250];
begin
  S:=Format(strLMFitStatus,[NLSFForm.Niter]);
  with NLSFForm.StatusBar do begin Panels[1].Text:=S; Update; end;
  if NLSFOptionsForm.UpdateCheckBox.Checked then NLSFForm.UpdateFit;
end;

function TNLSFForm.GetFitData: string;
begin
  Result:=Format(strLMFitData,
  [Expression,ChiSqrEdit.Text,StatusBar.Panels[1].Text]);
end;

function LMExpression(X: TReal; A: FitArray; NParam: Integer): TReal;
begin
  NLSFForm.Parameters:=TRealArray(A);{!!!!!!!!!} NLSFForm.X:=X;
  Result:=NLSFForm.Parser.Parse(NLSFForm.Expression);
end;

procedure LMDerivative(X: TReal; A,SigA : FitArray; Nparam: Integer;
                 var Deriv : FitArray);
var I: integer; A1: FitArray; Y1,Y2,dP: TReal;
begin
  for I:=1 to Nparam do
  if SigA[I]<0 then Deriv[I]:=0 else
  begin
    if Abs(A[I])<LMDelta then dP:=LMDelta else dP:=Abs(A[I])*LMDelta;
    A1:=A; A1[I]:=A1[I]-dP; Y1:=LMExpression(X,A1,Nparam);
    A1:=A; A1[I]:=A1[I]+dP; Y2:=LMExpression(X,A1,Nparam);
    Deriv[I]:=(Y2-Y1)/2/dP;
  end;
end;

procedure TNLSFForm.CalcNParam;
begin
  NParam:=1;
  while Pos('P'+IntToStr(NParam),UpperCase(Expression))>0 do Inc(NParam);
  Dec(NParam);
end;

procedure TNLSFForm.FunctionBitBtnClick(Sender: TObject);
var I: integer;
begin
  //Expression:=InputBox(strEnter,strLMFitExpression,Expression);
  if not InputString(strLMFitExpression,
    CalculatorForm.ExpressionComboBox.Items, Expression, true) then Exit;
  CalcNParam;
  Func:=LMExpression; Deriv:=LMDerivative;
  ParametersListView.Items.Clear;
  Plot.Series[1].YExpression:='';
  ResultPanel.Visible:=true;
  Plot.YAxis.Title:=Format(strLMFitAxisTitle,[Expression]);
  FitBitBtn.Enabled:=true; UpdateBitBtn.Enabled:=true;
  for I:=1 to NParam do with ParametersListView.Items.Add do
  begin Caption:='1'; SubItems.Add('p'+IntToStr(I)); SubItems.Add(' '); end;
end;

procedure TNLSFForm.UpdateBitBtnClick(Sender: TObject);
var I,P: integer; Expr: string;
begin
  Expr:=Expression;
  for I:=1 to Nparam do    // substitute parameter values instead of P1,P2,...
  repeat
    P:=Pos('P'+IntToStr(I),UpperCase(Expr));
    if P>0 then
    begin
      Delete(Expr,P,2); Insert(ParametersListView.Items[I-1].Caption,Expr,P);
    end;
  until P=0;
  if Assigned(Sender) then      // repair functional serie if cleared by error
  Plot.Series[1].LastLine:=NLSFOptionsForm.NumPointsSpinEdit.Value;
  Plot.Series[1].YExpression:=Expr; Plot.Update; // update with new parameters
end;

procedure TNLSFForm.OptionsBitBtnClick(Sender: TObject);
begin
  NLSFOptionsForm.NumPointsSpinEdit.Value:=Plot.Series[1].LastLine;
  NLSFOptionsForm.ShowModal;
  Plot.Series[1].LastLine:=NLSFOptionsForm.NumPointsSpinEdit.Value;
end;

procedure TNLSFForm.FitBitBtnClick(Sender: TObject);
var X,Y,Yfit,W: DataPointer;                           // CurveFit() arguments
    I,NPts: integer;
    ParDel,ChiDel,xx,yy: TReal;
    Code: TFitResult;
begin
{Make initial estimates for all parameters; set fixed parameters
(setting SigmaA[i] := -1 fixes A[i] and setting SigmaA[i] := 0
frees A[i]); set convergence criteria}
  Niter:=NLSFOptionsForm.NumIterSpinEdit.Value-1;
  for I:=1 to Nparam do
  begin
    A[I]:=StrToFloat(ParametersListView.Items[I-1].Caption);
    if ParametersListView.Items[I-1].Checked
    then SigmaA[I]:=-1 else SigmaA[I]:=0;
  end;
  // ParDel:=1E-10; ChiDel:=1E-7;  as recommended
  if NLSFOptionsForm.ParDelCheckBox.Checked
  then ParDel:=StrToFloat(NLSFOptionsForm.ParDelEdit.Text) else ParDel:=0;
  if NLSFOptionsForm.ChiDelCheckBox.Checked
  then ChiDel:=StrToFloat(NLSFOptionsForm.ChiDelEdit.Text) else ChiDel:=0;
  LMDelta:=StrToFloat(NLSFOptionsForm.DerivDeltaEdit.Text);
// additional setup:
  ChiSqrEdit.Text:=''; StatusBar.Panels[1].Text:='';
{allocate and fill data arrays. Note: range check MUST be OFF!!!}
  Npts:=Plot.Series[0].LastLine-Plot.Series[0].FirstLine+1;
  GetMem(X,Npts*SizeOf(TReal));
  GetMem(Y,Npts*SizeOf(TReal));
  GetMem(YFit,Npts*SizeOf(TReal));
  GetMem(W,Npts*SizeOf(TReal));
  Npts:=0; Screen.Cursor:=crHourGlass;
  StatusBar.Panels[2].Text:=strWorking; StatusBar.Refresh;
  try
{copy data (ONLY selected points)}
    with Plot.Series[0] do for I:=FirstLine to LastLine do
    begin
      Plot.GetDataPoint(TData(Container.Items[I]),
      XColumn,YColumn,XExpression,YExpression,XX,YY);
      if ((XX>=Plot.SelectionLeft) and (XX<=Plot.SelectionRight)
        and (YY>=Plot.SelectionBottom) and (YY<=Plot.SelectionTop))
        or (not Plot.SelectionVisible) then
      begin
        Inc(NPts); X^[NPts]:=XX; Y^[NPts]:=YY;
        W^[Npts]:=1;
        with NLSFOptionsForm do
        if (WeightType<>wtEqual) and (ColumnComboBox.ItemIndex>=0) then
        W^[Npts]:=
        TRealData(Container.Items[I]).RData[ColumnComboBox.ItemIndex+1];
      end;
    end;{with}
{perform fitting...}
    CurveFit(X,Y,W,NPts,NParam,A,SigmaA,ChiSqR,Yfit,Niter,ParDel,ChiDel,
      {wtEqual}NLSFOptionsForm.WeightType, Func, Deriv, Progress, Code);
  finally
{Display fitting results:
Where results are placed:
X^[1..Npts] - X points (source)
Y^[1..Npts] - Y points (source)
YFit^[1..Npts] - Y fit (result)
A[1..NParam] - fit params
SigmaA[1..NParam] +/- A[...]
Sqrt(ChiSqR) - standard deviation}
    FreeMem(X);                                          // dispose off arrays
    FreeMem(Y);
    FreeMem(YFit);
    FreeMem(W);
    Screen.Cursor:=crDefault; StatusBar.Panels[2].Text:='';
    with StatusBar.Panels[1] do
    case Code of
      frParamConv: Text:='Parameters converged';
      frChiConv: Text:='Chi squared converged';
      frMaxIter: Text:='All iterations completed';
      frFitError: Text:='Failure: Flamda overflow';
      frOther: Text:='Failure: Other error';
    end;
  end;
  UpdateFit;
  StatusBar.Panels[1].Text:=Format(strLMFitResult,
    [StatusBar.Panels[1].Text,Niter]);
end;

procedure TNLSFForm.UpdateFit;
var I: integer;
begin
  ChiSqrEdit.Text:=FloatToStrF(Sqrt(ChiSqR),ffGeneral,7,4);
  for I:=1 to Nparam do with ParametersListView.Items[I-1] do
  begin
    Caption:=FloatToStrF(A[I],ffGeneral,7,4);
    if SigmaA[I]<0 then SubItems[1]:=strLMFitParamFixed else
    SubItems[1]:=FloatToStrF(SigmaA[I],ffGeneral,7,4);
  end;
  UpdateBitBtnClick(nil);                                  // update fit serie
end;

procedure TNLSFForm.FormCreate(Sender: TObject);
var I: integer;
begin
  Parser:=TMathParser.Create;
  with Parser do
  begin
    Init(60); AddGonio; AddLogic; AddMath; AddMisc;
    for I:=1 to MaxCols do
    AddObject(@Parameters[I], 'p'+IntToStr(I), tfp_realvar);
    AddObject(@X,'cx',tfp_realvar);
  end;
end;

procedure TNLSFForm.FormDestroy(Sender: TObject);
begin Parser.Free; end;

procedure TNLSFForm.AddBitBtnClick(Sender: TObject);
var I: integer; D: TRealData;
begin
  D:=TRealData.Create;
  for I:=1 to Nparam do
  D.InsItem(StrToFloat(ParametersListView.Items[I-1].Caption));
  Container.Items.Add(D);
  with Container do Tag:=Tag+1;
  UpdateNextPrev;
end;

procedure TNLSFForm.PrevNextBitBtnClick(Sender: TObject);
var I: integer; D: TRealData;
begin
  with Container do if Sender=PrevBitBtn then Tag:=Tag-1 else Tag:=Tag+1;
  UpdateNextPrev;
  D:=Container.Items[Container.Tag];
  for I:=1 to D.Size do if I<=ParametersListView.Items.Count then
  with ParametersListView.Items[I-1] do
  begin Caption:=FloatToStrF(D.RData[I],ffGeneral,7,4); SubItems[1]:=''; end;
  UpdateBitBtnClick(Sender);
end;

procedure TNLSFForm.UpdateNextPrev;
begin
  PrevBitBtn.Enabled:=Container.Tag>0;
  NextBitBtn.Enabled:=Container.Tag<Container.Items.Count-1;
end;

procedure TNLSFForm.FormShow(Sender: TObject);
var I: integer;
begin
  inherited; Container.Clear; Container.Tag:=-1; UpdateNextPrev;
  NLSFOptionsForm.ColumnComboBox.ItemIndex:=-1;
  NLSFOptionsForm.WtEqualRadioButton.Checked:=true;
  NLSFOptionsForm.ColumnComboBox.Items.Assign(
  (Plot.Series[0].Container.Owner as TDataForm).Worksheet.Header);
  with NLSFOptionsForm.ColumnComboBox do       // add A..Z as for SeriePropDlg
  for I:=0 to Items.Count-1 do Items[I]:='['+char(byte('A')+I)+']: '+Items[I];
end;

end.

