{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit MonWnd;
{$B-}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ImgList, ToolWin, ScktComp, ActnList, Menus;

type
  TMonitorForm = class(TForm)
    CoolBar: TCoolBar;
    ListView: TListView;
    ExperimentToolBar: TToolBar;
    ImageList: TImageList;
    RunToolButton: TToolButton;
    PointToolButton: TToolButton;
    ScanToolButton: TToolButton;
    ReverseToolButton: TToolButton;
    ToolButton5: TToolButton;
    ConnectionToolButton: TToolButton;
    ClientSocket: TClientSocket;
    ActionList: TActionList;
    RunAction: TAction;
    PointAction: TAction;
    ScanAction: TAction;
    ReverseAction: TAction;
    ConnectionAction: TAction;
    ShowRecordingWindowAction: TAction;
    SetRecordingWindowAction: TAction;
    SetRecordingSerieAction: TAction;
    NextSerieAction: TAction;
    ControlToolBar: TToolBar;
    ShowRecordingWindowToolButton: TToolButton;
    SetRecordingWindowToolButton: TToolButton;
    ToolButton3: TToolButton;
    SetRecordingSerieToolButton: TToolButton;
    NextSerieToolButton: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure ClientSocketConnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocketDisconnect(Sender: TObject;
      Socket: TCustomWinSocket);
    procedure ClientSocketError(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ClientSocketRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientSocketWrite(Sender: TObject; Socket: TCustomWinSocket);
    procedure ExperimentActionExecute(Sender: TObject);
    procedure ShowRecordingWindowActionExecute(Sender: TObject);
    procedure ConnectionActionUpdate(Sender: TObject);
    procedure ConnectionActionExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ExperimentActionUpdate(Sender: TObject);
    procedure SetRecordingWindowActionExecute(Sender: TObject);
    procedure SetRecordingSerieActionExecute(Sender: TObject);
    procedure SetRecordingSerieActionUpdate(Sender: TObject);
    procedure SetRecordingWindowActionUpdate(Sender: TObject);
    procedure NextSerieActionExecute(Sender: TObject);
    procedure NextSerieActionUpdate(Sender: TObject);
  private
    { Private declarations }
    Lst: TStringList;                  // used to parse experiment server data
    procedure Minimize(Sender: TObject);     // show monitor if app. minimized
  public
    function Connected: boolean;
    procedure GetExpSettings;                    // ask server for expsettings
    { Public declarations }
  end;

var MonitorForm: TMonitorForm;

implementation

uses MainWnd, DataWnd, AppDlg, Strings, Common;

{$R *.DFM}

{ TMonitorForm }

function TMonitorForm.Connected: boolean;
begin Result:=ClientSocket.Active; end;

procedure TMonitorForm.Minimize(Sender: TObject);
begin
  if (WindowState<>wsMinimized) and Visible then ShowWindow(Handle, SW_Show);
end;

procedure TMonitorForm.GetExpSettings;
begin if Connected then ClientSocket.Socket.SendText(strExpSettings); end;

procedure TMonitorForm.FormCreate(Sender: TObject);
begin
  //Application.OnMinimize:=Minimize;
  MainForm.ApplicationEvents.OnMinimize:=Minimize;           // hook main form
  Lst:=TStringList.Create;              // create list for parsing server data
end;

procedure TMonitorForm.FormDestroy(Sender: TObject);
begin Lst.Free; {ClientSocket.Close; causes AV on exit!!!} end;

///// socket event handlers /////
procedure TMonitorForm.ClientSocketConnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin
  MainForm.DisplayHint(Format(strConnectionEstablished,
  [ClientSocket.Host, Socket.RemoteAddress]));
end;

procedure TMonitorForm.ClientSocketDisconnect(Sender: TObject;
  Socket: TCustomWinSocket);
begin MainForm.DisplayHint(strDisconnected); end;

procedure TMonitorForm.ClientSocketError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
var S: string;
begin
  case ErrorEvent of
    eeGeneral: S:='General';
    eeSend: S:='Send';
    eeReceive: S:='Receive';
    eeConnect: S:='Connect';
    eeDisconnect: S:='Disconnect';
    eeAccept: S:='Accept';
  end;
  S:=Format(strConnectionError,[S,ErrorCode]); ErrorCode:=0; // else exception
  MainForm.DisplayMessage(S);
end;

procedure TMonitorForm.ClientSocketRead(Sender: TObject;
  Socket: TCustomWinSocket);
var S: string; I,J,N: integer; R: TRealArray;
begin
  S:=Socket.ReceiveText;                              // read data from socket
/// Parse data transmitted by server: ////////////////////////////////////////
  Lst.Clear; Lst.Text:=S; if Lst.Count=0 then Exit;    // parse string by list
  if UpperCase(Lst.Names[0])=strMessage then                // server message?
  begin
    MessageDlg(Format(strServerMessage, [Lst.Values[strMessage]]),
      mtInformation,[mbOk],0); Exit;            // display message AND EXIT!!!
  end;
  if UpperCase(Lst.Names[0])=strExpSettings then       // experiment settings?
  begin
    Lst.Delete(0);                       // clear 1st "key" string message=xxx
    MainForm.ExpSettings:=Lst.Text; Exit;
  end;
  if UpperCase(Lst.Names[0])=strData then          // start to process DATA...
  begin
    J:=StrToIntDef(Lst.Values[strData],0); Lst.Delete(0);
    if J=0 then Exit;                                   // NO numbers in data!
    if J<>Lst.Count then MainForm.DisplayMessage(strInvalidData);
    {maybe, SEVERAL messages in buffer!!!
    if so, process continued with first J values}
  end else Exit;                               // don't process other commands
  try
    ListView.Items.BeginUpdate; N:=0;
    with ListView do for I:=0 to Items.Count-1 do       // clear previous data
    begin Items[I].SubItems[0]:=''; Items[I].SubItems[1]:=''; end;
    for I:=0 to {Lst.Count-1}J-1{!!!!} do with ListView.Items[I] do
    begin
      SubItems[0]:=Lst.Names[I]; SubItems[1]:=Lst.Values[Lst.Names[I]];
      Inc(N);
      try
        R[N]:=StrToFloat(Lst.Values[Lst.Names[I]]);         // safely evaluate
      except
        Dec(N); MainForm.DisplayMessage(strInvalidData); Break;   // warn user
      end;
    end;
  finally
    ListView.Items.EndUpdate;
  end;
/// NOW R[],N contain valid data (N may be 0!) ///////////////////////////////
  for I:=0 to Lst.Count-1 do Lst[I]:=Lst.Names[I];        // copy ONLY labels!
  MainForm.RecordPoint(N,R,Lst);
end;

procedure TMonitorForm.ClientSocketWrite(Sender: TObject;
  Socket: TCustomWinSocket);
begin Socket.SendText(AppPropsForm.PasswordEdit.Text); end;  // reply password

///// Action handlers /////
procedure TMonitorForm.ConnectionActionExecute(Sender: TObject);
begin
  if ConnectionAction.Checked then ClientSocket.Close else
  begin
    ClientSocket.Host:=AppPropsForm.HostNameEdit.Text;
    ClientSocket.Port:=AppPropsForm.IPPortSpinEdit.Value; ClientSocket.Open;
  end;
end;

procedure TMonitorForm.ConnectionActionUpdate(Sender: TObject);
begin
  ConnectionAction.Checked:=ClientSocket.Active;
  ConnectionToolButton.Down:=ConnectionAction.Checked;
end;

procedure TMonitorForm.ExperimentActionExecute(Sender: TObject);
  procedure doCMD(S: string);
  begin
    if Connected then ClientSocket.Socket.SendText(S);
    with MainForm do if Assigned(DMApplication) then DMApplication.Command(S);
  end;
begin
  if Sender=RunAction then doCMD(strRun);
  if Sender=PointAction then doCMD(strPoint);
  if Sender=ScanAction then doCMD(strScan);
  if Sender=ReverseAction then doCMD(strReverse);
end;

procedure TMonitorForm.ExperimentActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=Connected or Assigned(MainForm.DMApplication);
end;

procedure TMonitorForm.ShowRecordingWindowActionExecute(Sender: TObject);
var I: integer;
begin
  with MainForm do for I:=0 to MDIChildCount-1 do
  if (MDIChildren[I] as TDataForm).Recording then with MDIChildren[I] do
  begin Show; if WindowState=wsMinimized then WindowState:=wsNormal;Break;end;
end;

procedure TMonitorForm.SetRecordingWindowActionExecute(Sender: TObject);
var I: integer;  // switch recording state to active window (not as in DMW8,9)
begin
  with MainForm do for I:=0 to MDIChildCount-1 do
  if MDIChildren[I]<>ActiveMDIChild then
  (MDIChildren[I] as TDataForm).Recording:=false;
  with MainForm.ActiveDataForm do Recording:=not Recording;
end;

procedure TMonitorForm.SetRecordingWindowActionUpdate(Sender: TObject);
begin
  with MainForm do
  begin
    SetRecordingWindowAction.Enabled:=Assigned(ActiveDataForm);
    SetRecordingWindowAction.Checked:=Assigned(ActiveDataForm) and
    ActiveDataForm.Recording;
  end;
end;

procedure TMonitorForm.SetRecordingSerieActionExecute(Sender: TObject);
begin
  with MainForm.ActiveDataForm do
  IsRecordingSerie[Plot.SerieIndex]:=not IsRecordingSerie[Plot.SerieIndex];
end;

procedure TMonitorForm.SetRecordingSerieActionUpdate(Sender: TObject);
begin
  with MainForm do
  begin
    SetRecordingSerieAction.Enabled:=Assigned(ActiveDataForm);
    SetRecordingSerieAction.Checked:=Assigned(ActiveDataForm) and
    ActiveDataForm.IsRecordingSerie[ActiveDataForm.Plot.SerieIndex];
  end;
end;

procedure TMonitorForm.NextSerieActionExecute(Sender: TObject);
begin
  with MainForm.ActiveDataForm do
  begin
    IsRecordingSerie[Plot.SerieIndex]:=false;
    Worksheet.Col:=Worksheet.FixedCols;          // change worksheet selection
    Worksheet.Row:=Worksheet.FixedRows+Plot.ThisSerie.LastLine+1;
    IsRecordingSerie[Plot.SerieIndex+1]:=true;        // copy serie attributes
    Plot.Series[Plot.SerieIndex+1].XColumn:=Plot.ThisSerie.XColumn;
    Plot.Series[Plot.SerieIndex+1].YColumn:=Plot.ThisSerie.YColumn;
    Plot.Series[Plot.SerieIndex+1].XExpression:=Plot.ThisSerie.XExpression;
    Plot.Series[Plot.SerieIndex+1].YExpression:=Plot.ThisSerie.YExpression;
    Plot.Series[Plot.SerieIndex+1].FirstLine:=Plot.ThisSerie.LastLine{+1!!!!};
    Plot.Series[Plot.SerieIndex+1].LastLine:=Plot.ThisSerie.LastLine;
    MainForm.SerieUpDown.Position:=MainForm.SerieUpDown.Position+1;   // next!
  end;
end;

procedure TMonitorForm.NextSerieActionUpdate(Sender: TObject);
begin       // enabled if current serie is not last, "recording" and not empty
  with MainForm do NextSerieAction.Enabled:=Assigned(ActiveDataForm) and
  (ActiveDataForm.Plot.SerieIndex<ActiveDataForm.Plot.Series.Count-1) and
  ActiveDataForm.IsRecordingSerie[ActiveDataForm.Plot.SerieIndex] and
  (not ActiveDataForm.Plot.ThisSerie.Empty) and ActiveDataForm.Recording
  and (ActiveDataForm.Container.Items.Count>0);
end;

end.

