{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit NotesWnd;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ImgList, StdCtrls, ToolWin, ComCtrls, StdActns, ActnList;

type
  TNotesForm = class(TForm)
    ImageList: TImageList;
    ActionList: TActionList;
    CopyAction: TEditCopy;
    PasteAction: TEditPaste;
    CutAction: TEditCut;
    DeleteAction: TEditDelete;
    UndoAction: TEditUndo;
    OpenAction: TFileOpen;
    SaveAsAction: TFileSaveAs;
    SelectAllAction: TEditSelectAll;
    StatusBar: TStatusBar;
    CoolBar: TCoolBar;
    ToolBar: TToolBar;
    OpenToolButton: TToolButton;
    SaveToolButton: TToolButton;
    ToolButton5: TToolButton;
    CopyToolButton: TToolButton;
    PasteToolButton: TToolButton;
    CutToolButton: TToolButton;
    DeleteToolButton: TToolButton;
    UndoToolButton: TToolButton;
    Memo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OpenActionAccept(Sender: TObject);
    procedure SaveAsActionAccept(Sender: TObject);
    procedure FormStartDock(Sender: TObject;
      var DragObject: TDragDockObject);
    procedure FormEndDock(Sender, Target: TObject; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    { Private declarations }
    procedure wmDropFiles(var M: TWMDropFiles); message WM_DROPFILES;
    procedure UpdateCaption(FName: string);
  protected
    { Protected declarations }
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public declarations }
    FCaptionBuf, FFileName: string;
    procedure UpdateCursorPosition(var S: string);
    procedure LoadFile(FName: string); virtual;
  end;

var NotesForm: TNotesForm;

implementation

uses Strings, ShellAPI, MainWnd;

{$R *.dfm}

{$ifdef SafeMemo}
type
  TSafeMemo=class(TMemo)
  protected
    procedure Hook(var Msg: TWMSETTEXT); message WM_SETTEXT;
  end;

{ TSafeMemo }

procedure TSafeMemo.Hook(var Msg: TWMSETTEXT);
const S: string='';              // bufferize text and repair it when cleared!
begin
  if Msg.Text<>nil then begin inherited; S:=Msg.Text; end
  else SetWindowText(Handle,PChar(S));
end;
{$endif}

{ TNotesForm }

procedure TNotesForm.CreateParams(var Params: TCreateParams);
begin
  if UseTaskBar then FormStyle:=fsNormal; inherited;  
  if UseTaskBar then Params.ExStyle:=Params.ExStyle or WS_EX_APPWINDOW;
end;

procedure TNotesForm.wmDropFiles(var M: TWMDropFiles);
var Filename: array[0..MAX_PATH] of Char; Count: Integer;   // as in MainForm!
begin
  Count:=DragQueryFile(M.Drop,Cardinal(-1),nil,0);      // Get number of files
  try
    if Count>=0 then
    begin
      DragQueryFile(M.Drop, 0, Filename, SizeOf(Filename));   // only one file
      LoadFile(StrPas(FileName));
    end;
  finally
    DragFinish(M.Drop);
  end;
end;

procedure TNotesForm.LoadFile(FName: string);
begin
  if Memo.Modified and (MessageDlg(Format(strCloseQuery,[FCaptionBuf]),
     mtConfirmation,mbYesNoCancel,0)<>mrNo) then Exit;
  Memo.Lines.LoadFromFile(FName); UpdateCaption(FName);
end;

procedure TNotesForm.UpdateCursorPosition(var S: string);  // called from Idle
var X,Y: integer;
begin
  Y:=Memo.Perform(EM_LINEFROMCHAR, Memo.SelStart, 0);
  X:=Memo.Perform(EM_LINEINDEX, Y, 0);
  S:=Format(strNotes,[Y+1,Memo.SelStart-X+1]);
end;

procedure TNotesForm.UpdateCaption(FName: string);
begin
 FFileName:=ExtractFileName(FName); Caption:=FFileName+' - '+FCaptionBuf;
end;

procedure TNotesForm.OpenActionAccept(Sender: TObject);
begin LoadFile(OpenAction.Dialog.FileName); end;

procedure TNotesForm.SaveAsActionAccept(Sender: TObject);
begin
  Memo.Lines.SaveToFile(SaveAsAction.Dialog.FileName); Memo.Modified:=false;
  UpdateCaption(SaveAsAction.Dialog.FileName);
end;

// 4 handlers enable drag&drop from file managers
procedure TNotesForm.FormCreate(Sender: TObject);
begin
  DragAcceptFiles(Handle, true);
  FCaptionBuf:=Caption; UpdateCaption(strUntitled);
{$ifdef SafeMemo}
  Memo.Free; Memo:=TSafeMemo.Create(Self);
  Memo.Parent:=Self; Memo.Align:=alClient; Memo.Font.Name:='Fixedsys';
  Memo.WordWrap:=false; Memo.ScrollBars:=ssBoth;
  Memo.Name:='Memo'; Memo.Text:=''; Memo.BorderStyle:=bsNone;
{$endif}
end;

procedure TNotesForm.FormDestroy(Sender: TObject);
begin DragAcceptFiles(Handle, false); end;

procedure TNotesForm.FormStartDock(Sender: TObject;
  var DragObject: TDragDockObject);
begin DragAcceptFiles(Handle, false); end;

procedure TNotesForm.FormEndDock(Sender, Target: TObject; X, Y: Integer);
begin DragAcceptFiles(Handle, true); end;

//////// DOCKING SUPPORT ///////////

procedure TNotesForm.FormHide(Sender: TObject);
begin
  if (Parent is TPanel) and (Parent.VisibleDockClientCount=0)
  then MainForm.ShowDockPanel(Parent as TPanel, false);
end;

procedure TNotesForm.FormShow(Sender: TObject);
begin
  if (Parent is TPanel) and (Parent.VisibleDockClientCount=1) and
  MainForm.DocksLoaded then MainForm.ShowDockPanel(Parent as TPanel, true);
end;

end.
