{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit ScrptWnd;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, NotesWnd, StdCtrls, StdActns, ActnList, ImgList, ComCtrls,
  ToolWin, ListActns, ExtCtrls;

type
  TScriptingForm = class(TNotesForm)
    CompileAction: TAction;
    RunAction: TAction;
    EvalAction: TAction;
    ScriptToolBar: TToolBar;
    RunToolButton: TToolButton;
    EvalToolButton: TToolButton;
    CompileToolButton: TToolButton;
    LanguageComboBox: TComboBox;
    ToolButton1: TToolButton;
    ProcedureAction: TStaticListAction;
    ToolButton2: TToolButton;
    ProcedureComboBoxEx: TComboBoxEx;
    CompileVBSAction: TAction;
    CompileJSAction: TAction;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RunActionExecute(Sender: TObject);
    procedure EvalActionExecute(Sender: TObject);
    procedure CompileActionExecute(Sender: TObject);
    procedure ProcedureActionItemSelected(Sender: TCustomListAction;
      Control: TControl);
    procedure CompileMenuActionUpdate(Sender: TObject);
    procedure CompileMenuActionExecute(Sender: TObject);
  private
    { Private declarations }
    ScriptingHost, DMApplication: variant;
    procedure ShowError(Msg: string);
  public
    { Public declarations }
    procedure LoadFile(FName: string); override;
  end;

var ScriptingForm: TScriptingForm;

implementation

uses
  ComObj, Automate, MainWnd, Common, Strings, AppDlg,
  ActiveX, MSScriptControl_TLB{both only for parameters!};

{$R *.DFM}

{ TScriptingForm }

procedure TScriptingForm.LoadFile(FName: string);
begin
  inherited;
  if UpperCase(ExtractFileExt(FName))='.VBS'
  then LanguageComboBox.ItemIndex:=0;
  if UpperCase(ExtractFileExt(FName))='.JS'
  then LanguageComboBox.ItemIndex:=1;
end;

procedure TScriptingForm.ShowError(Msg: string);
var S: string[200]; N,L,C,F,I,SelPos: integer;
begin                               // Format() incorrectly work with variants
  S:=ScriptingHost.Error.Text; L:=ScriptingHost.Error.Line;
  C:=ScriptingHost.Error.Column; N:=ScriptingHost.Error.Number;
  SelPos:=0; for I:=0 to L-2 do SelPos:=SelPos+Length(Memo.Lines[I])+2;
  Memo.SelStart:=SelPos; Memo.SelLength:=Length(Memo.Lines[L-1]);
  F:=SendMessage(Memo.Handle, EM_GETFIRSTVISIBLELINE, 0, 0);
  SendMessage(Memo.Handle, EM_LINESCROLL, 0, L-F-1);          // scroll editor
  MessageDlg(Format(strScriptError,[N,Msg,S,L,C]), mtError, [mbCancel], 0);
  ScriptingHost.Error.Clear; Memo.SetFocus;
end;

/// Event handlers ///

procedure TScriptingForm.FormShow(Sender: TObject);
begin                                         // design-time values corrupted!
  inherited; LanguageComboBox.Top:=2; ProcedureComboBoxEx.Top:=2;
end;

procedure TScriptingForm.FormCreate(Sender: TObject);
begin
  inherited;
  try
    ScriptingHost:=CreateOleObject('MSScriptControl.ScriptControl');
    ScriptingHost.SitehWnd:=Application.Handle;     // make messageboxes modal
    with LanguageComboBox do
    begin ItemIndex:=0; ScriptingHost.Language:=Items[ItemIndex]; end;
    DMApplication:=TDMInternalApp.Create as IDispatch;
  except
    on EOleSysError do MainForm.ScriptAction.Enabled:=false;
  end;
end;

procedure TScriptingForm.FormDestroy(Sender: TObject);
begin inherited; VarClear(ScriptingHost);{?} end;

procedure TScriptingForm.RunActionExecute(Sender: TObject);
var V,Par: variant; I,NPar: integer; PName: string;
begin
  try
    if AppPropsForm.ScriptCheckBox.Checked
    then I:=AppPropsForm.ScriptSpinEdit.Value*1000 else I:=-1;
    ScriptingHost.Timeout:=I;
    I:=ProcedureAction.ItemIndex+1;
    NPar:=ScriptingHost.Modules['Global'].Procedures[I].NumArgs;
    PName:=ScriptingHost.Modules['Global'].Procedures[I].Name;
    if NPar>0 then
    begin
      Par:=VarArrayCreate([0,NPar-1],varVariant);
      for I:=0 to NPar-1 do
      Par[I]:=InputBox(strEnter,Format(strScriptParams,[I+1]),'');
    end;
    try
      Screen.Cursor:=crHourGlass;
      if not Floating then with MainForm do
      begin StatusImageList.Tag:=8; DisplayHint(strWorking); end else
      begin StatusBar.Panels[2].Text:=strWorking; StatusBar.Refresh; end;
      if NPar=0 then V:=ScriptingHost.Run(PName) else
      V:=IScriptControl(TVarData(ScriptingHost).VDispatch).Run(PName,
      PSafeArray(TVarData(Par).VArray));
    finally
      if not Floating then with MainForm do
      begin StatusImageList.Tag:=0; DisplayHint(''); end
      else StatusBar.Panels[2].Text:='';
      Screen.Cursor:=crDefault;
    end;
    if not VarIsEmpty(V) then
    MessageDlg(strScriptResult+CRLF+VarToStr(V),mtInformation,[mbOk],0);
  except
    on E: EOleException do ShowError(E.Message);
  end;
end;

procedure TScriptingForm.EvalActionExecute(Sender: TObject);
const S: string='';
var V: variant;
begin
  if InputQuery(strEnter,strScriptEval,S) then
  begin
    V:=ScriptingHost.Eval(S);
    if not VarIsEmpty(V) then
    MessageDlg(strScriptResult+CRLF+VarToStr(V),mtInformation,[mbOk],0)
    else MessageDlg(strScriptUndefined,mtInformation,[mbCancel],0);
  end;
end;

procedure TScriptingForm.CompileActionExecute(Sender: TObject);
var I,N: integer; S: string;
begin
  Screen.Cursor:=crHourGlass;
  if not Floating then with MainForm do
  begin StatusImageList.Tag:=8; DisplayHint(strCompiling); end else
  begin StatusBar.Panels[2].Text:=strCompiling; StatusBar.Refresh; end;
  ProcedureAction.Items.Clear;
  ProcedureActionItemSelected(nil,nil);
  ProcedureAction.Active:=false;
  try
    try
      with LanguageComboBox do ScriptingHost.Language:=Items[ItemIndex];
      ScriptingHost.Reset;
      ScriptingHost.AddObject('Server', DMApplication);
      ScriptingHost.AddCode(Memo.Text);
      for I:=1 to ScriptingHost.Modules['Global'].Procedures.Count do
      begin
        N:=ScriptingHost.Modules['Global'].Procedures[I].NumArgs;
        S:=ScriptingHost.Modules['Global'].Procedures[I].Name;
        with ProcedureAction.Items.Add do
        if N=0 then Caption:=S else Caption:=Format('%s(%d par-s)',[S,N]);
      end;
      ProcedureAction.Active:=true;
    finally
      if not Floating then with MainForm do
      begin StatusImageList.Tag:=0; DisplayHint(''); end
      else StatusBar.Panels[2].Text:='';
      Screen.Cursor:=crDefault;
    end;
  except
    on E: EOleException do ShowError(E.Message);
  end;
end;

procedure TScriptingForm.ProcedureActionItemSelected(
  Sender: TCustomListAction; Control: TControl);
begin
  if Control is TCustomCombo
  then ProcedureAction.ItemIndex:=(Control as TCustomCombo).ItemIndex
  else ProcedureAction.ItemIndex:=-1;
  RunAction.Enabled:=ProcedureAction.ItemIndex>=0;
end;

procedure TScriptingForm.CompileMenuActionUpdate(Sender: TObject);
begin
  CompileVBSAction.Checked:=LanguageComboBox.ItemIndex=0;
  CompileJSAction.Checked:=LanguageComboBox.ItemIndex=1;
end;

procedure TScriptingForm.CompileMenuActionExecute(Sender: TObject);
begin
  if Sender=CompileVBSAction then LanguageComboBox.ItemIndex:=0
  else LanguageComboBox.ItemIndex:=1;
end;

end.

