{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit SetupDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, ComCtrls, StdCtrls, Buttons, ExtCtrls;

type
  TSetupForm = class(TForm)
    CloseBitBtn: TBitBtn;
    HelpBitBtn: TBitBtn;
    PageControl: TPageControl;
    InstallTabSheet: TTabSheet;
    ImageList: TImageList;
    UninstallTabSheet: TTabSheet;
    OLERegisterGroupBox: TGroupBox;
    InstallScriptBitBtn: TBitBtn;
    InstallProppageBitBtn: TBitBtn;
    FileGroupBox: TGroupBox;
    DesktopFileExtBitBtn: TBitBtn;
    DataFileExtBitBtn: TBitBtn;
    ExtensionLabel: TLabel;
    ExtensionEdit: TEdit;
    ProppageCheckBox: TCheckBox;
    DMW9SpeedButton: TSpeedButton;
    OLEUnRegisterGroupBox: TGroupBox;
    UnInstallScriptBitBtn: TBitBtn;
    UnInstallProppageBitBtn: TBitBtn;
    UnInstallDM2000BitBtn: TBitBtn;
    DelFileGroupBox: TGroupBox;
    DelExtensionLabel: TLabel;
    DelExtensionComboBox: TComboBox;
    DelExtensionSpeedButton: TSpeedButton;
    DelRegistryBitBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    MBRegisterGroupBox: TGroupBox;
    InstallProviderBitBtn: TBitBtn;
    InstallProviderCheckBox: TCheckBox;
    DescriptionLabel: TLabel;
    DescriptionEdit: TEdit;
    Bevel: TBevel;
    MBUnRegisterGroupBox: TGroupBox;
    DelProviderSpeedButton: TSpeedButton;
    ProviderComboBox: TComboBox;
    procedure InstallTabSheetShow(Sender: TObject);
    procedure UninstallTabSheetShow(Sender: TObject);
    procedure InstallScriptBitBtnClick(Sender: TObject);
    procedure UnInstallScriptBitBtnClick(Sender: TObject);
    procedure InstallProppageBitBtnClick(Sender: TObject);
    procedure UnInstallProppageBitBtnClick(Sender: TObject);
    procedure UnInstallDM2000BitBtnClick(Sender: TObject);
    procedure InstallProviderBitBtnClick(Sender: TObject);
    procedure DesktopFileExtBitBtnClick(Sender: TObject);
    procedure DataFileExtBitBtnClick(Sender: TObject);
    procedure DMW9SpeedButtonClick(Sender: TObject);
    procedure DelRegistryBitBtnClick(Sender: TObject);
    procedure DelExtensionSpeedButtonClick(Sender: TObject);
    procedure DelExtensionComboBoxChange(Sender: TObject);
    procedure ProviderComboBoxChange(Sender: TObject);
    procedure DelProviderSpeedButtonClick(Sender: TObject);
  private
    { Private declarations }
    function AppPath: string;
    procedure Exec(CMD: string);
    function SCRegistered: boolean;
  public
    { Public declarations }
  end;

implementation

uses Registry, ComObj, MainWnd, AppDlg;

{$R *.dfm}

function TSetupForm.AppPath: string;
begin Result:=ExtractFilePath(Application.ExeName); end;

procedure TSetupForm.Exec(CMD: string);
var CMDLine: array[0..MAX_PATH] of char;
begin
  StrPCopy(CMDLine,CMD); if WinExec(CMDLine,SW_SHOWDEFAULT)<32
  then MessageDlg('Unable to run helper application!',mtError,[mbCancel],0);
end;

function TSetupForm.SCRegistered: boolean;
var SH: variant;
begin                             // returns true if can create script control
  try
    Result:=true;
    SH:=CreateOleObject('MSScriptControl.ScriptControl');
    SH:=Unassigned;
  except
    on EOleSysError do Result:=false;
  end;
end;

///////// Event handlers /////////////////

procedure TSetupForm.InstallTabSheetShow(Sender: TObject);
begin
  InstallScriptBitBtn.Enabled:=(not SCRegistered) and
    FileExists(AppPath+'msscript.ocx');
  InstallProppageBitBtn.Enabled:=FileExists(AppPath+'proppage.dll');
end;

procedure TSetupForm.UninstallTabSheetShow(Sender: TObject);
var S,SS,ExtRoot: string; R: TRegistry; SL: TStringList; I: integer;
begin
  UnInstallProppageBitBtn.Enabled:=FileExists(AppPath+'proppage.dll');
  R:=TRegistry.Create;
  try
    R.RootKey:=HKEY_CLASSES_ROOT;
    if SCRegistered then
    begin
      S:='CLSID\'+
      GUIDToString(ProgIDToClassID('MSScriptControl.ScriptControl'))
      +'\InprocServer32';
      UnInstallScriptBitBtn.Enabled:=R.OpenKey(S,false) and
      (UpperCase(R.ReadString(''))=UpperCase(AppPath+'msscript.ocx')) and
      FileExists(AppPath+'msscript.ocx');
    end else UnInstallScriptBitBtn.Enabled:=false;
    // now fill extension list!
    SL:=TStringList.Create;
    try
      R.CloseKey;
      R.OpenKeyReadOnly('');
      Screen.Cursor:=crHourGlass;
      R.GetKeyNames(SL);
      R.CloseKey;
      DelExtensionComboBox.Clear; DelExtensionComboBox.ItemIndex:=-1;
      DelExtensionSpeedButton.Enabled:=false;
      for I:=0 to SL.Count-1 do
      begin
        S:=SL.Strings[I];
        if S[1]='.' then
        begin
          R.OpenKeyReadOnly(S);
          ExtRoot:=R.ReadString('');
          R.CloseKey;
          R.OpenKeyReadOnly(ExtRoot+'\DefaultIcon');
          SS:=UpperCase(R.ReadString(''));
          R.CloseKey;
          if Pos('DM2000.EXE',SS)>0 then
          DelExtensionComboBox.Items.Add(Copy(S,2,Length(S)));
        end;
      end;
    finally
      SL.Free; Screen.Cursor:=crDefault;
    end;
    // read provider list
    R.RootKey:=HKEY_CURRENT_USER;
    ProviderComboBox.Clear; ProviderComboBox.ItemIndex:=-1;
    if R.OpenKeyReadOnly(MetaBaseRegKey+'\Providers') then
    begin R.GetValueNames(ProviderComboBox.Items); R.CloseKey; end;
    DelProviderSpeedButton.Enabled:=false;
  finally
    R.Free;
  end;
end;

procedure TSetupForm.InstallScriptBitBtnClick(Sender: TObject);
begin
  Exec('regsvr32.exe "'+AppPath+'msscript.ocx"');
  InstallTabSheetShow(Sender);                               // disable button
end;

procedure TSetupForm.UnInstallScriptBitBtnClick(Sender: TObject);
begin
  Exec('regsvr32.exe /u "'+AppPath+'msscript.ocx"');
  UninstallTabSheetShow(Sender);                             // disable button
end;

procedure TSetupForm.InstallProppageBitBtnClick(Sender: TObject);
begin
  Exec('regsvr32.exe "'+AppPath+'proppage.dll"');
  CreateRegKey(
    'Software\Microsoft\Windows\CurrentVersion\Shell Extensions\Approved',
    GUIDToString(Class_DataPropertySheet),
    'DM2000 MetaBase Shell Extension',HKEY_LOCAL_MACHINE);
end;

procedure TSetupForm.UnInstallProppageBitBtnClick(Sender: TObject);
begin Exec('regsvr32.exe /u "'+AppPath+'proppage.dll"'); end;

procedure TSetupForm.UnInstallDM2000BitBtnClick(Sender: TObject);
begin Exec(Application.ExeName+' /unregserver'); end;

procedure TSetupForm.InstallProviderBitBtnClick(Sender: TObject);
var S: string;
begin
  OpenDialog.InitialDir:=AppPath; OpenDialog.FileName:='';
  OpenDialog.Options:=OpenDialog.Options+[ofNoLongNames];
  OpenDialog.FilterIndex:=1;
  if OpenDialog.Execute then
  begin
    S:='rundll32.exe '+OpenDialog.FileName+',RegisterProvider';
    if InstallProviderCheckBox.Checked then S:=S+' /setup';
    Exec(S);
  end;
end;

procedure TSetupForm.DMW9SpeedButtonClick(Sender: TObject);
begin
  OpenDialog.FileName:='dmw32.exe'; OpenDialog.FilterIndex:=2;
  OpenDialog.Options:=OpenDialog.Options-[ofNoLongNames];
  if OpenDialog.Execute then with TRegistry.Create do
  try
    OpenKey(DataMasterRegKey,true); WriteString('DMW9',OpenDialog.FileName);
  finally
    Free;
  end;
end;

procedure TSetupForm.DesktopFileExtBitBtnClick(Sender: TObject);
const EF: integer=$10000;
begin
  CreateRegKey('.dmw','','DMWFile');
  with TRegistry.Create do
  try
    RootKey:=HKEY_CLASSES_ROOT;
    OpenKey('DMWFile',true);
    WriteString('','DM2000 desktop file');
    WriteString('AlwaysShowExt','');
    WriteInteger('BrowserFlags',8);
    WriteBinaryData('EditFlags',EF,4);
    CloseKey;
  finally
    Free;
  end;
  CreateRegKey('DMWFile\DefaultIcon','',Application.ExeName+',0');
  CreateRegKey('DMWFile\shell','','Open');
  CreateRegKey('DMWFile\shell\Open\command','',Application.ExeName+' "%1"');
  if ProppageCheckBox.Checked then
  CreateRegKey('DMWFile\shellex\PropertySheetHandlers\PropPage','',
  GUIDToString(Class_DataPropertySheet));
end;

procedure TSetupForm.DataFileExtBitBtnClick(Sender: TObject);
const EF: integer=$10000;
var Ext,ExtRoot: string;
begin
  Ext:=ExtensionEdit.Text;
  if Pos('.',Ext)<>0 then Delete(Ext,Pos('.',Ext),1);         // double-check!
  ExtRoot:=UpperCase(Ext)+'File';
  CreateRegKey('.'+Ext,'',ExtRoot);
  CreateRegKey('.'+Ext,'Content Type','text/plain');
  CreateRegKey('.'+Ext+'\ShellNew','NullFile','');
  with TRegistry.Create do
  try
    RootKey:=HKEY_CLASSES_ROOT;
    OpenKey(ExtRoot, true);
    WriteString('', DescriptionEdit.Text);
    WriteString('AlwaysShowExt', '');
    WriteInteger('BrowserFlags', 8);
    WriteBinaryData('EditFlags', EF, 4);
    CloseKey;
  finally
    Free;
  end;
  CreateRegKey(ExtRoot+'\DefaultIcon','',Application.ExeName+',1');
  CreateRegKey(ExtRoot+'\QuickView','','*');                           // W9x!
  CreateRegKey(ExtRoot+'\shell','','Open');
  CreateRegKey(ExtRoot+'\shell\Open\command','',Application.ExeName+' "%1"');
  CreateRegKey(ExtRoot+'\shell\Open\ddeexec','','Open=%1');
  CreateRegKey(ExtRoot+'\shell\Open\ddeexec\ifexec','',Application.ExeName);
  CreateRegKey(ExtRoot+'\shell\Open\ddeexec\topic','','DdeServerConv');
  CreateRegKey(ExtRoot+'\shell\Open\ddeexec\Application','','DM2000'); // W2K!
  CreateRegKey(ExtRoot+'\shell\Open in Notepad\command','','notepad.exe "%1"');
  if ProppageCheckBox.Checked then
  CreateRegKey(ExtRoot+'\shellex\PropertySheetHandlers\PropPage','',
  GUIDToString(Class_DataPropertySheet));
end;

procedure TSetupForm.DelRegistryBitBtnClick(Sender: TObject);
begin
  with TRegistry.Create do
  try
    DeleteKey('SOFTWARE\RRR\DM2000');
  finally                // warning: delete non-empty key may not work on NT!?
    Free;                                        // on W2K and W9.x all works!
  end;
  with AppPropsForm.SaveOnExitCheckBox do
  begin Checked:=false; Enabled:=false; end; 
end;

procedure TSetupForm.DelExtensionSpeedButtonClick(Sender: TObject);
var Ext,ExtRoot: string;
begin
  if DelExtensionComboBox.ItemIndex=-1 then Exit;
  with DelExtensionComboBox do Ext:=Items[ItemIndex];
  with TRegistry.Create do
  try
    RootKey:=HKEY_CLASSES_ROOT;
    OpenKeyReadOnly('.'+Ext);
    ExtRoot:=ReadString('');
    CloseKey;
    DeleteKey('.'+Ext);
    DeleteKey(ExtRoot);
  finally
    Free;
  end;
  UninstallTabSheetShow(Sender);            // update combo and other controls
end;

procedure TSetupForm.DelExtensionComboBoxChange(Sender: TObject);
begin
  DelExtensionSpeedButton.Enabled:=DelExtensionComboBox.ItemIndex<>-1;
end;

procedure TSetupForm.ProviderComboBoxChange(Sender: TObject);
begin
  DelProviderSpeedButton.Enabled:=ProviderComboBox.ItemIndex<>-1;
end;

procedure TSetupForm.DelProviderSpeedButtonClick(Sender: TObject);
var
  LH: THandle; FN: array[0..MAX_PATH] of char;
  UnRegisterProvider: procedure(hwnd: HWND; hinst: integer; CmdLine: pchar;
  nCmdShow: integer); stdcall;
begin
  if ProviderComboBox.ItemIndex=-1 then Exit;
  with TRegistry.Create do
  try
    if OpenKeyReadOnly(MetaBaseRegKey+'\Providers') then
    with ProviderComboBox do StrPCopy(FN,ReadString(Items[ItemIndex]))
    else Exit;
  finally
    Free;
  end;
  LH:=LoadLibrary(FN); if LH=0 then Exit;
  UnRegisterProvider:=GetProcAddress(LH,'UnRegisterProvider');
  if Assigned(UnRegisterProvider)
  then UnRegisterProvider(Handle, hInstance, nil, SW_SHOWDEFAULT);
  FreeLibrary(LH); UninstallTabSheetShow(Sender);
end;

end.
