{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit TransDlg;
{$B-}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, Plot, ComCtrls, Spin, FloatEd;

type
  TTransformationForm = class(TForm)
    OkBitBtn: TBitBtn;
    CancelBitBtn: TBitBtn;
    HelpBitBtn: TBitBtn;
    ApplyBitBtn: TBitBtn;
    PageControl: TPageControl;
    LinearTabSheet: TTabSheet;
    CustomTabSheet: TTabSheet;
    Bevel: TBevel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    DataRadioGroup: TRadioGroup;
    XELabel: TLabel;
    YELabel: TLabel;
    XAFloatEdit: TFloatEdit;
    YAFloatEdit: TFloatEdit;
    XBFloatEdit: TFloatEdit;
    YBFloatEdit: TFloatEdit;
    P1Label: TLabel;
    P1RLabel: TLabel;
    P1MinEdit: TEdit;
    P1MaxEdit: TEdit;
    P1TrackBar: TTrackBar;
    P1Edit: TEdit;
    P2Label: TLabel;
    P2Edit: TEdit;
    P2MinEdit: TEdit;
    P2RLabel: TLabel;
    P2MaxEdit: TEdit;
    P2TrackBar: TTrackBar;
    XTextSpeedButton: TSpeedButton;
    YTextSpeedButton: TSpeedButton;
    XComboBox: TComboBox;
    YComboBox: TComboBox;
    AutoApplyCheckBox: TCheckBox;
    CalculateCheckBox: TCheckBox;
    procedure ApplyBitBtnClick(Sender: TObject);
    procedure P1TrackBarChange(Sender: TObject);
    procedure P2TrackBarChange(Sender: TObject);
    procedure XTextSpeedButtonClick(Sender: TObject);
    procedure YTextSpeedButtonClick(Sender: TObject);
    procedure FloatEditChange(Sender: TObject);
  private
    { Private declarations }
    Serie: TSerie;
    x1,x2,y1,y2: string[50];
    xexpr,yexpr: string;
  public
    { Public declarations }
  end;

procedure Transform;

implementation

uses DataWnd, MainWnd, Strings, Data, TextDlg, CalcDlg;

{$R *.DFM}

procedure Transform;
var TransformationForm: TTransformationForm; I: integer;
    D: TRealData; X,Y: extended;
begin
  if not Assigned(MainForm.ActiveDataForm) then Exit;
  with MainForm.ActiveDataForm.Plot.ThisSerie do
  if ((XExpression<>'') or (YExpression<>'')) and
     (MessageDlg(strSerieEmptyExpr, mtWarning, [mbOk,mbCancel], 0)=mrCancel)
  then Exit;
  TransformationForm:=TTransformationForm.Create(Application);
  TransformationForm.HelpBitBtn.OnClick:=MainForm.HelpButtonClick;
  try
    with TransformationForm.DataRadioGroup do
    begin
      Enabled:=MainForm.ActiveDataForm.Plot.SelectionVisible;
      if not Enabled then ItemIndex:=1;
    end;
    with TransformationForm, MainForm.ActiveDataForm do
    begin
      with Plot, Plot.XAxis do
      begin
        x1:=FloatToStrF(SelectionLeft,LabelType,LabelWidth,LabelDecimals);
        x2:=FloatToStrF(SelectionRight,LabelType,LabelWidth,LabelDecimals);
        XBFloatEdit.Increment:=(Max-Min)/MajorTicks/MinorTicks;
      end;
      with Plot, Plot.YAxis do
      begin
        y1:=FloatToStrF(SelectionBottom,LabelType,LabelWidth,LabelDecimals);
        y2:=FloatToStrF(SelectionTop,LabelType,LabelWidth,LabelDecimals);
        YBFloatEdit.Increment:=(Max-Min)/MajorTicks/MinorTicks;
      end;
      Serie:=Plot.ThisSerie;
      xexpr:=Serie.XExpression; yexpr:=Serie.YExpression;         // remember!
      if (xexpr<>'') or (yexpr<>'') then PageControl.ActivePageIndex:=1;
      if xexpr<>'' then XComboBox.Text:=xexpr;             // copy expressions
      if yexpr<>'' then YComboBox.Text:=yexpr;
    end;
    with TransformationForm do
    begin
      XComboBox.Items.Assign(CalculatorForm.ExpressionComboBox.Items);
      YComboBox.Items.Assign(XComboBox.Items); // copy history from calculator
    end;
    if TransformationForm.ShowModal=mrOk then       // apply transformation...
    with TransformationForm, MainForm.ActiveDataForm.Plot do
    begin
      with XComboBox do                                        // save history
      if (Text<>'') and (Items.IndexOf(Text)=-1) then Items.Insert(0, Text);
      with YComboBox do if (Text<>'') and (Items.IndexOf(Text)=-1)
      then XComboBox.Items.Insert(0, Text);                // add to XCombo!!!
      if PageControl.ActivePageIndex=1 then     // save only for "custom" mode
      CalculatorForm.ExpressionComboBox.Items.Assign(XComboBox.Items);
      if (Serie.XExpression='') and (Serie.YExpression='') then Exit;     // ?
      if not CalculateCheckBox.Checked then Exit;
      ((Serie.Collection as TSeries).Plot.Owner as
      TDataForm).SaveUndoInfo(MainForm.TransformationAction);
      // MainForm.ActiveDataForm.SaveUndoInfo;   <-- this is wrong!!!
      MainForm.StatusImageList.Tag:=8; MainForm.DisplayHint(strWorking);
      Screen.Cursor:=crHourGlass;
      try
        with Serie do
        begin
          for I:=FirstLine to LastLine do                 // apply expressions
          begin
            D:=Container.Items[I];
            (Collection as TSeries).Plot.GetDataPoint(D,XColumn,YColumn,
              XExpression,YExpression,X,Y);
            D.RData[XColumn]:=X; D.RData[YColumn]:=Y;
          end;
          XExpression:=''; YExpression:=''; // clear expressions after transf.
        end;
      finally
        MainForm.StatusImageList.Tag:=0; MainForm.DisplayHint('');
        Screen.Cursor:=crDefault; Serie.Container.Modified:=true;
      end;
    end else with TransformationForm do           // restore serie expressions
    begin Serie.XExpression:=xexpr; Serie.YExpression:=yexpr; end;
  finally
    TransformationForm.Free; MainForm.UpdateSerieList;
  end;
end;

procedure TTransformationForm.ApplyBitBtnClick(Sender: TObject);
  function doPR(PRn,PRv,S: string): string;    // process "P1","P2" parameters
  var ps: integer;
  begin
   repeat
     ps:=Pos(PRn,UpperCase(S));
     if ps<>0 then begin Delete(S,ps,2); Insert(PRv,S,ps); end;
   until ps=0;
   Result:=S;
  end;
begin
  if PageControl.ActivePageIndex=0 then
  begin
    if DataRadioGroup.Itemindex=1 then                          // whole serie
    begin
      XComboBox.Text:='cx*'+XAFloatEdit.Text+'+'+XBFloatEdit.Text;
      YComboBox.Text:='cy*'+YAFloatEdit.Text+'+'+YBFloatEdit.Text;
    end else                                                 // only selection
    begin
    { This version works correctly, but too slow because of long expressions!!
      XEdit.Text:='(cx>='+x1+')*(cx<='+x2+')*(cy>='+y1+')*(cy<='+y2+
      ')*(cx*'+XAEdit.Text+'+'+XBEdit.Text+')+cx*(1-(cx>='+x1+')*(cx<='+x2+
      ')*(cy>='+y1+')*(cy<='+y2+'))';
      YEdit.Text:='(cx>='+x1+')*(cx<='+x2+')*(cy>='+y1+')*(cy<='+y2+
      ')*(cy*'+YAEdit.Text+'+'+YBEdit.Text+')+cy*(1-(cx>='+x1+')*(cx<='+x2+
      ')*(cy>='+y1+')*(cy<='+y2+'))'; }
      //if (XAFloatEdit.Text<>'1') or (XBFloatEdit.Text<>'0') then
      XComboBox.Text:=Format('Translate(%s,%s,%s,%s,%s,%s,%s,%s,%s)',
      [x1,x2,y1,y2,'cx','cy',XAFloatEdit.Text,XBFloatEdit.Text,'cx']);
      //if (YAFloatEdit.Text<>'1') or (YBFloatEdit.Text<>'0') then
      YComboBox.Text:=Format('Translate(%s,%s,%s,%s,%s,%s,%s,%s,%s)',
      [x1,x2,y1,y2,'cx','cy',YAFloatEdit.Text,YBFloatEdit.Text,'cy']);
    end;
  end;
  Serie.XExpression:=doPR('P1',P1Edit.Text,
  doPR('P2',P2Edit.Text,XComboBox.Text));
  Serie.YExpression:=doPR('P1',P1Edit.Text,
  doPR('P2',P2Edit.Text,YComboBox.Text));
  MainForm.UpdateSerieList;
end;

procedure TTransformationForm.P1TrackBarChange(Sender: TObject);
begin                                                     // PRTrackBar.MIN=0!
  with P1TrackBar do P1Edit.Text:=FloatToStrF(StrToFloat(P1MinEdit.Text)+
  (StrToFloat(P1MaxEdit.Text)-StrToFloat(P1MinEdit.Text))/Max*Position,
  ffGeneral,7,4); if AutoApplyCheckBox.Checked then ApplyBitBtn.Click;
end;

procedure TTransformationForm.P2TrackBarChange(Sender: TObject);
begin
  with P2TrackBar do P2Edit.Text:=FloatToStrF(StrToFloat(P2MinEdit.Text)+
  (StrToFloat(P2MaxEdit.Text)-StrToFloat(P2MinEdit.Text))/Max*Position,
  ffGeneral,7,4); if AutoApplyCheckBox.Checked then ApplyBitBtn.Click;
end;

procedure TTransformationForm.XTextSpeedButtonClick(Sender: TObject);
var S: string;
begin
  S:=XComboBox.Text; if InputText(S,XComboBox) then XComboBox.Text:=S;
end;

procedure TTransformationForm.YTextSpeedButtonClick(Sender: TObject);
var S: string;
begin
  S:=YComboBox.Text; if InputText(S,YComboBox) then YComboBox.Text:=S;
end;

procedure TTransformationForm.FloatEditChange(Sender: TObject);
begin if AutoApplyCheckBox.Checked then ApplyBitBtn.Click; end;

end.
