{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit WorksDlg;
{$B-}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Spin, ExtCtrls, ComCtrls, StdCtrls, ImgList, Buttons, Grids, ColorGrd,
  DataWnd, WorkGrid, Data, Common;

type
  TWorksheetPropsForm = class(TForm)
    OkBitBtn: TBitBtn;
    CancelBitBtn: TBitBtn;
    HelpBitBtn: TBitBtn;
    PageControl: TPageControl;
    ImageList: TImageList;
    ViewTabSheet: TTabSheet;
    ColumnsTabSheet: TTabSheet;
    FontDialog: TFontDialog;
    ColorDialog: TColorDialog;
    ItemRadioGroup: TRadioGroup;
    FontBitBtn: TBitBtn;
    LabelsStringGrid: TStringGrid;
    SetAsXButton: TButton;
    SetAsYButton: TButton;
    FormatGroupBox: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    SampleLabel: TLabel;
    WidthSpinEdit: TSpinEdit;
    FormatRadioGroup: TRadioGroup;
    DigitsSpinEdit: TSpinEdit;
    ColorGroupBox: TGroupBox;
    ColorGrid: TColorGrid;
    ColorBitBtn: TBitBtn;
    SampleWorksheet: TWorksheet;
    Container: TContainer;
    AlignRightCheckBox: TCheckBox;
    Label1: TLabel;
    ColumnSpinEdit: TSpinEdit;
    ColumnLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FontBitBtnClick(Sender: TObject);
    procedure ColorBitBtnClick(Sender: TObject);
    procedure SetAsXButtonClick(Sender: TObject);
    procedure SetAsYButtonClick(Sender: TObject);
    procedure FormatChange(Sender: TObject);
    procedure ColumnSpinEditChange(Sender: TObject);
    procedure AlignRightCheckBoxClick(Sender: TObject);
    procedure LabelsStringGridExit(Sender: TObject);
    procedure FormatRadioGroupClick(Sender: TObject);
    procedure ColorGridChange(Sender: TObject);
  private
    { Private declarations }
    XCol, YCol: integer;
    DataFormat: TFormatArray;
    procedure UpdateFormatSample;
    procedure UpdateLabelsStringGrid;
  public
    { Public declarations }
    procedure Execute(Wnd: TDataForm);
  end;

var WorksheetPropsForm: TWorksheetPropsForm;

implementation

uses Strings;

{$R *.DFM}

{ TWorksheetPropsForm }

procedure TWorksheetPropsForm.Execute(Wnd: TDataForm);
var I: integer;
begin
  XCol:=Wnd.XColumn; YCol:=Wnd.YColumn;   // copy data from window to controls
  LabelsStringGrid.RowCount:=Wnd.Worksheet.Header.Count+1{1 fixed!};
  for I:=1 to Wnd.Worksheet.Header.Count do
  begin
    LabelsStringGrid.Cells[0,I]:=char(byte('A')+I-1);
    LabelsStringGrid.Cells[2,I]:=Wnd.Worksheet.Header[I-1];
  end;
  UpdateLabelsStringGrid;   // update grid after setting XCol, YCol & RowCount
  DataFormat:=Wnd.DataFormat; ColumnSpinEditChange(Self);
  AlignRightCheckBox.Checked:=Wnd.Worksheet.AlignRight;
  SampleWorksheet.Color:=Wnd.Worksheet.Color;
  SampleWorksheet.FixedColor:=Wnd.Worksheet.FixedColor;
  SampleWorksheet.BlockColorF:=Wnd.Worksheet.BlockColorF;
  SampleWorksheet.BlockColorB:=Wnd.Worksheet.BlockColorB;
  SampleWorksheet.Font:=Wnd.Worksheet.Font;
  ColorGrid.ForegroundIndex:=-1; // reset!
  if ShowModal<>mrOK then Exit;       {copy data to window (if not cancelled)}
  Wnd.XColumn:=XCol; Wnd.YColumn:=YCol;
  Wnd.DataFormat:=DataFormat;
  for I:=1 to Wnd.Worksheet.Header.Count do
  //Wnd.Worksheet.Header[I-1]:=LabelsStringGrid.Cells[2,I];
  Wnd.Labels[I]:=LabelsStringGrid.Cells[2,I];
  Wnd.Worksheet.AlignRight:=AlignRightCheckBox.Checked;
  Wnd.Worksheet.Font:=SampleWorksheet.Font;
  Wnd.Worksheet.Color:=SampleWorksheet.Color;
  Wnd.Worksheet.FixedColor:=SampleWorksheet.FixedColor;
  Wnd.Worksheet.BlockColorF:=SampleWorksheet.BlockColorF;
  Wnd.Worksheet.BlockColorB:=SampleWorksheet.BlockColorB;
end;

procedure TWorksheetPropsForm.FormCreate(Sender: TObject);
var I: integer; R: TGridRect;
begin                         // perform additional initialization of controls
  with LabelsStringGrid do for I:=0 to 2 do
  Cells[I,0]:=WordStr(strLabelsStringGrid,I+1);
  R.Left:=1; R.Right:=2; R.Top:=1; R.Bottom:=2; SampleWorksheet.Selection:=R;
end;

{COLUMNS}
procedure TWorksheetPropsForm.SetAsXButtonClick(Sender: TObject);
begin XCol:=LabelsStringGrid.Selection.Top; UpdateLabelsStringGrid; end;

procedure TWorksheetPropsForm.SetAsYButtonClick(Sender: TObject);
begin YCol:=LabelsStringGrid.Selection.Top; UpdateLabelsStringGrid; end;

procedure TWorksheetPropsForm.UpdateFormatSample;
begin
  with DataFormat[ColumnSpinEdit.Value] do
  SampleLabel.Caption:=FloatToStrF(Pi,Ftype,Width,Decimals);
end;

procedure TWorksheetPropsForm.UpdateLabelsStringGrid;
var I: integer;
begin                                       // copy values from X,YCol to grid
  with LabelsStringGrid do
  begin
    for I:=1 to RowCount-1 do Cells[1,I]:='';                // clear previous
    Cells[1,XCol]:='[X]'; if XCol=YCol then Cells[1,XCol]:='[X,Y]'  // set new
    else Cells[1,YCol]:='[Y]';
  end;
end;

procedure TWorksheetPropsForm.FormatChange(Sender: TObject);
begin
  with DataFormat[ColumnSpinEdit.Value] do
  begin Width:=WidthSpinEdit.Value; Decimals:=DigitsSpinEdit.Value; end;
  UpdateFormatSample;
end;

procedure TWorksheetPropsForm.FormatRadioGroupClick(Sender: TObject);
begin           // note: cannot use same handler for spinedits and RadioGroup!
  with DataFormat[ColumnSpinEdit.Value] do
  begin
    case FormatRadioGroup.ItemIndex of
      0: FType:=ffGeneral;
      1: FType:=ffFixed;
      2: FType:=ffExponent;
    end;
  end;
  UpdateFormatSample;
end;

procedure TWorksheetPropsForm.ColumnSpinEditChange(Sender: TObject);
begin
  WidthSpinEdit.Value:=DataFormat[ColumnSpinEdit.Value].Width;
  DigitsSpinEdit.Value:=DataFormat[ColumnSpinEdit.Value].Decimals;
  case DataFormat[ColumnSpinEdit.Value].FType of
    ffGeneral: FormatRadioGroup.ItemIndex:=0;
    ffFixed: FormatRadioGroup.ItemIndex:=1;
    ffExponent: FormatRadioGroup.ItemIndex:=2;
  end;
  ColumnLabel.Caption:=LabelsStringGrid.Cells[2, ColumnSpinEdit.Value];
  UpdateFormatSample;
end;

procedure TWorksheetPropsForm.AlignRightCheckBoxClick(Sender: TObject);
begin SampleWorksheet.AlignRight:=AlignRightCheckBox.Checked; end;

procedure TWorksheetPropsForm.LabelsStringGridExit(Sender: TObject);
begin ColumnSpinEdit.Value:=LabelsStringGrid.Row; end;

{VIEW}
procedure TWorksheetPropsForm.FontBitBtnClick(Sender: TObject);
begin
  FontDialog.Font:=SampleWorksheet.Font;
  if FontDialog.Execute then SampleWorksheet.Font:=FontDialog.Font;
end;

procedure TWorksheetPropsForm.ColorBitBtnClick(Sender: TObject);
begin
  case ItemRadioGroup.ItemIndex of
    0: ColorDialog.Color:=SampleWorksheet.Color;
    1: ColorDialog.Color:=SampleWorksheet.FixedColor;
    2: ColorDialog.Color:=SampleWorksheet.BlockColorB;
    3: ColorDialog.Color:=SampleWorksheet.BlockColorF;
  end;
  ColorGrid.ForegroundIndex:=-1; if ColorDialog.Execute then
  case ItemRadioGroup.ItemIndex of
    0: SampleWorksheet.Color:=ColorDialog.Color;
    1: SampleWorksheet.FixedColor:=ColorDialog.Color;
    2: SampleWorksheet.BlockColorB:=ColorDialog.Color;
    3: SampleWorksheet.BlockColorF:=ColorDialog.Color;
  end;
end;

procedure TWorksheetPropsForm.ColorGridChange(Sender: TObject);
begin
  if ColorGrid.ForegroundIndex>=0 then
  case ItemRadioGroup.ItemIndex of
    0: SampleWorksheet.Color:=ColorGrid.ForegroundColor;
    1: SampleWorksheet.FixedColor:=ColorGrid.ForegroundColor;
    2: SampleWorksheet.BlockColorB:=ColorGrid.ForegroundColor;
    3: SampleWorksheet.BlockColorF:=ColorGrid.ForegroundColor;
  end;
end;

end.
